#include "SaveConfigView.h"
#include <Beep.h>
#include <Application.h>
#include "HLM.h"
#include <Window.h>
#include <stdio.h>
#include "Defines.h"
#include <FilePanel.h>
#include <Roster.h>
#include "ErrorBox.h"
#include "BubbleHelper.h"

#define INFOBAR_MSG	'SCV0'
#define SCV_LANGUAGE_MSG	'SCV1'
#define SCV_ADD_LANGUAGE_MSG	'SCV2'
#define SCV_REMOVE_LANGUAGE_MSG	'SCV3'
#define BALLOONHELP_MSG	'SCV4'

extern HLM *H;
extern BubbleHelper	*HLP;


SaveConfigView::SaveConfigView() : ConfigView(){
	// CHECKBOXES
	streamedCB=new BCheckBox(BRect(15,5,230,20), "scv_streamedCB", H->L("TSTREAMING"), NULL);
	AddChild(streamedCB);
	savewhenquitCB=new BCheckBox(BRect(15,25,230,40), "scv_savewhenquitCB", H->L("TSAVEWHENQUIT"), NULL);
	AddChild(savewhenquitCB);
	autoupdateCB=new BCheckBox(BRect(15,45,230,60), "scv_autoupdateCB", H->L("TAUTOUPDATEINFOBOX"), NULL);
	AddChild(autoupdateCB);
	
	// STRINGVIEWS
	onexitdoSV=new BStringView(BRect(15, 60, 280, 80), "scv_onexitdoSV", H->L("TONEXITDO_SV"));
	AddChild(onexitdoSV);
	
	// RADIOBUTTONS
	autosaveRB=new BRadioButton(BRect(30,85,300,100), "scv_autosaveRB", H->L("TAUTOSAVE_RB"), NULL);
	AddChild(autosaveRB);
	askforsaveRB=new BRadioButton(BRect(30,105,300,120), "scv_askforsaveRB", H->L("TASKFORSAVE_RB"), NULL);
	AddChild(askforsaveRB);
	donothingRB=new BRadioButton(BRect(30,125,300,140), "scv_donothingRB", H->L("TDONOTHING_RB"), NULL);
	AddChild(donothingRB);

	// balloon help cb
	balloonhelpCB=new BCheckBox(BRect(15,150,230,165), "scv_balloonhelpCB", H->L("CB:Bubble Help"), new BMessage(BALLOONHELP_MSG), B_FOLLOW_LEFT|B_FOLLOW_TOP);
	AddChild(balloonhelpCB);
	 
	languagePUM=new EditPopUpMenu("");
	
	//extern HLM	*H;
	lastLanguage=-1;
	ReloadAllLanguages();
	languageMF=new BMenuField(BRect(15, 170, 280, 185), "scv_languageMF", H->L("TSELECTLANGUAGE"), languagePUM);
	AddChild(languageMF);
	languageMF->SetDivider(languageMF->StringWidth(languageMF->Label())+5);
	
	
	
	//infobarIBTI=new IBToggleItem();
}

void	SaveConfigView::ReloadAllLanguages() {
	int32	lastMarked=0;
	
	if (languagePUM->FindMarked())
		lastMarked=GetLanguage();

	
	if (lastLanguage!=-1) {
		delete H;
		H=new HLM();
		if (H->LanguageAt(lastLanguage+'HLMg'))
			H->LoadLanguage(lastLanguage+'HLMg');
		else
			H->LoadLanguage('HLMg');
	}
	
	for (int32 i=languagePUM->CountItems()-1; languagePUM->ItemAt(i); i--)
		delete languagePUM->RemoveItem(i);

	for (int32 i='HLMg'; i<'HLMg'+50; i++) {
		if (strlen(H->LanguageName(i))>0) {
			languagePUM->AddItem(new EditMenuItem(H->LanguageName(i), new BMessage(SCV_LANGUAGE_MSG)));
		}
	}
	((EditMenuItem *)languagePUM->ItemAt(0))->SetRemovable(false);
	languagePUM->AddSeparatorItem();
	languagePUM->AddItem(new BMenuItem(H->L("PUM:Add Language"), new BMessage(SCV_ADD_LANGUAGE_MSG)));
	languagePUM->SetTargetForItems(this);
	languagePUM->SetRemoveMessage(new BMessage(SCV_REMOVE_LANGUAGE_MSG));
	languagePUM->SetRemoveMessageTarget(this);
	if (lastMarked+1<=languagePUM->CountItems()) {
		SetLanguage(lastMarked);
	} else {
		SetLanguage(0);
		//languagePUM->ItemAt(0)->SetMarked(true);
	}
	if (languagePUM->FindMarked()==NULL)
		languagePUM->ItemAt(0)->SetMarked(true);	
}

void	SaveConfigView::Invalidate() {
	streamedCB->SetLabel(H->L("TSTREAMING"));
	savewhenquitCB->SetLabel(H->L("TSAVEWHENQUIT"));
}

SaveConfigView::~SaveConfigView() {
	//delete infobarIBTI;
}
	
bool	SaveConfigView::IsStreamed() {
	return streamedCB->Value()==B_CONTROL_ON;
}

void	SaveConfigView::SetStreamed(bool state) {
	streamedCB->SetValue(state ? B_CONTROL_ON : B_CONTROL_OFF);
}

bool	SaveConfigView::IsSavedWhenQuit() {
	return savewhenquitCB->Value()==B_CONTROL_ON;
}

void	SaveConfigView::SetSaveWhenQuit(bool state) {
	savewhenquitCB->SetValue(state ? B_CONTROL_ON : B_CONTROL_OFF);
}

exit_modes	SaveConfigView::GetExitMode() {
	if (autosaveRB->Value()==B_CONTROL_ON) return EM_SAVE_PROJECT;
	if (askforsaveRB->Value()==B_CONTROL_ON) return EM_ASK_USER;
	if (donothingRB->Value()==B_CONTROL_ON) return EM_DO_NOTHING;
	return EM_ASK_USER;
}

void	SaveConfigView::SetExitMode(exit_modes mode) {
	switch(mode) {
		case EM_SAVE_PROJECT: {
			autosaveRB->SetValue(B_CONTROL_ON);
			break;
		}
		
		case EM_ASK_USER: {
			askforsaveRB->SetValue(B_CONTROL_ON);
			break;
		}
		
		case EM_DO_NOTHING: {
			donothingRB->SetValue(B_CONTROL_ON);
			break;
		}
		
		default: {
			break;
		}
	}
}

bool	SaveConfigView::AutoUpdate() {
	return autoupdateCB->Value()==B_CONTROL_ON;
}

void	SaveConfigView::SetAutoUpdate(bool state) {
	autoupdateCB->SetValue(state ? B_CONTROL_ON : B_CONTROL_OFF);
}



int32	SaveConfigView::GetLanguage() {
	return languagePUM->IndexOf(languagePUM->FindMarked());
}

void	SaveConfigView::SetLanguage(int32 lang) {
	languagePUM->ItemAt(lang)->SetMarked(true);
	lastLanguage=lang;
}


void	SaveConfigView::SetBalloonHelp(bool state) {
	balloonhelpCB->SetValue(state?B_CONTROL_ON:B_CONTROL_OFF);
	HLP->EnableHelp(state);
}


// PRIVATE


bool	MoveResource(BResources *res, int32 fromID, int32 toID) {
	//BResources	*temp=new BResources();
	size_t		outsize;
	const void	*data;
	const char	*name;
	
	type_code	tc;
	int32		id;
	size_t		length;
	
	if ((data=res->LoadResource(B_MESSAGE_TYPE, fromID, &outsize))!=NULL) {
		res->GetResourceInfo(data, &tc, &id, &length, &name);
		res->AddResource(B_MESSAGE_TYPE, toID, data, outsize, name);
		res->RemoveResource(res->LoadResource(B_MESSAGE_TYPE, fromID, &outsize));
		return true;
	} else
		return false;
/*	if (data!=NULL) {
		if (temp->AddResources(B_MESSAGE_TYPE, 0, data, outsize, H->LanguageName(fromID))==B_OK) {
			// res was saved temporarily
			// now delete the fromID from *res
			if (res->RemoveResources(data)==B_OK) {
				// removed resources successfully...
				res->AddResources(B_MESSAGE_TYPE);
			}
		}
	}*/
}


void	SaveConfigView::ReorderLanguages(BResources *res) {

	const void 	*data;
	size_t		datalength;
	int32		holeID, nextID;
	
	// search for the first unused ID
	for (int32 i='HLMg'; ((data=res->LoadResource(B_MESSAGE_TYPE, i, &datalength))!=NULL) && (i<'HLMg'+1024);i++) {
		holeID=i;
	}
	
	// if the following ID is used, we are not at the last ID, but we found that a language is missing here...
	holeID++;
	nextID=holeID+1;
	while (MoveResource(res, nextID, holeID)) {
		nextID++;
		holeID++;
	}
}


void	SaveConfigView::MessageReceived(BMessage *msg) {
	switch(msg->what) {
		case BALLOONHELP_MSG: {
			HLP->EnableHelp(BalloonHelp());
			break;
		}
		case HELIOS_LANGUAGE_UPDATE_MSG: {
			streamedCB->SetLabel(H->L("TSTREAMING"));
			savewhenquitCB->SetLabel(H->L("TSAVEWHENQUIT"));
			break;
		}
		
		case B_REFS_RECEIVED: {
			entry_ref	ref;
			
			if (msg->FindRef("refs", 0, &ref)==B_OK) {
				BFile		*file=new BFile(&ref, B_READ_ONLY);
				BFile		*app=new BFile();
				app_info	ai;
				int32		nextindex;
				const void	*data;
				size_t		datalength;
				const char	*name;
				
				be_app->GetAppInfo(&ai);
				app->SetTo(&ai.ref, B_READ_WRITE);
				BResources	*appres=new BResources(app, false);
				BResources	*fileres=new BResources(file, false);
				
				
				// get the next free index position
				for (int32 i='HLMg'; H->LanguageAt(i); i++)
					nextindex=i+1;
				
				// find the resource of the new language file
				for (int32 i='HLMg'; ((data=fileres->LoadResource(B_MESSAGE_TYPE, i, &datalength))==NULL) && (i<'HLMg'+1024);i++);
				
				// add the found resource to the application's resources
				if (data!=NULL) {
					// get the name of the new language
					type_code	tc;
					int32		id;
					size_t		size;
					bool		add_resource=true;
					fileres->GetResourceInfo(data, &tc, &id, &size, &name);
				
					if (appres->HasResource(B_MESSAGE_TYPE, name)) {
						ErrorBox	*eb=new ErrorBox(E_ORANGE_COLOR, H->L("Error:Language Exists Already"), H->L("Error:Language Exists Already Errortext"), H->L("TADD"), H->L("TCANCEL"));
						Window()->Hide();
						if (eb->Go()==1)
							add_resource=false;
						Window()->Show();
					}
					if (add_resource)
						appres->AddResource(B_MESSAGE_TYPE, nextindex, data, datalength, name);
				}
				
				
				
				appres->Sync();
				
				delete appres;
				delete fileres;
				delete app;
				delete file;
				ReloadAllLanguages();
			}			
			break;
		}
		
		case SCV_REMOVE_LANGUAGE_MSG: {
			int32	index;
			app_info	ai;
			BFile		*app=new BFile();
			
			be_app->GetAppInfo(&ai);
			app->SetTo(&ai.ref, B_READ_WRITE);
			
			BResources	*appres=new BResources(app, false);
			
			msg->FindInt32("from", &index);
			if (appres->RemoveResource(B_MESSAGE_TYPE, 'HLMg'+index)==B_OK) {
				appres->Sync();
				ReorderLanguages(appres);
				ReloadAllLanguages();
				//languagePUM->ItemAt(0)->SetMarked(true);
			}
			
			delete appres;
			delete app;			
			break;
		}
		
		case SCV_ADD_LANGUAGE_MSG: {
			BFilePanel	*filepanel=new BFilePanel(B_OPEN_PANEL, new BMessenger(this), NULL, 0, false, NULL, NULL, true, true);
			filepanel->Show();
			break;
		}
		
		case B_CANCEL: {
			BFilePanel	*fp;
			SetLanguage(lastLanguage);
			if (msg->FindPointer("source", 0, (void **)(&fp))==B_OK) {
				if (fp)
					delete fp;
			}
		}
		
		case SCV_LANGUAGE_MSG: {
			lastLanguage=GetLanguage();
			//int32 langID;
			
			//langID=H->GetLIDByName((languagePUM->ItemAt(GetLanguage()))->Label());
			//if (H->LanguageAt(GetLanguage()+'HLMg'))
			//	H->LoadLanguage(GetLanguage()+'HLMg');
				//printf("LID=%ld\n", langID);

			//Looper()->PostMessage(HELIOS_LANGUAGE_UPDATE_MSG);
			
			break;
		}
		default: {
			ConfigView::MessageReceived(msg);
			break;
		}
	}
}

void	SaveConfigView::AllAttached() {
	languagePUM->SetTargetForItems(this);
}
