#include "Application1.h"
#include "Defines.h"
#include <MediaFormats.h>
#include <fs_attr.h>
#include "PipeCmd.h"
#include <MediaFile.h>
#include <MediaTrack.h>
#include <FilePanel.h>
#include <app/Roster.h>
#include <AppFileInfo.h>
#include "ColumnTypes.h"
#include "FolderRow.h"
#include "AudioRow.h"
#include "BitmapMenuItem.h"
#include <Volume.h>
#include <errno.h>
#include "AboutWindow.h"
#include "FileChooser.h"
#include <Query.h>
#include "ConfigFields.h"
#include <Beep.h>
#include "ErrorBox.h"
#include <support/Debug.h>
#include <support/ClassInfo.h>
#include "Globals.h"
#include <be/support/UTF8.h>
#include "MKISOFSCmdLine.h"
#include "CDRECORDCmdLine.h"
#include "CDText.h"
#include "FileDialog.h"
//#include "BubbleHelper.h"
#include "InfoFile.h"

sem_id quitandsave_sem;
extern HLM	*H;
//extern BubbleHelper	*HLP;


const char *GetToolVersion(const char *path) {
	thread_id	thread;
	int		in, out, err;
	const char	**argv;
	int		argc=0;
	const char	*threadname="Helios:GetToolVersion()";
	char		buffer[1000];
	size_t		bytesread;
	BString		version="";
	
	buffer[0]=0;
	argv=(const char **)malloc(sizeof(char *) * (5));
	argv[argc++]=strdup(path);
	argv[argc++]=strdup("--version");
	argv[argc]=NULL;
	
	thread=pipe_command(argc, argv, in, out, err);
	rename_thread(thread, threadname);
	resume_thread(thread);
	
	
	while ((bytesread=read(err,(void *)buffer, 1000))>0) {
		buffer[bytesread]=0;
		version.Append(buffer);
		//printf("%s: %s\n",path, buffer);
		snooze(10000);
	}
	while ((bytesread=read(out,(void *)buffer, 1000))>0) {
		buffer[bytesread]=0;
		version.Append(buffer);
		//printf("%s: %s\n",path, buffer);
		snooze(10000);
	}

	free(argv);
	close(in);
	close(out);
	close(err);
	
	
	char 	str[1024];
	int32	length=1024;
	int32	state=0;
	int32	srclen=version.Length();
	convert_to_utf8(B_ISO1_CONVERSION, version.String(), &srclen, str, &length, &state);
	
	str[length]=0;
	version=str;
	
	// --version option not supported?
	if (version.FindFirst("--version")!=B_ERROR) {
		version=path;
		version << ": " << H->L("STR:(could not get version info)");
	} else {
		version.RemoveAll("\n");
		version.RemoveAll("\t");
		// for case-insensitivity replace instead of remove...
		version.IReplaceAll("Copyright ", "");
		version.IReplaceAll("(c) ", B_UTF8_COPYRIGHT);
		version.IReplaceAll("(c)", B_UTF8_COPYRIGHT);
	}
	return strdup(version.String());
}



// copies file "source" to "destination"
// destination file will be overwritten if it exists already.
bool Application1::FileCopy(const char *source, const char *destination, size_t srcfilesize) {
	BFile	*src, *dst;
	ssize_t	numread;
	char	*buffer=new char[512*1024];
	char	info[20];
	int	buffersize=64*1024;
	const int sectorsize=2048;
	float	progressed=0, sum=0;
	bool	retval=true;
	
	src=new BFile(source, B_READ_ONLY);
	dst=new BFile(destination, B_READ_WRITE | B_CREATE_FILE);
	
	BStopWatch *watch=new BStopWatch("MBps",true);
	ShowStatus2(H->L("L:Reading CD..."), -1);
	SetStatus2Info("--- MB/s");

	if ((src->InitCheck()==B_OK) && (dst->InitCheck()==B_OK)) {
		dst->SetSize(0);
		
		numread=src->Read(buffer, buffersize);
		while ((dst->Write(buffer, numread)==numread) && (buffersize>=sectorsize)) {
			numread=src->Read(buffer, buffersize);
			if ((numread<=0) && (buffersize>sectorsize)) {
				buffersize/=2;
				printf("HELIOS: Reducing Buffer Size To %d Bytes.\n", buffersize);
			}
			if (buffersize<=sectorsize) buffersize=0;
			if (window5->interrupted) {
				retval=false;
				break;
			}

			progressed+=numread;
			if (progressed>64*1024*8) {
					sprintf(info,"%0.1f MB/s",((float)progressed)/((float)watch->ElapsedTime()*1.048576));
					SetStatus2Info(info);
					watch->Reset();
					sum+=progressed;
					SetPercentage(100*sum/srcfilesize);
					progressed=0.0F;
			}

		}
		dst->Sync();
		src->Unset();
		dst->Unset();
	} else retval=false;
	src->Unset();
	dst->Unset();
	delete src;
	delete dst;
	delete [] buffer;
	delete watch;
	return retval;
}


BString *RemoveTrailingSlash(BString &path) {
	if (path.ByteAt(path.Length()-1)=='/')
		path.RemoveLast("/");
	return &path;
}

void AppendTrailingSlash(BString *path) {
	if (path->ByteAt(path->Length()-1)!='/')
		path->Append("/");
}

const char *printArguments(char *title, const char **args) {
	BString	str="";
	int i=0;
	printf("HELIOS: %s\n", title);
	while (args[i]!=NULL) {
		printf("\targument(%d)=%s\n", i+1, args[i]);
		i++;
	}
	printf("\n");
	
	i=0;
	while (args[i]!=NULL) {
		str << args[i] << " ";
		i++;
	}
	
	return strdup(str.String());
}



bool Application1::FileExists(BEntry *entry) {
	return entry->Exists();
}

bool Application1::FileExists(BEntry entry) {
	return FileExists(&entry);
}

bool Application1::FileExists(BFile *file) {
	return (file->InitCheck()!=B_ENTRY_NOT_FOUND);
}

bool Application1::FileExists(char *filename) {
	BEntry *entry=new BEntry(filename, true);
	bool	exists=entry->Exists();
	entry->Unset();
	delete entry;
	return exists;	
}

bool Application1::FileExists(const char *filename) {
	return FileExists((char *)filename);
}

// check, whether there is a project loaded or not
bool Application1::IsProject(bool show_alert=true) {
		if (!FileExists(BEntry(base_folder.String(), true))) {
			be_app->PostMessage(NEWPROJECT_MIMSG);
//			if (show_alert) {
//				ErrorBox	*eb=new ErrorBox(E_YELLOW_COLOR, TERROR,TNOPROJECT,TOK);
//				eb->Go();
//			}
			return false;
		} else
			return true;
}


// creates a new folder named "folder_name"
// return value: "true" if successful, "false" if not
status_t CreateFolder(const char *folder_name) {
	BDirectory	*dir=new BDirectory();
	status_t	returnvalue;
		
	returnvalue=dir->CreateDirectory(folder_name, NULL);
	delete dir;
	return returnvalue;
}


// return-value: zero-length-string=no file found or selected,
//			nonzero-length-string=path to file
			
const char *Application1::CheckCommandlineTool(const char *fullpath, const char *name) {
	BString 	string, path;
	BFile 		cdrecord;
	BQuery 	*query;
	BString		query_string="";
	
	
	if (cdrecord.SetTo(fullpath,B_READ_ONLY)!=B_OK) {
		// cdrecord not found
		cdrecord.Unset();
		string.SetTo(name);
		string.Append(" ");
		string.Append(H->L("TXNOTFOUND"));
		BAlert *alert=new BAlert("alert",string.String(), H->L("TCANCEL"), H->L("TSEARCH"));
		if (alert->Go()==1) {
			// search for requested file and show the result to the user.
			// the user should now be able to choose the file out of a list.
			FileChooser	*chooser=new FileChooser();
			query=new BQuery();
			query->SetVolume(new BVolume(dev_for_path("/boot/")));
			query_string.Append("name = ");
			query_string.Append(name);
			query->SetPredicate(query_string.String());
			query->Fetch();
			BEntry	entry;
			while (query->GetNextEntry(&entry , true)==B_OK) 
				if (entry.IsFile()) chooser->AddFile(&entry);
			entry.Unset();
			delete query;
			string=strdup(chooser->Go());
			return strdup(string.String());
		}
		return strdup("");
	} else {
		cdrecord.Unset();
		return strdup(fullpath);
	}
}


void Application1::SetHelpBubbles() {
	extern BubbleHelper *HLP;

	HLP->SetHelp(window1->view1->FindView("bar_newproject"), H->L("TIP:B:New Project"));
	HLP->SetHelp(window1->view1->FindView("bar_openproject"), H->L("TIP:B:Open Project"));
	HLP->SetHelp(window1->view1->FindView("bar_saveproject"), H->L("TIP:B:Save Project"));
	HLP->SetHelp(window1->view1->FindView("bar_preferences"), H->L("TIP:B:Preferences"));
	HLP->SetHelp(window1->view1->FindView("bar_burnnow"), H->L("TIP:B:Burn Disc"));
	HLP->SetHelp(window1->view1->FindView("bar_newfolder"), H->L("TIP:B:New Folder"));
	HLP->SetHelp(window1->view1->FindView("bar_deleteitem"), H->L("TIP:B:Delete Item"));
	HLP->SetHelp(window1->view1->FindView("bar_path"), H->L("TIP:TC:Current Virtual Path"));
	HLP->SetHelp(window1->view1->FindView("bar_parent"), H->L("TIP:B:Parent Folder"));
	HLP->SetHelp(window1->view1->FindView("bar_update"), H->L("TIP:B:Refresh"));
//	HLP->EnableHelp();
}


Application1::Application1():BApplication("application/x-HeliosBurner.application") {

	ASSERT(streambuffer_sem=create_sem(1, "streambuffer_sem"));
	quitandsave_sem=create_sem(0, "quitandsave_sem");
	fileadder_sem=create_sem(1, "fileadder_sem");
	fileremover_sem=create_sem(1, "fileremover_sem");
	filename_col=0;
	save_and_quit=false;
	project_has_changed=false;
	is_all_showing=false;
	CM=new ConfigManager("/boot/home/config/settings/Helios.settings", true);
	
	extern BubbleHelper *HLP;
	HLP=new BubbleHelper();

	// Load language manager
	H=new HLM();
	if (CM->HasData(LANGUAGE_ID)) {
		if (H->LanguageAt(CM->GetInt32(LANGUAGE_ID)+'HLMg')) {
			H->LoadLanguage(CM->GetInt32(LANGUAGE_ID)+'HLMg');
		} else
			H->LoadLanguage('HLMg');
	} else
		H->LoadLanguage('HLMg');

	window1 = new Window1();
	window2 = new Window2();
	statusSW=new StatusWindow();
	window5 = new Window5();
	configW=new ConfigWindow(H->L("TSETTINGSWINDOWTITLE"));
	
	
	audioW=NULL;
	configW->AddTopic(H->L("OLV_GENERALOPTIONS"));
	configW->AddConfigView(H->L("OLV_RECORDING"), (heliosCV=new HeliosConfigView()));
	configW->AddConfigView(H->L("OLV_DEVICES"), (deviceCV=new DeviceConfigView()));
	configW->AddConfigView(H->L("OLV_FILESANDFOLDERS"), (pathCV=new PathConfigView()));
	configW->AddConfigView(H->L("OLV_IMAGES"), (imageCV=new ImageConfigView()));
	configW->AddConfigView(H->L("OLV_FILESYSTEM"), (fspanelCV=new FSPanel()));
	configW->AddTopic(H->L("OLV_PROJECTOPTIONS"));
	configW->AddConfigView(H->L("OLV_NEWPROJECT"), (standardCV=new StandardConfigView()));
	configW->AddTopic(H->L("OLV_APPLICATIONOPTIONS"));
	configW->AddConfigView(H->L("OLV_TIMEOUTS"), (applicationCV=new ApplicationConfigView()));
	configW->AddConfigView(H->L("OLV_SOUND"), (soundCV=new SoundConfigView()));
	configW->AddConfigView(H->L("OLV_GENERAL"), (saveCV=new SaveConfigView()));
	configW->AddConfigView(H->L("OLV_COLORS"), (colorCV=new ColorConfigView()));
	configW->AddConfigView(H->L("OLV_VERSION"), (versionCV=new VersionConfigView()));
//	settingswindow=new SettingsWindow(BRect(50,50,400,391),TSETTINGSWINDOWTITLE);
	cdrecord_output=new ProtocolWindow(BRect(0,0,50,50), "cdrecord", B_DOCUMENT_WINDOW, -1);
	mkisofs_output=new ProtocolWindow(BRect(0,0,50,50), "mkisofs", B_DOCUMENT_WINDOW, -1);
//	if (!GetCDRDevices()) {
//		BAlert *alert=new BAlert("alert",TNODEVICESFOUND,TOK);
//		alert->Go();
//		be_app->Quit();
//	}
	//OLDSETTINGS ReadSettings();

	ReadSettings();
	SetHelpBubbles();
	// check for command line tools
	BString		string;
	bool		exit=false;
	// mkisofs
//	string=CheckCommandlineTool(settings._path_to_mkisofs, "mkisofs");
//	if (string.Length()!=0) sprintf(settings._path_to_mkisofs,"%s",string.String());
//	else Quit();
	string=CheckCommandlineTool(CM->GetString(PATH_TO_MKISOFS)->String(), "mkisofs");
	if (string.Length()!=0) CM->SetString(PATH_TO_MKISOFS, &string);
	else exit=true;
	// cdrecord
	string=CheckCommandlineTool(CM->GetString(PATH_TO_CDRECORD)->String(), "cdrecord");
	if (string.Length()!=0) CM->SetString(PATH_TO_CDRECORD, &string);
	else exit=true;
	// readcd
	string=CheckCommandlineTool(CM->GetString(PATH_TO_READCD)->String(), "readcd");
	if (string.Length()!=0) CM->SetString(PATH_TO_READCD, &string);
	else exit=true;
	// cdda2wav
	string=CheckCommandlineTool(CM->GetString(PATH_TO_CDDA2WAV)->String(), "cdda2wav");
	if (string.Length()!=0) CM->SetString(PATH_TO_CDDA2WAV, &string);
	else exit=true;
	// mkbfs
	string=CheckCommandlineTool(CM->GetString(PATH_TO_MKBFS)->String(), "mkbfs");
	if (string.Length()!=0) CM->SetString(PATH_TO_MKBFS, &string);
	else exit=true;


	if (!exit)  {
		ShowRecentProjects();
		if ((CM->GetString(PATH_TO_LAST_PROJECT)->Length()>0) && (saveCV->IsSavedWhenQuit())) {
			BEntry	*entry=new BEntry(CM->GetString(PATH_TO_LAST_PROJECT)->String(), true);
			if (entry->Exists()) {
				LoadProject(entry);
				ShowFolder();
			} else
				PostMessage(NEWPROJECT_MIMSG);
			delete entry;
		} else {
			PostMessage(NEWPROJECT_MIMSG);
	//		NewProject();
	//		currentproject="";
	//		ShowFolder();
	//		if (window1->Lock()) {
	//			window1->view1->SetCDType(currentproj->_cdtype);
	//			window1->view1->listview1->SetListMode(currentproj->_cdtype);
	//			window1->Unlock();
	//		}
	//		has_project_changed=false;
	//		Update_DATA_AUDIO();
		}
	}

	if (window1->Lock()) {
		window1->view1->UpdatePanels();
		window1->Unlock();
	}
	//	window4->Lock();
	//	if (settingswindow->readerPUM->CountItems()<2) 
	//		settingswindow->twodrivecopyCB->SetEnabled(false); else
		//		settingswindow->twodrivecopyCB->SetEnabled(true);
	//	window4->Unlock();

	//OLDSETTINGS SaveSettings();
	
	
	// starts the BLooper() of configW
	if (configW->Lock()) {
		float	left=configW->Frame().left;
		float	top=configW->Frame().top;
		configW->MoveTo(-10000, -10000);
		configW->Show();
		for (int i=0; configW->SelectView(i); i++);
		configW->Hide();
		configW->MoveTo(left, top);
		configW->Unlock();
	}
	
	// Add Tooltips...
//	window1TT=new ToolTip(window1, "window1TT");
//	window1TT->SetText(window1->view1->iconBAR->ChildViewAt("bar_update"), "Update display");
//	window1TT->SetText(window1->view1->mainBAR->ChildViewAt("bar_burnnow"), "Burn CD/DVD");
//	HLP=new BubbleHelper();
//	HLP->SetHelp(window1->view1->iconBAR->ChildViewAt("bar_update"), "Update display");
//	HLP->EnableHelp(true);

	FileDialog::fmainWindow=window1;
	window1->Show();
	just_quit=exit;
	if (exit)
		PostMessage(B_QUIT_REQUESTED);

}


Application1::~Application1(void) {
	
	delete H;
	extern BubbleHelper *HLP;
	delete HLP;
	
	delete_sem(streambuffer_sem);
	delete_sem(quitandsave_sem);
	delete_sem(fileadder_sem);
	delete_sem(fileremover_sem);
}

BBitmap *GetIcon(const char *path) {

	BBitmap *icon=new BBitmap(BRect(0.0,0.0,15.0,15.0), B_CMAP8);
	BEntry	*entry=new BEntry(path, true);
	entry_ref	r;
	// resolve link!!!
	entry->GetRef(&r);
	delete entry;
		
	BNodeInfo::GetTrackerIcon(&r, icon, B_MINI_ICON);
	return icon;
}

BBitmap *GetIcon(entry_ref *ref) {

	BBitmap *icon=new BBitmap(BRect(0.0,0.0,15.0,15.0), B_CMAP8);
	BEntry	*entry=new BEntry(ref, true);
	entry_ref	r;
	// resolve link!!!
	entry->GetRef(&r);
	delete entry;
		
	BNodeInfo::GetTrackerIcon(&r, icon, B_MINI_ICON);
	return icon;
}


void	Application1::ReadyToRun(void) {
	BString		str;
	app_info	ai;
	//printf("cdrecord version: \"%s\"\n", GetCDRecordVersion());
	if (configW->Lock()) {
		versionCV->SetCDRecordVersion(GetToolVersion(CM->GetString(PATH_TO_CDRECORD)->String()), GetIcon(CM->GetString(PATH_TO_CDRECORD)->String()));
		versionCV->SetReadCDVersion(GetToolVersion(CM->GetString(PATH_TO_READCD)->String()), GetIcon(CM->GetString(PATH_TO_READCD)->String()));
		versionCV->SetMkISOfsVersion(GetToolVersion(CM->GetString(PATH_TO_MKISOFS)->String()), GetIcon(CM->GetString(PATH_TO_MKISOFS)->String()));
		versionCV->SetCDDA2WAVVersion(GetToolVersion(CM->GetString(PATH_TO_CDDA2WAV)->String()), GetIcon(CM->GetString(PATH_TO_CDDA2WAV)->String()));
		//printf("%s\n", CM->GetString(PATH_TO_CDDA2WAV)->String());
		str=H->L("ABOUT_APPLICATIONNAME");
		str << " " << H->L("ABOUT_VERSIONNUMBER");
		GetAppInfo(&ai);
		versionCV->SetHeliosVersion(str.String(), GetIcon(&ai.ref));
		configW->Unlock();
	}
	
	// create project folder if it does not exist
	BString	p=pathCV->GetProjectPath();
		
	AppendTrailingSlash(&p);

	p.Append(STANDARDPROJECTPATH);

	BDirectory HeliosDir;
		
	//bf=*RemoveTrailingSlash(bf);
	if (HeliosDir.SetTo(p.String())!=B_OK)
		//HeliosDir.CreateDirectory(bf.String(),&HeliosDir);
		create_directory(p.String(), 0777);
	
}


void Application1::CheckIfProjectChanged(void) {
};




int32 QuitAndSaveThread(void *data) {
	acquire_sem(quitandsave_sem);
	((Application1 *)data)->PostMessage(B_QUIT_REQUESTED);
	return 0;
}





bool
Application1::QuitRequested(void) {

	if (applicationCV->IsAutoUpdating()) {
		if (window1->Lock()) {
			window1->view1->box2->ExitUpdateThread();
			window1->Unlock();
		}
	}
	//OLDSETTINGS SaveSettings();
		SaveSettings();
		CheckIfProjectChanged();
		while ((save_and_quit) && (project_has_changed)) {
			snooze(50000);
	 	};
		if (!just_quit)
		switch(saveCV->GetExitMode()) {
			case EM_SAVE_PROJECT: {
				if ((project_has_changed) && (IsProject(false))) {
					resume_thread(spawn_thread(QuitAndSaveThread, "QuitAndSaveThread", 1, this));
					be_app->PostMessage(SAVEPROJECT_MIMSG);
					return false;
				}
				break;
			}
			
			case EM_ASK_USER: {
				if ((project_has_changed) && (IsProject(false))) {
					ErrorBox	*eb=new ErrorBox(E_BLUE_COLOR, "", H->L("TASKFORSAVINGCHANGES"), H->L("TYES"), H->L("TNO"), H->L("TCANCEL"));
					eb->SetShortcut(2, B_ESCAPE);
					switch (eb->Go()) {
						case 0: {
							resume_thread(spawn_thread(QuitAndSaveThread, "QuitAndSaveThread", 1, this));
							be_app->PostMessage(SAVEPROJECT_MIMSG);
							return false;
						}
						
						case 1: {
							break;
						}
						
						case 2: {
							return false;
						}
					}
				}
			}
			
			case EM_DO_NOTHING: {
				break;
			}
		}

	delete CM;
	
	BEntry *entry=new BEntry(pathCV->GetDefaultImageName(), true);
	
	if (entry->InitCheck()==B_OK) {
		entry->Remove();
		entry->Unset();
	}
	
	delete entry;
	
	if (configW->Lock()) configW->Quit();
	if (cdrecord_output->Lock()) cdrecord_output->Quit();
	if (mkisofs_output->Lock()) mkisofs_output->Quit();
	if (window2->Lock()) window2->Quit();
	if (window1->Lock()) window1->Quit();


//	int32 i = 0;
//	BWindow* pWin;
//	for (pWin = WindowAt(i++); pWin && pWin->Lock(); pWin = WindowAt(i++)) {
//		// don't take no for an answer
//		//pWin->Quit();
//	}
//
//	while (CountWindows()>0) {WindowAt(0)->Lock();WindowAt(0)->Quit();}
	//BApplication::QuitRequested();
	return true;
}


void Application1::SetWorking(bool state) {
	static int count=0;
	
	if (state)
		count++;
	else
		count--;
	
	if ((count==1) || (count==0))
		if (window1->Lock()) {
			((BButton *)window1->view1->iconBAR->ChildViewAt("bar_update"))->SetEnabled(!state);
			((BButton *)window1->view1->mainBAR->ChildViewAt("bar_burnnow"))->SetEnabled(!state);
			window1->Unlock();
		}
}


int32 Application1::addFiles(void *data) {
	
	Application1	*app=(Application1 *)be_app;
	
	BMessage	*msg=new BMessage(*((BMessage *)data));
	delete (BMessage *)data;
	acquire_sem(app->fileadder_sem);
	
	app->SetWorking();
	
	
///////////////////////////////////
	uint32 		type;
	int32 		count;
	off_t		filesize=0;
	BEntry		*entry=new BEntry();
	entry_ref	ref;
	BPath		path;
	char		mp3name[250];
	bool		goadding=true;
	BString		current_folder=app->current_folder;
	int32		cdtype=app->window1->view1->CDType();
		
	app->filecounter=0;
	
	msg->GetInfo("refs", &type, &count);
	if (type != B_REF_TYPE) {
		delete entry;
		app->SetWorking(false);
		release_sem(app->fileadder_sem);
		return 0;
	}

	if (count==1) {
		if (msg->FindRef("refs", &ref)==B_OK) {
			BNode	*node=new BNode(&ref);
			BNodeInfo 	*info=new BNodeInfo(node);
			char		type[B_FILE_NAME_LENGTH];
				
			info->GetType(type);
			delete info;
			node->Unset();
			delete node;
				
			// if a helios project has been dropped into the window
			// ask the user whether he want to add the file to the project
			// or open the dropped project.
			if (strcmp(type, HELIOSPROJECTMIMETYPE)==0) {
				ErrorBox	*eb=new ErrorBox(E_BLUE_COLOR, H->L("TPROJECTFILEDROPPED"), H->L("TPROJECTFILEDROPPED_TEXT"), H->L("TLOAD"), H->L("TADD"));
				switch(eb->Go()) {
					case 0: {	// LOAD PROJECT FILE
						BEntry	*entry=new BEntry(&ref);
						app->LoadProject(entry);
						delete entry;
						goadding=false;
						break;
					}
					
					case 1: {	// ADD TO PROJECT
						goadding=true;
						break;
					}
					
					default: {
						goadding=true;
						break;
					}
				}
			}
		}
	}
		
	if (goadding) {
		for (int32 i = --count; i >= 0; i--) {
			if (msg->FindRef("refs", i, &ref) != B_OK) {
				if (app->window1->Alert("Error at FindRef()\n\nProbably you didn't drop a file.")) {
					delete entry;
					app->SetWorking(false);
					release_sem(app->fileadder_sem);
					return 0;
				}
			} else
				entry->SetTo(&ref,true);

	   		if (entry->GetPath(&path)==B_OK) {
	     		           	sprintf(mp3name,"%s",path.Leaf());
				if (entry->GetSize(&filesize)==B_OK) {
			 		app->cdir=current_folder;
			 		//printf("current_folder=\"%s\"\n", current_folder.String());
			 		project_has_changed=true;
			 		//printf("path.Path()=\"%s\"\npath.Leaf()=\"%s\"\n",path.Path(), path.Leaf());
		 			app->CreateEntry((char *)path.Path(), (char *)path.Leaf());
					if ((cdtype==AUDIOCD_INDEX) || (cdtype==CDEXTRA_INDEX)) {
						float size;			
			
						//MMediaInfo *info=new MMediaInfo(entry);
						TrackLength *info=new TrackLength(new BEntry(path.Path(), true));
						size=info->GetBytes();
						if (size>0) {
							if (app->window1->Lock()) {
								AudioRow	*ar;
								app->window1->view1->listview1->AddRow((ar=new AudioRow(app->window1->view1->listview1->iCountItems()+1, path.Leaf(), 0, size, size/176400)));
								ar->SetStartFrame((int64)0);
								ar->SetEndFrame((int64)(75.0F*(float)size/176400.0F));
								ar->SetLength((int64)(75.0F*(float)size/176400.0F));
								if (app->window1->view1->listview1->iCountItems()==1)
									ar->SetPregap(150);
								app->window1->Unlock();
							}
						}
						delete info;
					}
    				}
	  		}
	  	}	// for
		path.Unset();
		entry->Unset();
	}	// if (goadding)

	if (app->window1->Lock()) {
		char	text[100];
		
		sprintf(text, "%s (%ld)", H->L("L:Adding files..."), app->filecounter);
		app->window1->view1->statusBAR->SetText(text, 4000000);
		app->window1->Unlock();
	}

	if (app->saveCV->AutoUpdate())
		be_app->PostMessage(UPDATEINFO_MSG);
	else
		app->ShowFolder();
	delete entry;
///////////////////////////////////

	
	delete msg;
	app->SetWorking(false);
	release_sem(app->fileadder_sem);
	return 0;
}





int32 Application1::removeFiles(void *data) {
	Application1	*app=(Application1 *)be_app;
	
	BMessage	*msg=new BMessage(*((BMessage *)data));
	delete (BMessage *)data;
	acquire_sem(app->fileremover_sem);
	
	if (app->window1->Lock()) {
		app->window1->view1->listview1->SetEditMode(true);
		app->window1->Unlock();
	}
	
	app->SetWorking();
	
	
	
	BEntry *file=new BEntry();
	BString str="", filename="";
	int32	cdtype=app->window1->view1->CDType();
	BString	current_folder=app->current_folder;
	BString base_folder=app->base_folder;
	
	BRow	*row;
	BList	list;
	
	if (app->window1->Lock()) {
		while ((row=app->window1->view1->listview1->CurrentSelection())) {
			if ((cdtype==AUDIOCD_INDEX) || (cdtype==CDEXTRA_INDEX)) filename=((AudioRow *)(row))->GetFilename();
			if (cdtype==DATACD_INDEX) filename=((FolderRow *)(row))->GetFilename();
			if (cdtype==BOOTABLECD_INDEX) filename=((FolderRow *)(row))->GetFilename();
			if (cdtype==DVDVIDEO_INDEX) filename=((FolderRow *)(row))->GetFilename();
			if (filename.Compare("..")!=0) {	     		
				str.SetTo(current_folder.String()).Append(filename.String());
				list.AddItem(new BString(filename));

				app->window1->view1->listview1->BColumnListView::RemoveRow(row);
				delete row;

				if ((cdtype==AUDIOCD_INDEX) || (cdtype==CDEXTRA_INDEX)) {
					for (int j=0; j<app->window1->view1->listview1->iCountItems();j++) {
						if (str.Compare(((AudioRow *)(app->window1->view1->listview1->RowAt(j)))->GetFilename())==0) {
					    		app->window1->view1->listview1->RemoveRow(j);
							delete app->window1->view1->listview1->RowAt(j);
							j=-1;
				    		}
					}
				}
			} else {
				app->window1->view1->listview1->Deselect(row);
				((BButton *)app->window1->view1->iconBAR->ChildViewAt("bar_deleteitem"))->SetEnabled(app->window1->view1->listview1->CurrentSelection()!=NULL);
			}
		}
		app->window1->Unlock();
	}
	
	app->filecounter=0;

	for (int32 k=list.CountItems()-1; k>=0; k--) {
		str.SetTo(current_folder.String()).Append(((BString *)list.ItemAt(k))->String());
	
		if (file->SetTo(str.String(),false)==B_OK) {
			if (file->IsDirectory()) {
				app->DeleteAndCount(BDirectory(file));
			}
				
			if (cdtype==BOOTABLECD_INDEX) {
				BString cdpathtoimage=str;
					
				cdpathtoimage.RemoveFirst(base_folder);
				cdpathtoimage.Prepend("/");
				// going to remove the floppy boot image file...
				if (cdpathtoimage.Compare(app->window1->view1->bootablepanel->GetImageName())==0) {
					ErrorBox	*eb=new ErrorBox(E_ORANGE_COLOR, H->L("Error:Delete Floppy Image?"), H->L("Error:Delete Floppy Image?Error text"), H->L("TYES"), H->L("TNO"));
					if (eb->Go()==1) {
						file->Unset();
						if (app->window1->Lock()) {
							app->window1->view1->listview1->Deselect(app->window1->view1->listview1->CurrentSelection());
							app->window1->Unlock();
						}
						continue;
					} else
						if (app->window1->Lock()) {
							app->window1->view1->bootablepanel->SetImageName("");
							app->window1->Unlock();
						}
				}
			}
			if (file->InitCheck()==B_OK)
				file->Remove();
					
			if (app->filecounter % 20==0) {
				if (app->window1->Lock()) {
					char	text[100];
					
					sprintf(text, "%s (%ld)", H->L("L:Removing files..."), app->filecounter);
					app->window1->view1->statusBAR->SetText(text);
					app->window1->Unlock();
				}
			}
			app->filecounter++;
				
			file->Unset();
			// there has been something changed
			project_has_changed=true;
		}
		str.SetTo(filename.String());
		delete (BString *)(list.RemoveItem(k));
	}
	
	
	
	if (app->window1->Lock()) {
		char	text[100];
		
		sprintf(text, "%s (%ld)", H->L("L:Removing files..."), app->filecounter);
		app->window1->view1->statusBAR->SetText(text, 4000000);
		app->window1->Unlock();
	}
	delete file;
	app->UpdateTrackNumbers();
	app->RefreshStatusView();

	if (app->saveCV->AutoUpdate())
		be_app->PostMessage(UPDATEINFO_MSG);

	
	delete msg;
	app->SetWorking(false);
	if (app->window1->Lock()) {
		app->window1->view1->listview1->SetEditMode(false);
		app->window1->Unlock();
	}
	release_sem(app->fileremover_sem);
	return 0;
}




void
Application1::MessageReceived(BMessage *message) {
	
	if (applicationCV->IsAutoUpdating())
		window1->view1->box2->StopUpdating();
	
	switch (message->what)
	{
	
	
		case READAUDIOCD_MIMSG: {
			status_t error=B_OK;
			
			error=ReadAudioCD();
			
			if (error<0) {
				ErrorBox	*eb=new ErrorBox(E_RED_COLOR, errtype.String(), errstring.String(), H->L("TOK"));
				PlayErrorSound();
				eb->Go();
			}
			break;
		}
	

		case B_SILENT_RELAUNCH: {
			if (window1->Lock()) {
				window1->view1->statusBAR->SetText(H->L("L:Launch once"), 4000000);
				window1->Unlock();
			}
			break;
		}
		
		case B_SIMPLE_DATA: {
			resume_thread(spawn_thread(addFiles, "File Adder", 10, new BMessage(*message)));
			break;
		}
		
		case 'mm01': // update display
		{
		window1->view1->box2->Refresh();
		beep();

		break;
		}
		
		case PREFERENCES_MIMSG:
		{
		configW->Lock();
		if (configW->IsHidden())
			configW->Show();
		else
			configW->Activate();
		configW->Unlock();
		break;
		}
		
		case FINISHED_EDITING_MSG: {
		void *pointer;
		BEntry file;
		//off_t filesize;
		BString str="";
		
		message->FindPointer("row", &pointer);
		
		FolderRow *selected=(FolderRow *)(pointer);
		
		if (selected==NULL) return;
		window1->Lock();
		if (BString(selected->GetFilename()).Compare("..")!=0)
		{	     		
			     str="";
			     str << current_folder.String() << selected->GetFilename();
			     //printf("WANNA REMOVE %s\n",str.String());
			     if (file.SetTo(str.String(),false)==B_OK)
			    	 {
			    	 str="";
			    	 str << current_folder.String() << ((BFileField *)selected->GetField(FILENAME_FIELD))->String();
			    	 file.Rename(str.String(),false);
			    	 selected->SetFilename(((BFileField *)selected->GetField(FILENAME_FIELD))->String());
				project_has_changed=true;
	//		    	 window1->view1->listview1->InvalidateItem(selected);
			    	 }
		}
		file.Unset();
		//window1->view1->listview1->ItemAt(selected)->DrawItem(window1->view1, window1->view1->listview1->ItemFrame(selected),true);
		Update_path();
		window1->Unlock();
		break;
		}
		
		case DELETE_FILE_MSG: // remove selected items
		{
			resume_thread(spawn_thread(removeFiles, "File Remover", 10, new BMessage(*message)));
			break;
		BEntry *file=new BEntry();
		//off_t filesize;
		BString str="", filename="";
		filecounter=0;
//		//int32 i;
//		if (statusSW->Lock()) {
//			statusSW->SetFileCounter(0);
//			statusSW->Unlock();
//		}
//		watch=new BStopWatch("fileadder", true);
//		addershows=false;
		while (window1->view1->listview1->CurrentSelection())
//		for (int32 i=0; i<window1->view1->listview1->iCountItems();i++)
 			 {
//			if ((!addershows) && (watch))
////				if (watch->ElapsedTime()>1200000)
//						{
//						ShowStatus(H->L("TREMOVINGFILES"));
//						addershows=true;
//						}
//			  if (window1->view1->listview1->IsSelected(i)) 
			     {
			 //    i=window1->view1->listview1->IndexOf(window1->view1->listview1->CurrentSelection());
			    // BStringItem *si=(BStringItem *)window1->view1->listview1->ItemAt(i);
//			    if ((window1->view1->CDType()==AUDIOCD_INDEX) || (window1->view1->CDType()==CDEXTRA_INDEX)) filename=((AudioRow *)(window1->view1->listview1->RowAt(i)))->GetFilename();
//			    if (window1->view1->CDType()==DATACD_INDEX) filename=((FolderRow *)(window1->view1->listview1->RowAt(i)))->GetFilename();
//			    if (window1->view1->CDType()==BOOTABLECD_INDEX) filename=((FolderRow *)(window1->view1->listview1->RowAt(i)))->GetFilename();
			    if ((window1->view1->CDType()==AUDIOCD_INDEX) || (window1->view1->CDType()==CDEXTRA_INDEX)) filename=((AudioRow *)(window1->view1->listview1->CurrentSelection()))->GetFilename();
			    if (window1->view1->CDType()==DATACD_INDEX) filename=((FolderRow *)(window1->view1->listview1->CurrentSelection()))->GetFilename();
			    if (window1->view1->CDType()==BOOTABLECD_INDEX) filename=((FolderRow *)(window1->view1->listview1->CurrentSelection()))->GetFilename();
			    if (window1->view1->CDType()==DVDVIDEO_INDEX) filename=((FolderRow *)(window1->view1->listview1->CurrentSelection()))->GetFilename();
			     
			if (filename.Compare("..")!=0) 
			//if (!((current_folder.Compare(base_folder)==0) && (i==0)))
				{	     		
				     //str="";
				     str.SetTo(current_folder.String()).Append(filename.String());
				     //printf("REMOVE %s\n",str.String());
	
				     if (file->SetTo(str.String(),false)==B_OK)
				    	 {
					if (file->IsDirectory()) {
//						str.SetTo("rm -r \"").Append(current_folder).Append(filename.String()).Append("\"");
//						system(str.String());
						DeleteAndCount(BDirectory(file));
					}
					
					if (window1->view1->CDType()==BOOTABLECD_INDEX) {
						BString cdpathtoimage=str;
						
						cdpathtoimage.RemoveFirst(base_folder);
						cdpathtoimage.Prepend("/");
						// going to remove the floppy boot image file...
						if (cdpathtoimage.Compare(window1->view1->bootablepanel->GetImageName())==0) {
							ErrorBox	*eb=new ErrorBox(E_ORANGE_COLOR, H->L("Error:Delete Floppy Image?"), H->L("Error:Delete Floppy Image?Error text"), H->L("TYES"), H->L("TNO"));
							if (eb->Go()==1) {
								file->Unset();
								if (window1->Lock()) {
									window1->view1->listview1->Deselect(window1->view1->listview1->CurrentSelection());
									window1->Unlock();
								}
								continue;
							} else
								if (window1->Lock()) {
									window1->view1->bootablepanel->SetImageName("");
									window1->Unlock();
								}

						}
					}
					if (file->InitCheck()==B_OK)
						file->Remove();

					
					if (filecounter % 20==0) {
						if (window1->Lock()) {
							char	text[100];
							
							sprintf(text, "%s (%ld)", H->L("L:Removing files..."), filecounter);
							window1->view1->statusBAR->SetText(text);
							window1->Unlock();
						}
					}
					filecounter++;

					/*if (statusSW->Lock()) {
						statusSW->SetFileCounter(statusSW->GetFiles()+1);
						statusSW->Unlock();
					}*/
					
					file->Unset();
					// there has been something changed
					project_has_changed=true;
				    	 }
				    str.SetTo(filename.String());
				    if (window1->Lock()) {
				    	  BRow	*row;
					    window1->view1->listview1->BColumnListView::RemoveRow((row=window1->view1->listview1->CurrentSelection()));
					    delete row;
					    window1->Unlock();
				   }
				    //delete window1->view1->listview1->RowAt(i);

				    // if audio mode, check for item with the same name, because these are
				    // splitted parts and have to be removed, too!
				    window1->Lock();
				    if ((window1->view1->CDType()==AUDIOCD_INDEX) || (window1->view1->CDType()==CDEXTRA_INDEX)) {
				    	for (int j=0; j<window1->view1->listview1->iCountItems();j++) {
				    		if (str.Compare(((AudioRow *)(window1->view1->listview1->RowAt(j)))->GetFilename())==0) {
				    			window1->view1->listview1->RemoveRow(j);
							//delete window1->view1->listview1->RowAt(j);
							j=-1;
				    		}
				    	}
				    }
				    window1->Unlock();
//				     i=-1;
				}
   			    }
  			}
		if (window1->Lock()) {
			char	text[100];
		
			sprintf(text, "%s (%ld)", H->L("L:Removing files..."), filecounter);
			window1->view1->statusBAR->SetText(text, 4000000);
			window1->Unlock();
		}
//		delete watch;
		delete file;
//		if (addershows) HideStatus();
		UpdateTrackNumbers();
		RefreshStatusView();
		if (saveCV->AutoUpdate())
			if (window1->Lock()) {
				// update the info box after files have been deleted
				window1->view1->box2->Refresh();
				window1->Unlock();
			}
		break;
		}
		
		case MOVE_UP_MSG:
		{
		MoveUp();
		UpdateTrackNumbers();
		break;
		}
		
		case MOVE_DOWN_MSG:
		{
		MoveDown();
		UpdateTrackNumbers();
		break;
		}
		
		case 'mm40': {
//			settingswindow->Lock();
//			if (settingswindow->writerPUM->IndexOf(settingswindow->writerPUM->FindMarked())
//					==settingswindow->readerPUM->IndexOf(settingswindow->readerPUM->FindMarked())) 
//			settingswindow->twodrivecopyCB->SetEnabled(false); else
//			settingswindow->twodrivecopyCB->SetEnabled(true);
//			settingswindow->Unlock();	
			break;
		}


		// this message contains information about the indices of an audio track
		case AUDIOWINDOW_RETURN_MSG: {
//			AudioRow	*ar;
//			BList		*list;
//			
//			msg->FindPointer("List", &list);
//			msg->FindPointer("AudioRow", &ar);
//			
//			if (ar) {
//				ar->SetIndexList(list);
//			}
			audioW=NULL;
			break;
		}
		
		case GOTOPARENT_MSG: {
			BString str="";

			if (current_folder.Length()<=base_folder.Length())
				break;
			str=current_folder;

			if (str[str.Length()-1]=='/') str.RemoveLast("/");
			str.Remove(str.FindLast("/"),str.Length()-str.FindLast("/"));

			AppendTrailingSlash(&str);
			current_folder=str;

			ShowFolder();
			Update_path();
			break;
		}

		case 'mm03': // doppelklick auf listitem
		{
			// auf audio-datei geklickt?
			if ((window1->view1->CDType()==AUDIOCD_INDEX) || (window1->view1->CDType()==CDEXTRA_INDEX)) {
				// edit audio file properties
				if (audioW==NULL) {
					AudioRow	*ar=(AudioRow *)window1->view1->listview1->CurrentSelection();
					BString		str=base_folder;
					str.Append(ar->GetFilename());
					audioW=new AudioWindow(str.String(), ar);
					audioW->SetTitle(ar->GetFilename());
					audioW->SetIndexList(ar->GetIndexList());
					audioW->SetCDText(ar->GetCDTitle());
					audioW->SetPregap(ar->GetPregap());
					if (CM->HasData(AUDIOWINDOW_RECT))
						audioW->MoveTo(CM->GetRect(AUDIOWINDOW_RECT)->LeftTop());
					audioW->Show();
				}
				break;
			}
		
		// clv update
		/*window1->Lock();
		window1->view1->listview1->Refresh();
		window1->Unlock();*/
		
		FolderRow *cs=(FolderRow *)window1->view1->listview1->CurrentSelection();
		
		if (cs==NULL) break;
		
		// clicked a normal file
		if (!cs->IsFolder()) {
			BString	s=current_folder;
			s.Append(cs->GetFilename());
			BEntry	*entry=new BEntry(s.String());
			
			entry_ref	documentRef;
			
			entry->GetRef(&documentRef);
			delete entry;
			
			if (be_roster->Launch(&documentRef, 0, NULL, NULL)==B_OK|B_ALREADY_RUNNING) {
				if (window1->Lock()) {
					char	text[200];
					
					sprintf(text, "%s %s", cs->GetFilename(), H->L("L:opened."));
					window1->view1->statusBAR->SetText(text, 4000000);
					window1->Unlock();
				}
			}
						
			break;
		}
		
		//printf("ENTER FROM %s\t%s\n",current_folder.String(),si->Text());
		BEntry entry;
		BString str="";
		str=current_folder;
		if (BString(cs->GetFilename()).Compare("..")==0)
			{
			if (str[str.Length()-1]=='/') str.RemoveLast("/");
			str.Remove(str.FindLast("/"),str.Length()-str.FindLast("/"));
			}
		else
			str.Append(cs->GetFilename());
		entry.SetTo(str.String());
		if (entry.IsDirectory())
			{
			//if (str[str.Length()-1]!='/') str.Append("/");
			AppendTrailingSlash(&str);
			current_folder=str;
			//printf("ENTERED %s\t%s\n",current_folder.String(),si->Text());
			}
		entry.Unset();
		ShowFolder();
		Update_path();
		break;
		}
		
		case 'mm04': // Liste updaten
		{
		ShowFolder();
		// Update_path();
		//CM->SetString(TEMP_PATH, settingswindow->heliospathTC->Text());
		//OLDSETTINGS sprintf(settings._heliospath,settingswindow->heliospathTC->Text());
		//CM->SetString(TEMP_PATH, AppendTrailingSlash(*CM->GetString(TEMP_PATH)));
/*		if (settings._heliospath[strlen(settings._heliospath)-1]!='/')
			sprintf(settings._heliospath, "%s/", settings._heliospath);*/
		//pathCV->SetTemporaryPath(CM->GetString(TEMP_PATH)->String());
//		if (settingswindow->Lock()) {
//			settingswindow->heliospathTC->SetText(CM->GetString(TEMP_PATH)->String());
//			settingswindow->Unlock();
//		}
		//OLDSETTINGS SaveSettings();				
		break;
		}
	
		case 'mm05': // Pathdisplay updaten
		{
		//ShowFolder();
		//Update_path();
		break;
		}
		
		case 'mm07': {
			Update_DATA_AUDIO();
			HideStatus();
			break;
		}
		
		case PAUSECHANGE_MSG:
		{
		ChangePauses();
		break;
		}

		case DATAUDIO_MSG:
		{
		window1->Lock();
		window1->view1->UpdatePanels();
		window1->Unlock();
		Update_DATA_AUDIO();
		ShowFolder(true);		// update audio, too!
		Update_path();
		if (saveCV->AutoUpdate())
			// update info box when switching between disc modes
			window1->view1->box2->Refresh();
		window1->Lock();
		window1->Unlock();
		break;
		}
	
		case UPDATEINFO_MSG: {
			if (window1->Lock()) {
				window1->view1->statusBAR->SetText(H->L("L:Updating..."), 0);
				window1->Unlock();
			}
			if (!IsProject()) break;
			// just clicked the "update" button...
			window1->view1->box2->Refresh();
			ShowFolder();
			Update_path();
			if (window1->Lock()) {
				window1->view1->statusBAR->SetText(H->L("L:Updated."), 4000000);
				window1->Unlock();
			}
			break;
		}
	
		case BURN_MSG: {
			
			status_t	error;
			if (!IsProject()) break;
			
			if (!CheckCDSize()) {
				ErrorBox *alert=new ErrorBox(E_BLUE_COLOR, H->L("TERROR"),H->L("TOVERSIZED"),H->L("TOK"), H->L("TIGNORE"));
				PlayErrorSound();
				if (alert->Go()==0)
					break;
			}
			
			window5->Lock();
			if (heliosCV->IsSimulation())
				window5->SetTitle(H->L("TSTATUSWINDOWTITLESIMULATION"));
			else
				window5->SetTitle(H->L("TSTATUSWINDOWTITLE"));
			window5->Unlock();
			
			switch(window1->view1->CDType()) {
				case	BOOTABLECD_INDEX:
				case	DATACD_INDEX:
					if ((!saveCV->IsStreamed())
							//||
						/* bfs? *//* (window1->view1->GetFilesystem()==2)*/)
						error=MakeImage_NOSTREAMING();
					else
						if (saveCV->IsStreamed())
							error=MakeImage();
					break;
					
				case	CDEXTRA_INDEX:
					error=BurnCDExtra();
					break;
				case	AUDIOCD_INDEX:
					error=BurnAudio(false);
					break;
				case	DVDVIDEO_INDEX: {
					if ((!saveCV->IsStreamed())
							//||
						/* bfs? */ /*(window1->view1->GetFilesystemDVDVideo()==2)*/)
						error=MakeImage_NOSTREAMING();
					else
						if (saveCV->IsStreamed())
							error=MakeImage();
					break;
				}
				default:
					break;
			}
			if (error==B_OK)
				PlayFinishedBurningSound();
				//if (soundCV->HasFinishedWritingSound()) beep();
			else {
				ErrorBox	*eb=new ErrorBox(E_RED_COLOR, errtype.String(), errstring.String(), H->L("TOK"));
				PlayErrorSound();
				eb->Go();
			}
			//HideStatus();
			window5->Lock();
			window5->SetTitle(H->L("TSTATUSWINDOWTITLE"));
			window5->Unlock();
			break;
		}

		case FILESYSTEM_MSG:
		{
	//	GetImageSize();
		break;
		}
		
		case RENAME_FILE_MSG:
		{
		FolderRow *selected=(FolderRow *)window1->view1->listview1->CurrentSelection();
		if (selected==NULL) break;
		window2->Lock();
		window2->todo=1;
		window2->SetTitle(H->L("TRENAMEITEM"));
		window2->view2->textcontrol3->SetLabel(H->L("TNEWNAME"));
		window2->view2->textcontrol3->SetText(selected->GetFilename());
		window2->Unlock();
		window2->Show();
		break;
		}
	
		case NEW_FOLDER_MSG:
		{
		if (!IsProject()) break;
		window2->Lock();
		window2->todo=2;
		window2->SetTitle(H->L("TCREATEFOLDER"));
		window2->view2->textcontrol3->SetLabel(H->L("TFOLDERNAME"));
		window2->view2->textcontrol3->SetText("");
		window2->Unlock();
		window2->Show();
		break;
		}


//		case SET_INDICES_MSG:
//		{
//		window2->Lock();
//		window2->todo=3;
//		window2->SetTitle(TINDICES);
//		window2->view2->textcontrol3->SetLabel(TINDICESLIST);
//		AudioRow *selected=(AudioRow *)window1->view1->listview1->CurrentSelection();
//		window2->view2->textcontrol3->SetText(selected->GetIndexList());
//		window2->Unlock();
//		window2->Show();
//		break;
//		}
		
//		case 'mm13': // indexliste erstellen
//		{
//		BString str;
//		window1->Lock();
//		str.SetTo(window2->view2->textcontrol3->Text());
//		AudioRow *selected=(AudioRow *)window1->view1->listview1->CurrentSelection();
//		selected->SetIndexList(str.String());
//		has_project_changed=true;
//		window1->Unlock();
//		break;
//		}


		case SETCDTEXTTITLE_MSG:
		{
		window2->Lock();
		window2->todo=5;
		window2->SetTitle(H->L("TCDTEXTTITLE"));
		window2->view2->textcontrol3->SetLabel(H->L("T___CDTEXTTITLE"));
		AudioRow *selected=(AudioRow *)(window1->view1->listview1->CurrentSelection());
		window2->view2->textcontrol3->SetText(selected->GetCDTitle());
		window2->Unlock();
		window2->Show();
		break;
		}
		
		case 'mm15': // cd-text title
		{
		BString str;
		window1->Lock();
		str.SetTo(window2->view2->textcontrol3->Text());
		AudioRow *selected=(AudioRow *)(window1->view1->listview1->CurrentSelection());
		selected->SetCDTitle(str.String());
		//has_project_changed=true;
		window1->Unlock();
		break;
		}


		case CDRECORDPROTOCOL_MIMSG: {
			if (cdrecord_output->Lock()) {
//				CM->SetBool(CDRECORD_LOG, cdrecord_output->DoesLogToFile());
//				CM->SetString(CDRECORD_LOGFILENAME, cdrecord_output->GetLogFileName());
//				cdrecord_output->SetLogToFile(CM->GetBool(CDRECORD_LOG));
//				cdrecord_output->SetLogFileName(CM->GetString(CDRECORD_LOGFILENAME)->String());
				if (cdrecord_output->IsHidden())
					cdrecord_output->Show();
				cdrecord_output->Unlock();
			}
			break;
		}

		case MKISOFSPROTOCOL_MIMSG: {
			if (mkisofs_output->Lock()) {
//				CM->SetBool(MKISOFS_LOG, mkisofs_output->DoesLogToFile());
//				CM->SetString(MKISOFS_LOGFILENAME, mkisofs_output->GetLogFileName());
//				mkisofs_output->SetLogToFile(CM->GetBool(MKISOFS_LOG));
//				mkisofs_output->SetLogFileName(CM->GetString(MKISOFS_LOGFILENAME)->String());
				if (mkisofs_output->IsHidden())
					mkisofs_output->Show();
				mkisofs_output->Unlock();
			}
			break;
		}
		
		
		case 'mm11': // markierten eintrag umbenennen
		{
		BEntry file;
		//off_t filesize;
		BString str="";
		FolderRow *selected=(FolderRow *)(window1->view1->listview1->CurrentSelection());
		
		if (selected==NULL) return;
		window1->Lock();
		if (BString(selected->GetFilename()).Compare("..")!=0)
		{	     		
			     str="";
			     str << current_folder.String() << selected->GetFilename();
			     //printf("WANNA REMOVE %s\n",str.String());
			     if (file.SetTo(str.String(),false)==B_OK)
			    	 {
			    	 str="";
			    	 str << current_folder.String() << window2->view2->textcontrol3->Text();
			    	 file.Rename(str.String(),false);
			    	 selected->SetFilename(window2->view2->textcontrol3->Text());
				project_has_changed=true;
	//		    	 window1->view1->listview1->InvalidateItem(selected);
			    	 }
		}
		file.Unset();
		//window1->view1->listview1->ItemAt(selected)->DrawItem(window1->view1, window1->view1->listview1->ItemFrame(selected),true);
		Update_path();
		window1->Unlock();
		break;
		}
		
		
		
		case 'mm12': // neuen ordner erstellen
		{
	//	BEntry file;
		//off_t filesize;
		BDirectory newdir;
		BString str="";
		int	count=0;
		char	dirname[2048];
		
		window1->Lock();
		
		do {
			count++;
			if (count==1)
				sprintf(dirname, "New Folder");
			else
				sprintf(dirname, "New Folder %d", count);
			str="";
			str.Append(current_folder.String()).Append(dirname);
		} while (newdir.CreateDirectory(str.String(),NULL)==B_FILE_EXISTS);
		
		project_has_changed=true;
		ShowFolder();
		window1->Unlock();
		break;
		}
		
		case NEWPROJECT_MIMSG: {
			NewProject();
			currentproject="";
			ShowFolder();
//			if (window1->Lock()) {
//				window1->view1->SetCDType(currentproj->_cdtype);
//				window1->view1->listview1->SetListMode(currentproj->_cdtype);
//				window1->Unlock();
//			}
//			ShowFolderPanel();			
			break;
		}

		case OPENPROJECT_MIMSG: {
			//ShowOpenPanel();
			
			BEntry *entry=FileDialog::OpenDialog(NULL, pathCV->GetProjectPath());

//			entry_ref ref;
//			if (message->FindRef("refs",  &ref) != B_OK) break;
			
			if (entry->InitCheck()==B_OK) {
				if (window1->Lock()) {
					window1->view1->statusBAR->SetText(H->L("L:Opening Project..."), 0);
					window1->Unlock();
				}
				
				
				LoadProject(entry);
				ShowFolder();
				if (window1->Lock()) {
					window1->view1->statusBAR->SetText(H->L("L:Project opened."), 4000000);
					window1->Unlock();
				}
				save_and_quit=false;
			} else {
				if (window1->Lock()) {
					window1->view1->statusBAR->SetText(H->L("L:Operation cancelled."), 4000000);
					window1->Unlock();
				}
			}
			entry->Unset();
			delete entry;
			break;
		}

		case SAVEPROJECTAS_MIMSG: {
			if (!IsProject()) break;
			BEntry 		*entry;
			BString		original_base_folder=base_folder;
			char		statustext[128];
			char		cmdline[4096];
			
			entry=FileDialog::SaveDialog(NULL, pathCV->GetProjectPath(), SuggestProjectName());
			if (entry->InitCheck()==B_OK) {
				if (window1->Lock()) {
					window1->view1->statusBAR->SetText(H->L("L:Saving Project..."), 0);
					window1->Unlock();
				}
				SaveProject(entry, true);
				AppendTrailingSlash(&original_base_folder);
				sprintf(cmdline, "cp -R -d \"%s\"* \"%s\"", original_base_folder.String(), base_folder.String());
				if (window1->Lock()) {
					window1->view1->statusBAR->SetText(H->L("TCOPYINGFILES"), 0);
					window1->Unlock();
				}
				system(cmdline);
				//printf("new path=%s\n", base_folder.String());
				
				sprintf(statustext, H->L("L:Project saved."));
				save_and_quit=false;
			} else {
				sprintf(statustext, H->L("L:Operation cancelled."));
			}
				
			entry->Unset();
			delete entry;
			if (window1->Lock()) {
				window1->view1->statusBAR->SetText(statustext, 4000000);
				window1->Unlock();
			}
			break;
		}


		case REMOVEPROJECT_MIMSG: {
			if (!IsProject()) break;
			BString		bf=base_folder;
			
			bf=*RemoveTrailingSlash(bf);
			
			printf("bf=%s\n", bf.String());
			BEntry	*folder_of_project=new BEntry(bf.String(), false);
			BEntry	*projectfile=new BEntry(currentproject.String(), true);
			printf("cp=%s\n", currentproject.String());

			ErrorBox	*eb=new ErrorBox(E_BLUE_COLOR, "", H->L("TAREYOUSURE"), H->L("TCANCEL"), H->L("TMOVETOTRASH"), H->L("TERASE"));
			switch (eb->Go()) {
				case 2: {
					DeleteAllIn(BDirectory(bf.String()));
					folder_of_project->Remove();
					projectfile->Remove();
					CM->SetString(PATH_TO_LAST_PROJECT, "");
					currentproject="";
					base_folder="";
					current_folder="";
					ShowFolder();
					window1->SetTitle(H->L("TWINDOWTITLE"));
					PostMessage(NEWPROJECT_MIMSG);
					ShowRecentProjects();
					break;
				}
				
				case 1: {
					// first, find the trash directory of this volume
					char	path_buffer[B_FILE_NAME_LENGTH];
					
					if (find_directory(B_TRASH_DIRECTORY, dev_for_path(currentproject.String()), true, path_buffer, B_FILE_NAME_LENGTH)!=B_OK) {
						ErrorBox	*eb=new ErrorBox(E_RED_COLOR, H->L("TERROR"), H->L("Error:Removing Project"), H->L("TOK"));
						eb->Go();
						break;
					}
					
					folder_of_project->MoveTo(new BDirectory(path_buffer));
					projectfile->MoveTo(new BDirectory(path_buffer));
					CM->SetString(PATH_TO_LAST_PROJECT, "");
					currentproject="";
					base_folder="";
					current_folder="";
					ShowFolder();
					window1->SetTitle(H->L("TWINDOWTITLE"));
					PostMessage(NEWPROJECT_MIMSG);
					ShowRecentProjects();
					break;
				}
				
				case 0: {
					break;
				}
			};
			folder_of_project->Unset();
			delete folder_of_project;
			projectfile->Unset();
			delete projectfile;
			break;
		}


//		case B_CANCEL: {
//			if (filepanel!=NULL)
//				delete filepanel;
//			save_and_quit=false;
//			break;
//		}

		case OPEN_FPMSG: {	
			entry_ref ref;
			if (message->FindRef("refs",  &ref) != B_OK) break;
			
			if (window1->Lock()) {
				window1->view1->statusBAR->SetText(H->L("L:Opening Project..."), 0);
				window1->Unlock();
			}
			
			BEntry *entry=new BEntry(&ref, true);
			LoadProject(entry);
			ShowFolder();
			entry->Unset();
			delete entry;
			if (window1->Lock()) {
				window1->view1->statusBAR->SetText(H->L("L:Project opened."), 4000000);
				window1->Unlock();
			}
			break;
		}

		case FOLDER_FPMSG: {	
			entry_ref ref;
			BPath	path;
			if (message->FindRef("refs",  &ref) != B_OK) break;
			
			BEntry *entry=new BEntry(&ref, true);
			entry->GetPath(&path);
		
			BDirectory HeliosDir;
			BString bf=path.Path();
			if (bf[bf.Length()-1]=='/') bf.RemoveLast("/");
			if (HeliosDir.SetTo(path.Path())!=B_OK)
				HeliosDir.CreateDirectory(bf.String(),&HeliosDir);
			HeliosDir.Unset();
			base_folder=path.Path();
			//if (base_folder[base_folder.Length()-1]!='/') base_folder.Append("/");
			AppendTrailingSlash(&base_folder);
			current_folder=base_folder;
			NewProject();
			
			ShowFolder();
			entry->Unset();
			delete entry;

			break;
		}

//		case SAVE_FPMSG: {	
//			entry_ref ref;
//			BString name="";
//			BString n="";
//			BEntry	en;
//			BPath path;
//			
//			if (message->FindRef("directory",  &ref) != B_OK) break;
//			en.SetTo(&ref, true);
//			en.GetPath(&path);
//			name.SetTo(path.Path());
//			//if (name.ByteAt(name.Length()-1)!='/') name.Append("/");
//			name=*AppendTrailingSlash(&name);
//			message->FindString("name",&n);
//			name.Append(n.String());
//			SaveProject(new BEntry(name.String(), true));
//			en.Unset();
//			path.Unset();
//			break;
//		}

		case SAVEPROJECT_MIMSG: {
			if (!IsProject()) break;
			BEntry *entry=new BEntry(currentproject.String(), true);
			char	statustext[128];
			printf("MessageReceived():currentproject=%s\n", currentproject.String());
			if (!entry->Exists()) {
				entry->Unset();
				delete entry;
				entry=FileDialog::SaveDialog(NULL, pathCV->GetProjectPath(), SuggestProjectName());
			}
			if (entry->InitCheck()==B_OK) {
				if (window1->Lock()) {
					window1->view1->statusBAR->SetText(H->L("L:Saving Project..."), 0);
					window1->Unlock();
				}
				SaveProject(entry);
				sprintf(statustext, H->L("L:Project saved."));
				save_and_quit=false;
			} else {
				sprintf(statustext, H->L("L:Operation cancelled."));
			}
				
			entry->Unset();
			delete entry;
			if (window1->Lock()) {
				window1->view1->statusBAR->SetText(statustext, 4000000);
				window1->Unlock();
			}
			break;
		}
		
		case STANDARDPROJECT_MIMSG: {
			if (!IsProject()) break;
			SetAsStandardProject();
			break;
		}

		case FASTBLANK_MIMSG: {
			if (BlankRW("-blank=fast")==B_OK)
				PlayFinishedBurningSound();
			else {
				ErrorBox	*eb=new ErrorBox(E_RED_COLOR, errtype.String(), errstring.String(), H->L("TOK"));
				PlayErrorSound();
				eb->Go();
			}
			break;
		}

		case COMPLETEBLANK_MIMSG: {
			if (BlankRW("-blank=all")==B_OK)
				PlayFinishedBurningSound();
			else {
				ErrorBox	*eb=new ErrorBox(E_RED_COLOR, errtype.String(), errstring.String(), H->L("TOK"));
				PlayErrorSound();
				eb->Go();
			}
			break;
		}

		case UNCLOSESESSION_MIMSG: {
			if (BlankRW("-blank=unclose")==B_OK)
				PlayFinishedBurningSound();
			else {
				ErrorBox	*eb=new ErrorBox(E_RED_COLOR, errtype.String(), errstring.String(), H->L("TOK"));
				PlayErrorSound();
				eb->Go();
			}
			break;
		}

		case BLANKSESSION_MIMSG: {
			if (BlankRW("-blank=session")==B_OK)
				PlayFinishedBurningSound();
			else {
				ErrorBox	*eb=new ErrorBox(E_RED_COLOR, errtype.String(), errstring.String(), H->L("TOK"));
				PlayErrorSound();
				eb->Go();
			}
			break;
		}
		
		case EXPORT_M3U_MIMSG: {
			if (!IsProject()) break;
			
			char	name[1024];
			sprintf(name, "%s.m3u", SuggestProjectName());
			
			BEntry	*entry=FileDialog::SaveDialog(H->L("FP:Export"), pathCV->GetProjectPath(), name);

			if (entry->InitCheck()==B_OK) {
				if (window1->Lock()) {
					window1->view1->statusBAR->SetText(H->L("L:Exporting project..."), 0);
					window1->Unlock();
				}
				ExportAsM3U(entry);
				if (window1->Lock()) {
					window1->view1->statusBAR->SetText(H->L("L:Project exported."), 4000000);
					window1->Unlock();
				}
				entry->Unset();
			} else {
				if (window1->Lock()) {
					window1->view1->statusBAR->SetText(H->L("L:Operation cancelled."), 4000000);
					window1->Unlock();
				}
			}
			delete entry;
			break;
		}
		
/*		case EXPORT_M3U_FPMSG: {
			entry_ref ref;
			BString name="";
			BString n="";
			BEntry	en;
			BPath path;
			
			if (message->FindRef("directory",  &ref) != B_OK) break;
			en.SetTo(&ref, true);
			en.GetPath(&path);
			name.SetTo(path.Path());
			//if (name.ByteAt(name.Length()-1)!='/') name.Append("/");
			AppendTrailingSlash(&name);
			message->FindString("name",&n);
			name.Append(n.String());
			ExportAsM3U(new BEntry(name.String(), true));
			en.Unset();
			path.Unset();
			break;
		}
*/		
		case EXPORT_APML_MIMSG: {
			if (!IsProject()) break;
			
			char	name[1024];
			sprintf(name, "%s.apml", SuggestProjectName());
			
			BEntry	*entry=FileDialog::SaveDialog(H->L("FP:Export"), pathCV->GetProjectPath(), name);

			if (entry->InitCheck()==B_OK) {
				if (window1->Lock()) {
					window1->view1->statusBAR->SetText(H->L("L:Exporting project..."), 0);
					window1->Unlock();
				}
				ExportAsAPML(entry);
				if (window1->Lock()) {
					window1->view1->statusBAR->SetText(H->L("L:Project exported."), 4000000);
					window1->Unlock();
				}
				entry->Unset();
			} else {
				if (window1->Lock()) {
					window1->view1->statusBAR->SetText(H->L("L:Operation cancelled."), 4000000);
					window1->Unlock();
				}
			}
			delete entry;
			break;
		}
		
/*		case EXPORT_APML_FPMSG: {
			entry_ref ref;
			BString name="";
			BString n="";
			BEntry	en;
			BPath path;
			
			if (message->FindRef("directory",  &ref) != B_OK) break;
			en.SetTo(&ref, true);
			en.GetPath(&path);
			name.SetTo(path.Path());
			//if (name.ByteAt(name.Length()-1)!='/') name.Append("/");
			AppendTrailingSlash(&name);
			message->FindString("name",&n);
			name.Append(n.String());
			ExportAsAPML(new BEntry(name.String(), true));
			en.Unset();
			path.Unset();
			break;
		}
*/
		case EXPORT_FILELIST_MIMSG: {
			if (!IsProject()) break;
			
			char	name[1024];
			sprintf(name, "%s.txt", SuggestProjectName());
			
			BEntry	*entry=FileDialog::SaveDialog(H->L("FP:Export"), pathCV->GetProjectPath(), name);

			if (entry->InitCheck()==B_OK) {
				if (window1->Lock()) {
					window1->view1->statusBAR->SetText(H->L("L:Exporting project..."), 0);
					window1->Unlock();
				}
				ExportAsFileList(entry);
				if (window1->Lock()) {
					window1->view1->statusBAR->SetText(H->L("L:Project exported."), 4000000);
					window1->Unlock();
				}
				entry->Unset();
			} else {
				if (window1->Lock()) {
					window1->view1->statusBAR->SetText(H->L("L:Operation cancelled."), 4000000);
					window1->Unlock();
				}
			}
			delete entry;
			break;
		}
		
/*		case EXPORT_FILELIST_FPMSG: {
			entry_ref ref;
			BString name="";
			BString n="";
			BEntry	en;
			BPath path;
			
			if (message->FindRef("directory",  &ref) != B_OK) break;
			en.SetTo(&ref, true);
			en.GetPath(&path);
			name.SetTo(path.Path());
			//if (name.ByteAt(name.Length()-1)!='/') name.Append("/");
			AppendTrailingSlash(&name);
			message->FindString("name",&n);
			name.Append(n.String());
			ExportAsFileList(new BEntry(name.String(), true));
			en.Unset();
			path.Unset();
			break;
		}
*/
		case EXPORT_TRACKERTEMPLATE_MIMSG: {
			if (!IsProject()) break;
			BEntry *entry=FileDialog::SaveDialog(H->L("FP:Export"), "/boot/home/config/settings/Tracker/Tracker New Templates", SuggestProjectName());
			if (entry->InitCheck()==B_OK) {
				SaveProject(entry, true);
				entry->Unset();
			}
			delete entry;
//			ShowExportPanel(new BMessage(EXPORT_TRACKERTEMPLATE_FPMSG));
			break;
		}
		

		case FOLDERMENU_MSG: {	// user clicked on statusview's folders-popupmenu
			BString folder;
			if (message->FindString("folder", &folder)==B_OK) {
				//if (folder[folder.Length()-1]!='/') folder.Append("/");
				AppendTrailingSlash(&folder);
				current_folder=folder;
				ShowFolder();
				Update_path();
			}
			break;
		}

		case IMAGE_MSG: {
			break;
		}
		
		
		case BURNIMAGEFILE_MIMSG: {
			status_t	error=B_OK;
			BEntry 		*entry;
			BPath		path;
			
			if (!CheckCDSize()) {
				ErrorBox *alert=new ErrorBox(E_BLUE_COLOR, H->L("TERROR"),H->L("TOVERSIZED"),H->L("TOK"), H->L("TIGNORE"));
				PlayErrorSound();
				if (alert->Go()==0)
					break;
			}
			entry=FileDialog::OpenDialog(H->L("FP:Select Image File"));
			entry->GetPath(&path);
			if (!entry->Exists()) {
/*				ErrorBox *alert=new ErrorBox(E_BLUE_COLOR, H->L("TERROR"),H->L("TNOTEXIST_ALERT"),H->L("TOK"));
				PlayErrorSound();
				//alert->SetShortcut(0,B_ENTER);
				alert->Go();*/
			} else {
				error=BurnImage(BString(path.Path()));
				if (error==B_OK) {
					PlayFinishedBurningSound();
					ErrorBox *alert=new ErrorBox(E_BLUE_COLOR, "",H->L("TDELETEIMAGEFILE"),H->L("TNO"),H->L("TYES"));
					alert->SetShortcut(0,B_ESCAPE);
					path.Unset();
					if (alert->Go()==1)
						entry->Remove();
				} else {
					ErrorBox	*eb=new ErrorBox(E_RED_COLOR, errtype.String(), errstring.String(), H->L("TOK"));
					PlayErrorSound();
					eb->Go();
				}
			}
			path.Unset();
			entry->Unset();
			delete entry;
			break;
		}
		
		case READCDIMAGE_MIMSG: {
			status_t	error=B_OK;
			BEntry 		*entry;
			BPath		path;
			
			entry=FileDialog::SaveDialog(H->L("FP:Enter New Image File"));
			/*if (entry->Exists()) {
				ErrorBox *alert=new ErrorBox(E_BLUE_COLOR, "",H->L("TREPLACE_ALERT"),H->L("TNO"),H->L("TYES"));
				alert->SetShortcut(0,B_ESCAPE);
				if (alert->Go()==1)
					entry->Remove();
				else {
					entry->Unset();
					break;
				}
			}
			*/
			if (entry->InitCheck()==B_OK) {
				entry->GetPath(&path);
				error=ReadImage(new BString(path.Path()));
				if (error==B_OK) {
					PlayFinishedImageSound();
				} else {
					ErrorBox	*eb=new ErrorBox(E_RED_COLOR, errtype.String(), errstring.String(), H->L("TOK"));
					PlayErrorSound();
					eb->Go();
				}
				path.Unset();
			}
			entry->Unset();
			delete entry;
			break;
		}


		case READFLOPPYIMAGE_MIMSG: {
			status_t	error=B_OK;
			BEntry 		*entry;
			BPath		path;
			
			entry=FileDialog::SaveDialog(H->L("FP:Enter New Image File"));
			/*if (entry->Exists()) {
				ErrorBox *alert=new ErrorBox(E_BLUE_COLOR, "",H->L("TREPLACE_ALERT"),H->L("TNO"),H->L("TYES"));
				alert->SetShortcut(0,B_ESCAPE);
				if (alert->Go()==1)
					entry->Remove();
				else {
					entry->Unset();
					break;
				}
			}
			*/
			if (entry->InitCheck()==B_OK) {
				entry->GetPath(&path);
				error=ReadFloppyImage(path.Path());
				if (error==B_OK) {
					PlayFinishedImageSound();
				} else {
					ErrorBox	*eb=new ErrorBox(E_RED_COLOR, errtype.String(), errstring.String(), H->L("TOK"));
					PlayErrorSound();
					eb->Go();
				}
				path.Unset();
			}
			entry->Unset();
			delete entry;
			break;
		}
		
		case COPYDATACD_MIMSG: {
			if (deviceCV->IsTwoDriveCopy())
				CopyOnTheFly();
			else
				CopyDataCD();
			PlayFinishedBurningSound();
			break;
		}
		
		case COPYAUDIOCD_MIMSG: {
			status_t	error;
			//int 		track_count=0;
			
			if (deviceCV->IsTwoDriveCopy())
				error=CopyOnTheFlyAudio();
			else
				error=CopyAudioCD();
				
			if (error>=0) {
				//track_count=ReadAudioCD();
				PlayFinishedImageSound();
			} else {
				ErrorBox	*eb=new ErrorBox(E_RED_COLOR, errtype.String(), errstring.String(), H->L("TOK"));
				PlayErrorSound();
				eb->Go();
				break;
			}
			//printf("track_count=%d\n",track_count);
			//ShowFolder();
			//if (error==B_OK)
				
			break;
		}

		case CREATEFILEIMAGE_MIMSG: {
			status_t	error=B_OK;
			BEntry 		*entry;
			BPath		path;

			if (!IsProject()) break;
			entry=FileDialog::SaveDialog(H->L("FP:Enter New Image File"));
			/*if (entry->Exists()) {
				ErrorBox *alert=new ErrorBox(E_BLUE_COLOR, "",H->L("TREPLACE_ALERT"),H->L("TNO"),H->L("TYES"));
				alert->SetShortcut(0,B_ESCAPE);
				if (alert->Go()==1)
					entry->Remove();
				else
					break;
			}
			*/
			if (entry->InitCheck()==B_OK) {
				entry->GetPath(&path);
				error=MakeImageOnly(path.Path());
				if (error==B_OK) {
					PlayFinishedImageSound();
				} else {
					ErrorBox	*eb=new ErrorBox(E_RED_COLOR, errtype.String(), errstring.String(), H->L("TOK"));
					PlayErrorSound();
					eb->Go();
				}
				path.Unset();
			}
			errtype="";
			errstring="";
			entry->Unset();
			delete entry;
			break;
		}
		
		

		case BURNSPEED_MSG:
		{
//		BString str=TRECORDERSPEED;
//		str << " (" << settingswindow->writerspeedSL->Value()*2 << "x)";
//		settingswindow->Lock();
//		settingswindow->writerspeedSV->SetText(str.String());
//		settingswindow->Unlock();
		Calculate_time();
		break;
		}
		
		case SPLIT_TRACK_MSG:
		{
		BString str="";
		window2->Lock();
		window2->todo=4;
		window2->SetTitle(H->L("TSPLITTRACK"));
		window2->view2->textcontrol3->SetLabel(H->L("TSPLITOFFSET"));
		AudioRow *selected=(AudioRow *)(window1->view1->listview1->CurrentSelection());
		// display the half size of track as a suggestion for the split offset
		str << selected->GetLength()/2 << "s";
		window2->view2->textcontrol3->SetText(str.String());
		window2->Unlock();
		window2->Show();
		break;
		}
		
		case 'mm14': // splits the selected track into two parts
		{
		AudioRow *ai=(AudioRow *)(window1->view1->listview1->CurrentSelection());
		BString str=window2->view2->textcontrol3->Text();
		size_t splitoffset; // in bytes
		
		if (str.FindFirst("f")==B_ERROR) {
			// typed in as seconds
			str.RemoveAll("s");
			splitoffset=atol(str.String())*176400;			
		} else {
			// typed in as frames
			str.RemoveAll("f");
			splitoffset=176400*atol(str.String())/75;
		}
		window1->Lock();
		window1->view1->listview1->AddRow(new AudioRow(ai->GetTrackNumber()+1, ai->GetFilename(),0,ai->GetBytes()-splitoffset, (ai->GetBytes()-splitoffset)*176400), window1->view1->listview1->IndexOf(ai)+1);
		ai->SetBytes(splitoffset);
		if (!ai->IsSplitted()) {ai->SetSplitIndex(0);ai->SetSplitted(true);}
		project_has_changed=true;
		UpdateTrackNumbers();
		window1->Unlock();
		break;
		}

		case FIXATE_CHKBOX_MSG:
		{
		Calculate_time();
		break;
		}
		
		case SAVE_SETTINGS: {
			SaveSettings();
			break;
		}
		
		case HELIOS_LANGUAGE_UPDATE_MSG: {
			BWindow	*window;
			
			for (int32 w=0; w<CountWindows(); w++) {
				window=WindowAt(w);
				
					window->Lock();
				for (int32 v=0; v<window->CountChildren(); v++) {
					window->ChildAt(v)->Looper()->PostMessage(HELIOS_LANGUAGE_UPDATE_MSG, window->ChildAt(v));
				}
					window->Unlock();
			}
			break;
		}
		
		default:
		{
			BApplication::MessageReceived(message);
			break;
		}	
	}

	if (applicationCV->IsAutoUpdating())
		window1->view1->box2->StartUpdating();
}

void
Application1::RefsReceived(BMessage *message)
{

	uint32 		type;
	int32 		count;
	BEntry		*entry=new BEntry();
	entry_ref	ref;

	message->GetInfo("refs", &type, &count);
	
	if (type != B_REF_TYPE) { delete entry; return;}
	if (message->FindRef("refs", 0, &ref) != B_OK) {
		if (window1->Alert("Error at BMessage.FindRef()\n\nProbably you didn't drop a project file.")==true) { delete entry; return;}
	} else	{
		if (entry->SetTo(&ref,true)==B_OK)
			LoadProject(entry);
	}
	delete entry;	
	ShowFolder();
}

void Application1::AboutRequested(void) {
	app_info	info;
	AboutWindow	*aboutW=new AboutWindow(BRect(100,100,400,280), "About");
	BScreen	*screen=new BScreen();

	aboutW->MoveTo((int)((screen->Frame().Width()-aboutW->Bounds().Width())/2), (int)((screen->Frame().Height()-aboutW->Bounds().Height())/2));
	delete screen;
	if (GetAppInfo(&info)==B_OK) {
		BBitmap bmp(BRect(0,0,31,31), B_CMAP8);
		if (BNodeInfo::GetTrackerIcon(&info.ref, &bmp, B_LARGE_ICON)==B_OK)
			aboutW->SetIcon(&bmp);
	}
	aboutW->SetApplicationName(H->L("ABOUT_APPLICATIONNAME"));
	aboutW->SetVersionNumber(H->L("ABOUT_VERSIONNUMBER"));
	aboutW->SetCopyrightString(H->L("ABOUT_COPYRIGHTSTRING"));
	aboutW->SetText(H->L("ABOUT_TEXT"));
	aboutW->Show();
}





/****************************************************************************
 *
 *	Shows content of current folder in file window.
 *
 ***************************************************************************/
void
Application1::ShowFolder(bool audio) {
	BDirectory dir;
	BEntry de;
	BPath path;
	int tracks=0;
	entry_ref	ref;
	
	
	dir.SetTo(current_folder.String());
	dir.Rewind();
	window1->Lock();
	if ((audio) || ((window1->view1->CDType()==DATACD_INDEX) || (window1->view1->CDType()==DVDVIDEO_INDEX) || (window1->view1->CDType()==BOOTABLECD_INDEX))) {
//		window1->view1->listview1->iMakeEmpty();
		while (window1->view1->listview1->CountRows()>0) {
			BRow	*row=window1->view1->listview1->RowAt(0);
			window1->view1->listview1->BColumnListView::RemoveRow(row);
			delete row;
		}
	}
	if (current_folder!=base_folder) {
	//printf("\n\n[%s]\n[%s]\n\n", current_folder.String(), base_folder.String());
		FolderRow	*frROW=new FolderRow("..", true, NULL, true);
		window1->view1->listview1->AddRow(frROW ,(int32)0);
		frROW->GetField(FILENAME_FIELD)->SetEditable(false);
	}
	window1->Unlock();

			while (dir.GetNextEntry(&de, false)==B_OK) {
				if (de.IsDirectory()) {
					de.GetPath(&path);
					if ((window1->view1->CDType()==DATACD_INDEX) ||
						(window1->view1->CDType()==BOOTABLECD_INDEX) ||
						(window1->view1->CDType()==DVDVIDEO_INDEX)) {	
							BFile file(&de, B_READ_ONLY);
							off_t filesize;
							file.GetSize(&filesize);
							de.GetRef(&ref);
							if (window1->Lock()) {
								window1->view1->listview1->AddRow(new FolderRow(path.Leaf(), 0, true, GetIcon(&ref)),(int32)0);
								window1->Unlock();
							}
							file.Unset();
					}
				}
			}
			dir.Rewind();
	
			while (dir.GetNextEntry(&de,false)==B_OK)
			{
			if ((de.IsSymLink()) || (de.IsFile()))
				{
				de.GetPath(&path);
				if ((window1->view1->CDType()==DATACD_INDEX) ||
					(window1->view1->CDType()==BOOTABLECD_INDEX) ||
					(window1->view1->CDType()==DVDVIDEO_INDEX))
					{
					BFile file(&de, B_READ_ONLY);
					off_t filesize;
					if (file.GetSize(&filesize)!=B_OK) filesize=0;
					de.GetRef(&ref);
					if (window1->Lock()) {
						window1->view1->listview1->AddRow(new FolderRow(path.Leaf(),filesize, false, GetIcon(&ref)), window1->view1->listview1->CountRows());
						window1->Unlock();
					}
					file.Unset();
					} else
					if (audio)
					{
					float size;

					
					TrackLength *info=new TrackLength(new BEntry(path.Path(), true));
					size=info->GetBytes();

					//info.~MMediaInfo();

					if (size>0) {
						if (window1->Lock()) {
							AudioRow	*ar;
							window1->view1->listview1->AddRow((ar=new AudioRow(++tracks, path.Leaf(),0,size, size/176400)));
							ar->SetStartFrame((int64)0);
							ar->SetEndFrame((int64)(75.0F*(float)size/176400.0F));
							ar->SetLength((int64)(75.0F*(float)size/176400.0F));
							if (tracks==1)
								ar->SetPregap(150);
							window1->Unlock();
						}
					}
					delete info;
					
					}
				//printf("FILEFOUND %s\n",path.Leaf());
				}

			}
	dir.Unset();
	de.Unset();
	path.Unset();
	
	
	RefreshStatusView();
	
	window1->Lock();
	BString	cp=current_folder;
	cp.ReplaceFirst(base_folder.String(), "");
	cp.Prepend("/");
	window1->view1->SetCurrentPath(cp.String());
	if (window1->view1->listview1->CountRows()>0)
		window1->view1->listview1->ScrollTo(window1->view1->listview1->RowAt(0));
	PopulateFoldersMenu();
	window1->Unlock();
}


void Application1::RefreshStatusView() {
	int32 items=window1->view1->listview1->CountRows();
	if (current_folder!=base_folder) items--;
	if (window1->Lock()) {
		window1->view1->listview1->statusView->SetNumberOfItems(items);
		window1->Unlock();
	}
}




//fügt rekursiv den ordner/die datei hinzu
bool Application1::CreateEntry(char *pathname, char *entryname)
{
BEntry entry, de;
BString str="";
BPath path;
BDirectory d;

//if ((!addershows) && (watch))
//	if (watch->ElapsedTime()>1200000)
//						{
//						ShowStatus(H->L("TADDINGFILES"));
//						addershows=true;
//						}



entry.SetTo(pathname, false); //links nicht nachverfolgen
if (entry.IsDirectory()) {
	BDirectory dir;
	if (dir.SetTo(&entry)!=B_OK) {entry.Unset(); return false;}
	//if (cdir[cdir.Length()-1]!='/') cdir.Append("/");
	AppendTrailingSlash(&cdir);
	cdir.Append(entryname);
	//printf("CDIR %s\n",cdir.String());
	if (filecounter % 20==0) {
		if (window1->Lock()) {
			char	text[100];
			
			sprintf(text, "%s (%ld)", H->L("L:Adding files..."), filecounter);
			window1->view1->statusBAR->SetText(text);
			window1->Unlock();
		}
	}
	filecounter++;
//	if (statusSW->Lock()) {
//		statusSW->SetFileCounter(statusSW->GetFiles()+1);
//		statusSW->Unlock();
//	}

	if (dir.CreateDirectory(cdir.String(),NULL)!=B_OK) {entry.Unset(); dir.Unset();return false;}
	dir.Rewind();
	while (dir.GetNextEntry(&de,false)==B_OK) {
		de.GetPath(&path);
		str="";
		if (de.IsDirectory()) {
		//	de.GetPath(&path);
		//	str="";
			//if (cdir[cdir.Length()-1]!='/') cdir.Append("/");
			AppendTrailingSlash(&cdir);
			str.Append(cdir).Append(path.Leaf());
			//printf("CREATEENTRY %s\n",path.Path());
			CreateEntry((char *)path.Path(), (char *)path.Leaf());
			cdir.Remove(cdir.FindLast("/"),cdir.Length()-cdir.FindLast("/"));
			//printf("PARENTDIR %s\n",cdir.String());
		}
		
		if (de.IsFile()) {
		//	de.GetPath(&path);
		//	str="";
			//if (cdir[cdir.Length()-1]!='/') cdir.Append("/");
			str.Append(cdir).Append(path.Leaf());
			//printf("CREATEENTRY %s\n",path.Path());
			CreateEntry((char *)path.Path(), (char *)path.Leaf());
		}
	}
	dir.Unset();
} else
if (entry.IsFile()) {
	entry.GetPath(&path);
	str=cdir;
	BSymLink	*sl=new BSymLink(&entry);
	if (filecounter % 20==0) {
		if (window1->Lock()) {
			char	text[100];
			
			sprintf(text, "%s (%ld)", H->L("L:Adding files..."), filecounter);
			window1->view1->statusBAR->SetText(text);
			window1->Unlock();
		}
	}
	filecounter++;
//	if (statusSW->Lock()) {
//		statusSW->SetFileCounter(statusSW->GetFiles()+1);
//		statusSW->Unlock();
//	}
	AppendTrailingSlash(&str);
	str.Append(path.Leaf());
	//printf("str=\"%s\"\npathname=%s", str.String(), pathname);
	d.CreateSymLink(str.String(), pathname, sl);
	delete sl;
	//str << "ln -s \"" << pathname << "\" \"" << cdir << "\"";
	//printf("%s\n",str.String());
	//system(str.String());
}

//window1->Lock();
//window1->view1->stringview1->SetText("");
//window1->Unlock();
entry.Unset();
de.Unset();
path.Unset();
d.Unset();

return true;
}


// unused by now
void Application1::Update_path() {
/*
	int32 selected=window1->view1->listview1->CurrentSelection();
	BString str=current_folder;
	str.RemoveFirst(base_folder);
	pathdisplay.SetTo(window1->view1->textcontrol1->Text());
	pathdisplay.Prepend("/").Append("/").Append(str.String());
	
	if ((selected>=0) && (window1->view1->listview1->CurrentSelection(1)<0)) {
		BStringItem *si=(BStringItem *)window1->view1->listview1->ItemAt(selected);
		if (window1->view1->radiobutton1->Value()==B_CONTROL_ON)
			pathdisplay.Append(si->Text()); else
			pathdisplay.SetTo(si->Text());
		
		window1->Lock();
		window1->view1->stringview1->SetText(pathdisplay.String());
		window1->Unlock();
	} else {
		window1->Lock();
		if (window1->view1->radiobutton1->Value()==B_CONTROL_ON)
			window1->view1->stringview1->SetText(pathdisplay.String()); 
		else
			window1->view1->stringview1->Clear();
		window1->Unlock();
	}
*/
}


bool Application1::GetImageSize()
{
		int arguments=0;
		thread_id Thread;
		const char **args;
		BString str="";
		BString filesystem="";
		int in, out, err;
		size_t	sz=0;
		
		switch(window1->view1->CDType()) {
			case AUDIOCD_INDEX: {
				float time=0;
				//pause=atof(window1->view1->textcontrol2->Text());
		
				for (int i=0; i<window1->view1->listview1->CountRows();i++) {
					AudioRow *si=(AudioRow *)window1->view1->listview1->RowAt(i);
	
					time+=si->GetLength();
					time+=(float)si->GetPregap();
				}
				window1->view1->box2->SetCDSize((size_t)((time/75.0F)*153600.0F));
				return true;
				break;
			}

			case DATACD_INDEX: {
				BList	*list=window1->view1->datapanel->GetFilesystem();
				if (list->CountItems()==1)
					if (((BString *)list->ItemAt(0))->Compare("BFS")==0) {
						imgsize=0;
						files=0;
						directories=0;
					
						// get size of directories and files
						GetSize(BDirectory(base_folder.String()));
						imgsize+=files*4096+directories*133200;	// die informationen für fensterpositionen und
												// positionen der icons in den fenstern
												// werden auch abgespeichert...
						imgsize+=(5144*1024)+64*imgsize/(1024*1024)+384;
						window1->view1->box2->SetCDSize((off_t)imgsize);
						delete ((BString *)list->RemoveItem(0L));
						delete list;
						return true;
					}
					
				for (int i=list->CountItems()-1; i>=0; i--)
					delete ((BString *)list->RemoveItem(i));
				delete list;
				
				break;
			}

			case BOOTABLECD_INDEX: {
//				if (window1->view1->GetFilesystemBootCD()==2) {
//					imgsize=0;
//					files=0;
//					directories=0;
//				
//					// get size of directories and files
//					GetSize(BDirectory(base_folder.String()));
//					imgsize+=files*4096+directories*133200;	// die informationen für fensterpositionen und
//											// positionen der icons in den fenstern
//											// werden auch abgespeichert...
//					imgsize+=(5144*1024)+64*imgsize/(1024*1024)+384;
//					window1->view1->box2->SetCDSize((off_t)imgsize);
//					return true;
//				}
				break;
			}
			
			case CDEXTRA_INDEX: {
				float time=0;
		
				for (int i=0; i<window1->view1->listview1->CountRows();i++) {
					AudioRow *si=(AudioRow *)window1->view1->listview1->RowAt(i);
					time+=si->GetLength();
					time+=(float)si->GetPregap();
				}
				sz=(size_t)((float)time/75.0F*153600.0F);
				
				BList	*list=window1->view1->cdextrapanel->GetFilesystem();
				if (list->CountItems()==1)
					if (((BString *)list->ItemAt(0))->Compare("BFS")==0) {
						imgsize=0;
						files=0;
						directories=0;
					
						// get size of directories and files
						GetSize(BDirectory(base_folder.String()));
						imgsize+=files*4096+directories*133200;	// die informationen für fensterpositionen und
												// positionen der icons in den fenstern
												// werden auch abgespeichert...
						imgsize+=(5144*1024)+64*imgsize/(1024*1024)+384;
						window1->view1->box2->SetCDSize((off_t)imgsize+sz);
						delete ((BString *)list->RemoveItem(0L));
						delete list;
						return true;
					}
				for (int i=list->CountItems()-1; i>=0; i--)
					delete ((BString *)list->RemoveItem(i));
				delete list;
				break;
			}
			
			case DVDVIDEO_INDEX: {
				BList	*list=window1->view1->dvdvideopanel->GetFilesystem();
				if (list->CountItems()==1)
					if (((BString *)list->ItemAt(0))->Compare("BFS")==0) {
						imgsize=0;
						files=0;
						directories=0;
					
						// get size of directories and files
						GetSize(BDirectory(base_folder.String()));
						imgsize+=files*4096+directories*133200;	// die informationen für fensterpositionen und
												// positionen der icons in den fenstern
												// werden auch abgespeichert...
						imgsize+=(5144*1024)+64*imgsize/(1024*1024)+384;
						window1->view1->box2->SetCDSize((off_t)imgsize);
						delete ((BString *)list->RemoveItem(0L));
						delete list;
						return true;
					}
					
				for (int i=list->CountItems()-1; i>=0; i--)
					delete ((BString *)list->RemoveItem(i));
				delete list;
				
				break;
			}
			
			default: break;
		}
		
			
		

		
		
		args = (const char **)malloc(sizeof(char *) * (100));
		//arguments=MakeMKISOFSCommandline(args, "", true);
		if (MKISOFS_CommandLine(args, &arguments, true)!=B_OK)
			return false;

		//printArguments("MKISOFS_CommandLine", args);
		//printf("argc=%d\n", arguments);
		Thread=pipe_command(arguments, args, in, out, err);
		set_thread_priority(Thread,2);
		if (Thread<0) {
			errtype= H->L("TERROR");
			errstring="mkisofs\n\n";
			errstring << H->L("TCOULDNOTLOAD");
			free(args);
			HideStatus2();
			return B_ERROR;
		}
		rename_thread(Thread,MKISOFSTHREADNAME);
		size_t bytesread=0;
		char buffer[10000];
		bool gotit=false;

		resume_thread(Thread);
		do {
			bytesread=read(err,(void *)buffer, 10000);
			buffer[bytesread]=0;

			// output it to the log-window
			 mkisofs_output->AddText(buffer);

			if (bytesread>0) {
				if (strstr(buffer,"to be written =")) {
					BString s;
					s=strstr(buffer,"to be written =");
					s.RemoveFirst("to be written =");
					window1->view1->box2->SetCDSize((sz+atol(s.String())*2048));
					cdextra_datasize=atol(s.String())*2048;
					gotit=true;
				} // strstr			
			}
			snooze(10000);
		} while ((Thread==find_thread(MKISOFSTHREADNAME)) || (bytesread>0)/*|| (!gotit)*/);
/*
		bytesread=read(err,(void *)buffer, 10000);
		buffer[bytesread]=0;

		// output it to the log-window
		 mkisofs_output->AddText(buffer);

		if (bytesread>0) {
			if (strstr(buffer,"to be written =")) {
				BString s;
				s=strstr(buffer,"to be written =");
				s.RemoveFirst("to be written =");
				window1->view1->box2->SetCDSize((sz+atol(s.String())*2048));
				gotit=true;
			} // strstr			
		}
*/
		close(in);close(out);close(err);
			
		free(args);
	return true;
}

/*
 *	Thread function for data transport mkisofs <-> cdrecord
 */
typedef struct {
	int in;
	int out;
	int32 buffersize;
	bool *working;
} bufferfunc;
/*
int32 DataBuffer(void *data) {
	
	char *buffer(NULL);
	size_t written=0;
	size_t bytesread=10, byteswritten=0;
	bufferfunc *d=(bufferfunc *)data;
	buffer=(char *)malloc(d->buffersize);
	//int32 i=0;
	
	while ((d->working) && (written>=0) && (bytesread>0)) {
		bytesread=read(d->in,buffer, d->buffersize);
	//	if ((i%1000)==0) printf("HELIOSSTATUS: written: %ld bytesread: %ld\n", written, bytesread);
		if (bytesread>0) {
			written=write(d->out,buffer,bytesread);
			//if (written<=0) printf("HELIOS: ERROR 101\n");
			if (written>0) byteswritten+=written;
		}
//		snooze(10000);
		//i++;
	}
	
	free(buffer);
	printf("HELIOS: %ld bytes redirected.\n", byteswritten);
	return 0;
}
*/
int32 DataBuffer(void *data) {
	
	char *buffer1(NULL);
	size_t written=0;
	size_t bytesread1=1;
	bufferfunc *d=(bufferfunc *)data;
	buffer1=new char[d->buffersize];
printf("HELIOS: Buffer Thread Started.\n");
	while ((d->working) && (bytesread1 >0) && (written>=0)) {
		bytesread1=read(d->in,buffer1, d->buffersize);
		if (bytesread1>0)
			written=write(d->out,buffer1,bytesread1);
	}
	
	delete buffer1;
	printf("HELIOS: Exit Buffer Thread Without Errors.\n");
	return 0;
}


status_t Application1::WaitForDisc(const char *device) {
		if (!applicationCV->IsTimeout()) return B_OK;

		char 	info[4];
		int	timeout=applicationCV->GetTimeoutLength()*10;
		
		ShowStatus2(H->L("TWAITINGFORDISC"),0);
		while (DeviceStatus(device)!=E_NO_ERROR) {
			snooze(100000);
			timeout--;
			if ((timeout==0) || (window5->interrupted)) {
				HideStatus2();
				return B_TIMEOUT;
			}

			sprintf(info,"%d\"",timeout/10);
			SetStatus2Info((const char *)info);
			SetPercentage((float)timeout*100/((float)applicationCV->GetTimeoutLength()*10));
		}
		return B_OK;
}





void Application1::ReadOutputOfCdrecord(int outstream, thread_id thread, int tracks) {
	size_t bytesread=0;
	char *buffer=new char[65536];
	int speed=0;
	int lasttrack=0, track=1;
	float length=0;	// in MB
	float op=0,progressed=0;
	char info[40], info2[40];
	BString s;

	do {
		bytesread=0;
			//bytesread=read(mkisofs_err,buffer, 10000);
			//buffer[bytesread]=0;
			//mkisofs_output->AddText(buffer);
		bytesread=read(outstream,(void *)buffer, 65536);
		
		
		if (bytesread>0) {
			buffer[bytesread]=0;
		
			cdrecord_output->AddText(buffer);
			
			
			
			// get speed
			if (speed==0)
			if (strstr(buffer,"at speed")) {
				s=strstr(buffer,"at speed");
				s.Remove(0,8);
				s.Remove(s.FindFirst("in"),s.Length()-s.FindFirst("in"));
				speed=atoi(s.String());
				sprintf(info,"%dx (%0.1f MB/s)",speed,153600.0*(float)speed/(1024.0*1024.0));
				SetStatus2Info(info);
			}
			
			// Performing OPC...
			if (strstr(buffer,"Performing OPC...")) {
				SetStatus2(H->L("TPERFORMINGOPC"),-1);
			}
				// finished fixating...
			if (strstr(buffer,"Fixating time:")) {
				SetStatus2(H->L("TFINISHEDWRITING"),-1);
			}
				// start burning...
			if (strstr(buffer,"Starting new track")) {
				SetStatus2(H->L("TBURNING"),-1);
			}
				// Fixating...
			if (strstr(buffer,"Fixating...")) {
				SetStatus2(H->L("TFIXATING"),-1);
				SetPercentage(100.0);
			}
				// get track number
			if (strstr(buffer,"Track")) {
				s=strstr(buffer,"Track");
				s.RemoveFirst("Track");
				s.Remove(s.FindFirst(":"),s.Length()-s.FindFirst(":"));
				track=atoi(s.String());
			}	
				
			if (lasttrack!=track) {
				lasttrack=track;
				op=0;
				progressed=0;
			}
				// get track length
			//if (length==0)
			if (strstr(buffer,"of")) {
				s=strstr(buffer,"of");
				s.RemoveFirst("of");
				s.Remove(s.FindFirst("MB"),s.Length()-s.FindFirst("MB"));
				length=atof(s.String());
			}		

			// get write position
			if (strstr(buffer,"Track")) {
				s=strstr(buffer,"Track");
				s.Remove(0,9);
				s.Remove(s.FindFirst("of"),s.Length()-s.FindFirst("of"));
				progressed=atof(s.String());
				if (op<progressed) {
					op=progressed;
					if (length>0) {
						sprintf(info,"%0.1f%%",100.0*(float)progressed/(float)length);
						sprintf(info2,"%s %d/%d",H->L("TTRACK"),track, tracks);
						SetStatus2Info(info, info2);
						SetPercentage(100.0*(float)progressed/(float)length);
					}
				}
			}		
		}
		snooze(250000);
	}while (thread==find_thread(CDRECORDTHREADNAME));
	
	delete [] buffer;
}	









status_t Application1::MakeImage()
{
		int arguments=0;
		static thread_id buffer_Thread=-1, mkisofs_Thread=-1, cdrecord_Thread=-1;
		const char **args;
		BString str="";
		BString device="dev=";
		BString filesystem="";
		bool working;
		bufferfunc	bf;
		int mkisofs_in, mkisofs_out, mkisofs_err, cdrecord_in, cdrecord_out, cdrecord_err;
		BString	lastarg;

		window5->interrupted=false;
		if (find_thread(CDRECORDTHREADNAME)==cdrecord_Thread) kill_thread(cdrecord_Thread);
		if (find_thread(BUFFERTHREADNAME)==buffer_Thread) kill_thread(buffer_Thread);
		if (find_thread(MKISOFSTHREADNAME)==mkisofs_Thread) kill_thread(mkisofs_Thread);

		// wait until a disc is in the drive and drive is ready
		if (WaitForDisc(deviceCV->GetWriterDevicePath())==B_TIMEOUT) {
			errtype=H->L("TTIMEOUTTYPE");
			errstring=H->L("TTIMEOUTERROR");
			HideStatus2();
			return B_ERROR;
		}
			

		//									//
		//   begin to produce the options for mkisofs...	//
		//									//

		//GetImageSize();
		if (window1->Lock()) {
			window1->view1->box2->Refresh();
			window1->Unlock();
		}
		device << deviceCV->GetWriterDeviceNumber();
		args = (const char **)malloc(sizeof(char *) * (50));
		// Make MKISOFS options
		
		if (MKISOFS_CommandLine(args, &arguments)!=B_OK) {
			HideStatus2();
			free(args);
			return B_ERROR;
		}
	
	
//		lastarg=args[--arguments];
//		
//		// multisession disc selected???
//		if (window1->view1->IsMultisession()) {
//			// ask whether this session should be the first, last or another
//			// session of the disc
//			ErrorBox *alert=new ErrorBox(E_ORANGE_COLOR, "",H->L("MULTISESSION_ALERT"),H->L("MULTISESSION_ALERT_B0"),H->L("MULTISESSION_ALERT_B1"),H->L("MULTISESSION_ALERT_B2"));
//			int selected=alert->Go();
//		
//			multi_flag=true;
//			if (selected!=2)
//				if (GetMultisessionSectors(start_sec, end_sec)!=B_OK) {
//					errtype=H->L("TERROR");
//					errstring=H->L("MULTISESSION_FAILS_ALERT");
//					free(args);
//					HideStatus2();
//					return B_ERROR;
//				}
//		
//			if (selected==0) {		// it is another session!
//				if (window1->view1->CDType()!=CDEXTRA_INDEX) {
//					args[arguments++] = strdup("-M");
//					args[arguments++] = strdup(deviceCV->GetWriterDeviceNumber());
//				}
//				args[arguments++] = strdup("-C");
//				filesystem << start_sec << "," << end_sec;
//				args[arguments++] = strdup(filesystem.String());
//			}
//
//			if (selected==1) {		// it is the last session!
//				if (window1->view1->CDType()!=CDEXTRA_INDEX) {
//					args[arguments++] = strdup("-M");
//					args[arguments++] = strdup(deviceCV->GetWriterDeviceNumber());
//				}
//				args[arguments++] = strdup("-C");
//				filesystem << start_sec << "," << end_sec;
//				args[arguments++] = strdup(filesystem.String());
//				// leave the "-multi"-option for cdrecord!
//				multi_flag=false;
//			}
//		
//		}
//		args[arguments++]=strdup(base_folder.String());
//		args[arguments]=NULL;
		printArguments("MKISOFS Arguments",args);


		// launch mkisofs thread now
		mkisofs_Thread=pipe_command(arguments, args, mkisofs_in, mkisofs_out, mkisofs_err);
		set_thread_priority(mkisofs_Thread,MKISOFS_PRIORITY);
		if (mkisofs_Thread<0) {
			errtype= H->L("TERROR");
			errstring="mkisofs\n\n";
			errstring << H->L("TCOULDNOTLOAD");
			free(args);
			HideStatus2();
			return B_ERROR;
		}
		SetStatus2(H->L("TWAITING"),mkisofs_Thread);
		rename_thread(mkisofs_Thread,MKISOFSTHREADNAME);
		// Make CDRECORD options
		arguments=0;
		
		CDRECORD_CommandLine(args, &arguments, window1->view1->IsMultisession());
		
//		args[arguments++] = strdup(CM->GetString(PATH_TO_CDRECORD)->String());
//		args[arguments++] = strdup(device.String());
//		if (heliosCV->IsBurnproof()) args[arguments++] = strdup("driveropts=burnproof");
//		args[arguments++] = strdup("-v");
//		if (heliosCV->IsOverburning()) args[arguments++] = strdup("-overburn");
//		// set fifo-buffer size
//		
//		if (heliosCV->GetFIFOSize()!=4) {
//			str.SetTo("fs=");
//			str << heliosCV->GetFIFOSize() << "M";
//			args[arguments++] = strdup(str.String());
//		}
//		// wait for input stream
//		args[arguments++] = strdup("-waiti");
////		args[arguments++] = strdup("-speed=");
//		args[arguments++] =strdup("-speed=");
//		str.SetTo("");
//		str << heliosCV->GetWriterSpeed();
//		args[arguments++] = strdup(str.String());
//		if (!window1->view1->IsMultisession())
//			if (!heliosCV->IsFixating()) args[arguments++] = strdup("-nofix");
//		if (multi_flag) args[arguments++] = strdup("-multi");
//		if (heliosCV->IsSimulation()) args[arguments++] = strdup("-dummy");
//		if (heliosCV->IsDAO()) args[arguments++] = strdup("-dao");

		args[arguments++] = strdup("tsize=");
		str.SetTo("");
		if (window1->view1->CDType()==CDEXTRA_INDEX)
			str << cdextra_datasize;
		else
			str << window1->view1->box2->GetCDSize();
//		args[arguments++] = strdup("tsize=");
		args[arguments++] = strdup(str.String());
		args[arguments++] = strdup("-data");
		args[arguments++] = strdup("-");
		args[arguments]=NULL;
		
		printArguments("cdrecord streaming args",args);

		// launch cdrecord thread now
		cdrecord_Thread=pipe_command(arguments, args, cdrecord_in, cdrecord_out, cdrecord_err);
		set_thread_priority(cdrecord_Thread,CDRECORD_PRIORITY);
		if (cdrecord_Thread<0) {
			errtype=H->L("TERROR");
			errstring="cdrecord\n\n";
			errstring << H->L("TCOULDNOTLOAD");
			free(args);
			HideStatus2();
			return B_ERROR;
		}
		SetStatus2(H->L("TWAITING"),cdrecord_Thread);
		rename_thread(cdrecord_Thread,CDRECORDTHREADNAME);
		working=true;
		bf.working=&working;
		bf.buffersize=4*1024;
		bf.in=mkisofs_out;
		bf.out=cdrecord_in;
		
		buffer_Thread=spawn_thread(DataBuffer,BUFFERTHREADNAME,BUFFER_PRIORITY,(void *)(&bf));
		resume_thread(buffer_Thread);	
		resume_thread(mkisofs_Thread);	
		resume_thread(cdrecord_Thread);	
	
		mkisofs_output->Start(mkisofs_err);
		
	
		ReadOutputOfCdrecord(cdrecord_out, cdrecord_Thread);		
		
		
		working=false;
		bf.working=&working;
		HideStatus2();
		mkisofs_output->Stop();
		//ioctl(mkisofs_out, 26 , cdrecord_in);
		if (find_thread(BUFFERTHREADNAME)==buffer_Thread) kill_thread(buffer_Thread);
		if (find_thread(MKISOFSTHREADNAME)==mkisofs_Thread) kill_thread(mkisofs_Thread);
		close(mkisofs_in);close(mkisofs_out);close(mkisofs_err);
		close(cdrecord_in);close(cdrecord_out);close(cdrecord_err);
		snooze(10000);
//		kill_thread(reader_Thread);
//		kill_thread(writer_Thread);
		if (window5->interrupted) {
			errtype=H->L("TERROR");
			errstring=H->L("TINTERRUPTEDBYUSER");
			free(args);
			return B_ERROR;
		}
		if (deviceCV->IsWriterEjected())
			eject_media(deviceCV->GetWriterDevicePath());
		free(args);
		return B_OK;
}


status_t Application1::MakeImage_NOSTREAMING()
{
		BString temp=*CM->GetString(TEMP_PATH);
		status_t	retvalue=B_ERROR;
		AppendTrailingSlash(&temp);
		temp.Append(pathCV->GetDefaultImageName());
		switch(window1->view1->CDType()) {
			case DATACD_INDEX: {
				BList	*list=window1->view1->datapanel->GetFilesystem();
				if (list->CountItems()==1)
					if (((BString *)list->ItemAt(0))->Compare("BFS")==0) {
						retvalue=MakeBFSImage(temp.String());
						for (int i=list->CountItems()-1; i>=0; i--)
							delete ((BString *)list->RemoveItem(i));
						delete list;
						break;
					}
				
				retvalue=MakeMKISOImage(temp.String());
				for (int i=list->CountItems()-1; i>=0; i--)
					delete ((BString *)list->RemoveItem(i));
				delete list;
				break;
			}
		}

		if (retvalue==B_OK)
			retvalue=BurnImage(temp.String());
		return retvalue;
}



int Application1::ReadAudioCD() {
		const char **args;
		int arguments=0;
		thread_id Thread;
		BString str="";
		int in, out, err;
				
		if (WaitForDisc(deviceCV->GetReaderDevicePath())==B_TIMEOUT) {
			errtype=H->L("TTIMEOUTTYPE");
			errstring=H->L("TTIMEOUTERROR");
			HideStatus2();
			return B_ERROR;
		}

		// allocate some memory for the command line array
		args = (const char **)malloc(sizeof(char *) * (23));

		// fill array with arguments
		args[arguments++] = strdup(CM->GetString(PATH_TO_CDDA2WAV)->String());
		args[arguments++] = strdup("-D");
		args[arguments++] = strdup(deviceCV->GetReaderDeviceNumber());
		args[arguments++] = strdup("-x");
		args[arguments++] = strdup("-B");
		args[arguments++] = strdup("-g");
		args[arguments++] = strdup("-t");
		args[arguments++] = strdup("1");
		str=base_folder;
		str.Append("Track");
		args[arguments++] = strdup(str.String());
		args[arguments] = NULL;


		// and _please_ launch a new thread!
		Thread=pipe_command(arguments, args, in, out, err);
		set_thread_priority(Thread,CDDA2WAV_PRIORITY);
		if (Thread<0) {
			errtype= H->L("TERROR");
			errstring="cdda2wav\n\n";
			errstring << H->L("TCOULDNOTLOAD");
			free(args);
			HideStatus2();
			return -1;
		}
		ShowStatus2(H->L("TREADINGAUDIO"),Thread);
		rename_thread(Thread,CDDA2WAVTHREADNAME);
		resume_thread(Thread);
		
		size_t bytesread=0;
		char buffer[10000];
		char info[40], info2[40];
		float percentage, oldpercentage=1000;
		BString	per="";
		int	trackno=0;
		
		while (Thread==find_thread(CDDA2WAVTHREADNAME)) {
			bytesread=read(err,(void *)buffer, 1000);
			buffer[bytesread]=0;
			if (bytesread>0) {
				if (strstr(buffer,"%")) {
					per=buffer;
					per.Remove(per.FindFirst("%"), per.Length()-per.FindFirst("%"));
					while (per.Length()>3) per.Remove(0,1);
					percentage=atof(per.String());
					if (percentage<oldpercentage) {
						trackno++;
						if (trackno>1) {
							BString	path, leaf="Track_";
							off_t size;			
				
							leaf << ((trackno-1<10)?"0":"") << trackno-1 << ".wav";
							path=base_folder;
							path.Append(leaf);
		
							TrackLength *info=new TrackLength(new BEntry(path.String(), true));
							size=info->GetBytes();
							if (size>0) {
								ShowFolder(true);
								if (window1->Lock()) {
									//window1->view1->listview1->AddRow(new AudioRow(window1->view1->listview1->iCountItems()+1, leaf.String(), 0, size, size/176400));
									window1->Unlock();
								}
							}
							delete info;
						}						
					}
					sprintf(info,"%0.0f%%", percentage);
					sprintf(info2,"%s %d",H->L("TTRACK"), trackno);
					SetStatus2Info(info, info2);
					SetPercentage(percentage);
					oldpercentage=percentage;
				} // strstr			
			}
			snooze(250000);
		}
		close(in);close(out);close(err);
		ShowFolder(true);
		HideStatus2();
		if (window5->interrupted) {
			errtype=H->L("TERROR");
			errstring=H->L("TINTERRUPTEDBYUSER");
			free(args);
			return -1;
		}
		if (deviceCV->IsReaderEjected())
			eject_media(deviceCV->GetReaderDevicePath());
			
		free(args);
		return trackno;
}



status_t Application1::BurnImage(BString filename) {
		int arguments=0;
		thread_id Thread;
		const char **args;
		BString str="";
		BString device="dev=";
		BString filesystem="";
		BString fixate="";
		int in, out, err;

		// check if file exists
		if (!FileExists(filename.String())) {
			errtype=H->L("TERROR");
			errstring=H->L("TNOTEXIST_ALERT");
			return B_ERROR;
		}

		if (window5->Lock()) {
			if (heliosCV->IsSimulation())
				window5->SetTitle(H->L("TSTATUSWINDOWTITLESIMULATION"));
			else
				window5->SetTitle(H->L("TSTATUSWINDOWTITLE"));
			window5->Unlock();
		}

		ShowStatus2(H->L("TFLUSHINGBUFFERS"), 0);
		sync();
		ShowStatus2(H->L("TWAITINGFORDISC"),0);
		
		// Check for image file size. Does it fit?		
		if (window1->Lock()) {
			BFile	*file=new BFile(filename.String(), B_READ_ONLY);
			if (file->InitCheck()==B_OK) {
				off_t	filesize;

				file->GetSize(&filesize);
				window1->view1->box2->SetCDSize(filesize);
				file->Unset();
				if (!CheckCDSize()) {
					window1->Unlock();
					errtype=H->L("TERROR");
					errstring=H->L("TOVERSIZED");
					delete file;
					return B_ERROR;
				}
			}
			delete file;
			window1->Unlock();
		}

		if (WaitForDisc(deviceCV->GetWriterDevicePath())==B_TIMEOUT) {
			errtype=H->L("TTIMEOUTTYPE");
			errstring=H->L("TTIMEOUTERROR");
			HideStatus2();
			return B_ERROR;
		}

		device << deviceCV->GetWriterDeviceNumber();
		str="";
		args = (const char **)malloc(sizeof(char *) * (22));
		CDRECORD_CommandLine(args, &arguments, window1->view1->IsMultisession());
//		args[arguments++] = strdup(CM->GetString(PATH_TO_CDRECORD)->String());
//		args[arguments++] = strdup(device.String());
//		// set fifo-buffer size
//		str.SetTo("fs=");
//		str << heliosCV->GetFIFOSize() << "M";
//		args[arguments++] = strdup(str.String());
//		// set writer speed
//		device.SetTo("-speed=");
//		device << heliosCV->GetWriterSpeed();
//		args[arguments++] = strdup(device.String());
//		if (heliosCV->IsBurnproof()) args[arguments++]=strdup("driveropts=burnproof");
//		if (!heliosCV->IsFixating()) args[arguments++] = strdup("-nofix");
//		if (heliosCV->IsOverburning()) args[arguments++] = strdup("-overburn");
//		if (heliosCV->IsDAO()) args[arguments++] = strdup("-dao");
//		if (heliosCV->IsSimulation()) args[arguments++] = strdup("-dummy");
//		if (window1->view1->IsMultisession()) args[arguments++] = strdup("-multi");
		args[arguments++] = strdup("-data");
//		args[arguments++] = strdup("-v");
		args[arguments++] = strdup(filename.String());
		args[arguments]=NULL;
		
		printArguments("cdrecord non-streaming args",args);
		
		
		
		Thread=pipe_command(arguments, args, in, out, err);
		set_thread_priority(Thread,CDRECORD_PRIORITY);
		if (Thread<0) {
			errtype=H->L("TERROR");
			errstring="cdrecord\n\n";
			errstring<<H->L("TCOULDNOTLOAD");
			HideStatus2();
			free(args);
			return B_ERROR;
		}
		SetStatus2(H->L("TWAITING"),Thread);
		rename_thread(Thread,CDRECORDTHREADNAME);
		resume_thread(Thread);
		
		ReadOutputOfCdrecord(out, Thread);		


		close(in);close(out);close(err);
		window5->Lock();
			window5->SetTitle(H->L("TSTATUSWINDOWTITLE"));
		window5->Unlock();
		HideStatus2();
		free(args);

		if (window5->interrupted) {
			errtype=H->L("TERROR");
			errstring=H->L("TINTERRUPTEDBYUSER");
			return B_ERROR;
		}
			
		if (deviceCV->IsWriterEjected())
			eject_media(deviceCV->GetWriterDevicePath());
		
		return B_OK;
}









void Application1::ShowStatus(const char *text)
{
		if (statusSW->Lock()) {
			statusSW->SetText(text);
			statusSW->MoveTo(window1->Frame().left+(window1->Bounds().Width()-statusSW->Bounds().Width())/2,
						window1->Frame().top+(window1->Bounds().Height()-statusSW->Bounds().Height())/2);
			if (statusSW->IsHidden()) statusSW->Show();
			statusSW->Unlock();
		}
}


void Application1::SetStatus(const char *text)
{
		if (statusSW->Lock()) {
			statusSW->SetText(text);
			statusSW->Unlock();
		}
}



void Application1::HideStatus()
{
		if (statusSW->Lock()) {
			if (!statusSW->IsHidden()) statusSW->Hide();
			statusSW->Unlock();
		}
}




void Application1::ShowStatus2(const char *text, thread_id Thread)
{
		if (window5->Lock()) {
			window5->Thread=Thread;
			window5->EnableControls();
			window5->interrupted=false;
			window5->view5->stringview12->SetText(text);
			window5->MoveTo(window1->Frame().left+(window1->Bounds().Width()-window5->Bounds().Width())/2,
						window1->Frame().top+(window1->Bounds().Height()-window5->Bounds().Height())/2);
			window5->SetWorkspaces(B_CURRENT_WORKSPACE);
			window5->view5->progressSB->Reset();
			if (window5->IsHidden()) window5->Show();
			window5->Unlock();
		}
}


void Application1::SetStatus2(const char *text, thread_id Thread)
{
		if (window5->Lock()) {
			if (Thread!=-1) {
				window5->Thread=Thread;
				window5->view5->progressSB->Reset();
			}
			window5->EnableControls();
			window5->interrupted=false;
			window5->view5->stringview12->SetText(text);
			if (window5->IsHidden()) window5->Show();
			window5->Unlock();
		}
}



void Application1::HideStatus2()
{
		if (window5->Lock()) {
			if (!window5->IsHidden()) window5->Hide();
			window5->Unlock();
		}
}


void Application1::SetStatus2Info(const char *text, const char *leading)
{
		BString s=text;
		if (window5->Lock()) {
			if (s.Compare(window5->view5->progressSB->TrailingText())!=0)
			window5->view5->progressSB->SetTrailingText(text);
			s=leading;
			if (s.Compare(window5->view5->progressSB->Text())!=0)
			window5->view5->progressSB->SetText(leading);
			window5->Unlock();
		}
}


void Application1::SetPercentage(float percentage) {
	if (window5->Lock()) {
		if (percentage>100) percentage=100;
		if (percentage<0) percentage=0;
		window5->view5->progressSB->Update(percentage-window5->view5->progressSB->CurrentValue());
		window5->Unlock();
	}
}


// returns always true (very important function ;-)
bool Application1::OnlyFiles() {
	return true;
}


void Application1::ChangePauses()
{
float pause=0;

window1->Lock();
pause=atof(window1->view1->textcontrol2->Text());
if (pause>4800) pause=4800;
if (pause<0) pause=0;
//sprintf(bla,"%.2f",pause);
for (int i=0; i<window1->view1->listview1->iCountItems();i++)
			{
			AudioRow *si=(AudioRow *)window1->view1->listview1->RowAt(i);
			if (window1->view1->listview1->IsSelected(i))
				{
				si->SetPregap(pause);
				//window1->view1->listview1->InvalidateRow(i);
				//si->Select();
				}
			}
window1->view1->textcontrol2->SetText("");
//window1->view1->textcontrol2->SetText(bla);
//window1->view1->textcontrol2->MakeFocus(false);
window1->Unlock();
}

void Application1::DoAudioMenu() {
	window1->Lock();

	while (window1->view1->listview1->cmenu->CountItems()>0)
		window1->view1->listview1->cmenu->RemoveItem(0L);
	window1->view1->listview1->cmenu->AddItem(window1->view1->listview1->deleteItem);
	window1->view1->listview1->cmenu->AddSeparatorItem();
	window1->view1->listview1->cmenu->AddItem(window1->view1->listview1->moveupItem);
	window1->view1->listview1->cmenu->AddItem(window1->view1->listview1->movedownItem);
	window1->view1->listview1->cmenu->AddSeparatorItem();
	window1->view1->listview1->cmenu->AddItem(window1->view1->listview1->indexItem);
	window1->view1->listview1->cmenu->AddItem(window1->view1->listview1->splitItem);
//	window1->view1->listview1->cmenu->AddSeparatorItem();
//	window1->view1->listview1->cmenu->AddItem(window1->view1->listview1->CDTEXTTitleItem);
	window1->view1->listview1->cmenu->SetTargetForItems(be_app_messenger);
	window1->Unlock();
}


void Application1::DoDataMenu() {

	if (window1->view1->listview1->cmenu==NULL) return;
	window1->Lock();

	while (window1->view1->listview1->cmenu->CountItems()>0)
		window1->view1->listview1->cmenu->RemoveItem(0L);
	window1->view1->listview1->cmenu->AddItem(window1->view1->listview1->deleteItem);
	window1->view1->listview1->cmenu->AddItem(window1->view1->listview1->newfolderItem);
	window1->view1->listview1->cmenu->AddItem(window1->view1->listview1->renamefileItem);
	window1->view1->listview1->cmenu->SetTargetForItems(be_app_messenger);
	window1->Unlock();
}



void Application1::Update_DATA_AUDIO()
{

	if (window1->view1->listview1->cmenu==NULL) return;
	window1->Lock();
	//window1->view1->stringview1->Clear();
		if ((window1->view1->CDType()==DATACD_INDEX) ||
			(window1->view1->CDType()==BOOTABLECD_INDEX) ||
			(window1->view1->CDType()==DVDVIDEO_INDEX))
			// auf DATA
			{
			window1->view1->textcontrol2->SetEnabled(false);
			//window1->view1->textcontrol1->SetEnabled(true);
			window1->view1->export_M3U->SetEnabled(false);
			window1->view1->export_APML->SetEnabled(false);
			//window1->view1->menufield2->SetEnabled(true);
			DoDataMenu();
			} else
			// auf AUDIO
		if ((window1->view1->CDType()==AUDIOCD_INDEX) || (window1->view1->CDType()==CDEXTRA_INDEX))
			{
			current_folder=base_folder;
			window1->view1->textcontrol2->SetEnabled(true);
			//window1->view1->textcontrol1->SetEnabled(false);
			window1->view1->export_M3U->SetEnabled(true);
			window1->view1->export_APML->SetEnabled(true);
			//window1->view1->menufield2->SetEnabled(false);
			DoAudioMenu();
			}
	window1->view1->listview1->SetListMode(window1->view1->CDType());
	window1->Unlock();
}









void  OutPutRAW(const char *filename, int fd) {
	BFile *infile=new BFile();
	char *buffer(NULL);
	const int buffersize=4*1024*1024;
	int64 framesdecoded=0;

	printf("HELIOS: writing RAW audio: %s\n", filename);
	buffer=(char *)malloc(buffersize);
	infile->SetTo(filename, B_READ_ONLY);
	do {
		framesdecoded=infile->Read((void *)buffer, buffersize);
		write(fd, &buffer,framesdecoded);
	} while (framesdecoded>0);
	free(buffer);
	infile->Unset();
	delete infile;
}





bigtime_t xyround(float f) {

	if ((float)(f-(bigtime_t)f)>0.000F) return ((bigtime_t)f)+1; else return (bigtime_t)f;
}





// Decodes compressed audio data to raw audio
bool ReadData(BMediaFile *mfile, int fd, int64 start, int64 end) {
	media_multi_audio_format raw;
	int64 framesRead;
	media_format mf;
	BMediaTrack *track;
	int64 j=0;
	int64 length=0;
	const int buffersize=2*1024*1024;
	char *audioData(NULL);

	if (mfile->CountTracks()!=1) return false;
	track=mfile->TrackAt(0);
		if (track->InitCheck()!=B_OK) return false;
	if (track->EncodedFormat(&mf)==B_OK)
		printf("HELIOS: audio input is %0.0fHz/%ld channels\n", mf.u.raw_audio.frame_rate, mf.u.raw_audio.channel_count);
	mf.type=B_MEDIA_RAW_AUDIO;
	mf.u.raw_audio.frame_rate=44100.0F;					// 44.1kHz
	mf.u.raw_audio.channel_count=2;						// stereo
	mf.u.raw_audio.format=raw.B_AUDIO_SHORT; 				// 16 bit
	mf.u.raw_audio.byte_order=B_MEDIA_LITTLE_ENDIAN;			// Intel
	mf.u.raw_audio.buffer_size=buffersize;
	if (track->DecodedFormat(&mf)==B_OK)
		printf("HELIOS: audio output is %0.0fHz/%ld channels\n", mf.u.raw_audio.frame_rate, mf.u.raw_audio.channel_count);

	length=2352LL*(end-start);
	//printf("PREBUFFER: %lld/%lld\n", j, length);
	
	audioData = (char*)malloc(buffersize);
	framesRead=1;
	track->SeekToFrame(&start);
	while ((j<length) && (framesRead>0)) {
		track->ReadFrames(audioData, &framesRead);
		if (framesRead*4>length-j) framesRead=(length-j)/4;
		if (framesRead>0)
			write(fd, audioData,framesRead*4); 
		j+=framesRead*4;
		//printf("BUFFER: %lld/%lld\n", j, length);
	}
	for (int i=0; i<buffersize;i++) audioData[i]=0;
	while (j<length) {
		if (length-j>buffersize) framesRead=buffersize; else framesRead=length-j;
		write(fd, audioData,framesRead);
		j+=framesRead;
	}
	mfile->ReleaseAllTracks();
	free(audioData);
	return true;
}

// opens the media file
// returns true if successful, false if not
bool OpenFile(const char *filename, int fd, int64 start, int64 end) {
	entry_ref 	ref;
	BMediaFile 	*mfile;
	BEntry		*file=new BEntry(filename, true);
	bool		ok;
	
	if (file->InitCheck()!=B_OK) {delete file; return false;}
	if (file->GetRef(&ref)!=B_OK) {file->Unset();delete file; return false;}
	file->Unset();
	delete file; 
	mfile=new BMediaFile(&ref); 
	if (mfile->InitCheck()!=B_OK) 
		{
		OutPutRAW(filename, fd);
		delete mfile;
		return true;
		}
	ok=ReadData(mfile, fd, start, end);
	delete mfile;
	return ok;
}



typedef struct 
	audio_parameters {
		char 	*filename[99];
		int	trackcount;
		int	fd;
		int64	start[99];
		int64	end[99];
};

/*
 *	The following thread-function steps through the file list to process all items
 */
int32 AudioBufferProducer(void *data) {
	audio_parameters	*ap=(audio_parameters *)data;
	
	for (int i=0; i<ap->trackcount; i++) {
		printf("HELIOS filename[%d/%d]: %s start: %lld stop: %lld\n", i,ap->trackcount, ap->filename[i], ap->start[i], ap->end[i]);
		OpenFile(ap->filename[i],ap->fd, ap->start[i], ap->end[i]);
	}
	return 0;
}







status_t Application1::BurnAudio(bool multisession=false)
{
		int arguments=0;
		thread_id buffer_Thread, cdrecord_Thread;
		const char **args;
		BString str="";
		BString toraw="";
		BString temp="";
		BString device="dev=";
		BString endstr="";
		BString fixate="";
		BString bproof="";
//		BEntry file;
		audio_parameters	ap;
		int  cdrecord_in, cdrecord_out, cdrecord_err;

		// wait till a disc is in the drive and drive is ready
		if (WaitForDisc(deviceCV->GetWriterDevicePath())==B_TIMEOUT) {
			errtype=H->L("TTIMEOUTTYPE");
			errstring=H->L("TTIMEOUTERROR");
			HideStatus2();
			return B_ERROR;
		}

		args = (const char **)malloc(sizeof(char *) * (700));

		// Make CDRECORD options
		device << deviceCV->GetWriterDeviceNumber();
		arguments=0;
		args[arguments++] = strdup(CM->GetString(PATH_TO_CDRECORD)->String());
		args[arguments++] = strdup(device.String());
		if (heliosCV->IsBurnproof()) args[arguments++] = strdup("driveropts=burnproof");
		args[arguments++] = strdup("-v");
		if (heliosCV->IsOverburning()) args[arguments++] = strdup("-overburn");
		args[arguments++] = strdup("-waiti");
		// set fifo-buffer size
		str.SetTo("fs=");
		str << heliosCV->GetFIFOSize() << "M";
		args[arguments++] = strdup(str.String());
		if (heliosCV->IsDAO()) args[arguments++] = strdup("-dao");
		str.SetTo("speed=");
		str << heliosCV->GetWriterSpeed();
		args[arguments++] = strdup(str.String());
		if (!heliosCV->IsFixating()) args[arguments++] = strdup("-nofix");
		if (heliosCV->IsSimulation()) args[arguments++] = strdup("-dummy");
		if (multisession) args[arguments++] = strdup("-multi");

		


	//	str.SetTo("mcn=");
	//	str << real_time_clock();
	//	args[arguments++] = strdup(str.String());
		//richtige reihenfolge und parameter für einzelne tracks
	
		if (window1->view1->WriteCDText()) {
			BString		cdt_path=base_folder;
			char		albumtitle[161];
		
			cdt_path << "cdtext.data";
			CDTextFile	*cdt=new CDTextFile(cdt_path.String(), B_READ_WRITE | B_CREATE_FILE);
		
			switch(window1->view1->CDType()) {
				case AUDIOCD_INDEX: {
					sprintf(albumtitle, "%s",window1->view1->GetCDAlbum());
					break;
				}
				case CDEXTRA_INDEX: {
					sprintf(albumtitle, "%s",window1->view1->GetCDAlbumCDExtra());
					break;
				}
				default: {
					albumtitle[0]=0;
					break;
				}
			}
			if (strlen(albumtitle)>0)
				cdt->SetAlbum(albumtitle);
			for (int i=0; i<window1->view1->listview1->iCountItems();i++) {
				AudioRow *si=(AudioRow *)window1->view1->listview1->RowAt(i);
				if (strlen(si->GetCDTitle())>0)
					cdt->SetTrack(i+1, si->GetCDTitle());
			}
		
			cdt->SetCharacterCode(0x00);
			cdt->SetLanguage(0x09);	// english
			cdt->SetTracks(window1->view1->listview1->CountRows());
			cdt->FlushAll();
			delete cdt;
	
			str.SetTo("textfile=");
			str << cdt_path.String();
			args[arguments++] = strdup(str.String());
			args[arguments++] = strdup("-text");
		}
		
		
		if (window1->view1->WriteCDText()) {
			//args[arguments++] = strdup("-text");
			//args[arguments++] = strdup("-useinfo");
			SaveCDTEXTAlbum();
		}
		for (int i=0; i<window1->view1->listview1->iCountItems();i++)
			{
			AudioRow *si=(AudioRow *)window1->view1->listview1->RowAt(i);
			str.SetTo("pregap=");
			str << /*(i==0)?(150):*/((int)(si->GetPregap()));
			args[arguments++] = strdup(str.String());

			args[arguments++] = strdup("-swab");
			args[arguments++] = strdup("-audio");
			args[arguments++] = strdup("-pad");
			
			/*if (window1->view1->WriteCDText()) {
				str.SetTo("textfile=");
				str << GenerateInfFilename(i);
				args[arguments++] = strdup(str.String());
			}*/
			
//			args[arguments++] = strdup("textfile=");
//			args[arguments++] = strdup("-useinfo");
			str.SetTo("tsize=");

			str << si->GetBytes();
			args[arguments++] = strdup(str.String());
			if (strlen(si->GetCdrecordReadyList())>1) {
				str.SetTo("index=");
				str.Append(si->GetCdrecordReadyList());
				args[arguments++] = strdup(str.String());
			}
			
			args[arguments++] = strdup("-");
			}
		args[arguments]=NULL;
	
		printArguments("cdrecord non-streaming args",args);
		//SaveCDTEXTAlbum();

		// launch cdrecord thread now
		cdrecord_Thread=pipe_command(arguments, args, cdrecord_in, cdrecord_out, cdrecord_err);
		if (cdrecord_Thread<0) {
			errtype=H->L("TERROR");
			errstring="cdrecord\n\n";
			errstring << H->L("TCOULDNOTLOAD");
			free(args);
			HideStatus2();
			return B_ERROR;
		}
		SetStatus2(H->L("TWAITING"),cdrecord_Thread);
		set_thread_priority(cdrecord_Thread,CDRECORD_PRIORITY);
		rename_thread(cdrecord_Thread,CDRECORDTHREADNAME);
//		working=true;
//		bf.working=&working;
//		bf.buffersize=4096000;
//		bf.in=toraw_out;
//		bf.out=cdrecord_in;
		
//		//Parameter für "toRAW" erstellen
		arguments=0;
		AudioRow *si;
		
//		toraw="";
//		toraw.Append(base_folder).append("cdtextfile.cdt");
//		CDTextFile *textfile=new CDTextFile(new BEntry(toraw.String()));
//		textfile->SetAlbum("");
		
		
		
		for (int i=0; i<window1->view1->listview1->iCountItems();i++)
			{
			si=(AudioRow *)window1->view1->listview1->RowAt(i);
			toraw.SetTo(base_folder);
			toraw.Append(si->GetFilename());
			if (!si->IsSplitted())
				ap.start[arguments]=si->GetStartFrame();
				ap.end[arguments]=si->GetEndFrame();
				ap.filename[arguments++] = strdup(toraw.String());
			}
		ap.trackcount=arguments;
		ap.fd=cdrecord_in;

	
	

		// Check files
		BEntry		entry;
		entry_ref	ref;
		BMediaFile	*mfile;
		BString		st;
	
		for (int i=0; i<ap.trackcount; i++) {
			entry.SetTo(ap.filename[i], true);
			entry.GetRef(&ref);
			mfile=new BMediaFile(&ref); 
			char nam[B_FILE_NAME_LENGTH];
			entry.GetName(nam);
			st=nam;
			printf("name=%s\n", ap.filename[i]);
			if ((entry.InitCheck()!=B_OK) || (mfile->InitCheck()!=B_OK)) {
				if (st.Length()>40) st.Remove(39, st.Length()-40);
				st << "\n\n" << H->L("TAUDIOFILEERROR");
				ErrorBox	*eb=new ErrorBox(E_RED_COLOR, H->L("TERROR"), st.String(), H->L("TABORT"), H->L("TCONTINUE"));
				PlayErrorSound();
				if (eb->Go()==0) {
					delete mfile;
					entry.Unset();
					kill_thread(cdrecord_Thread);
					free(args);
					HideStatus2();
					errtype=H->L("TERROR");
					errstring=H->L("TINTERRUPTEDBYUSER");
					return B_ERROR;
				}
			} else {
				int error=0;
				media_format mf;
				BMediaTrack *track;
				ErrorBox *eb=NULL;

				if (mfile->CountTracks()==1) {
					track=mfile->TrackAt(0);
					if (track->InitCheck()==B_OK) {
						if (track->EncodedFormat(&mf)==B_OK) {
							//printf("HELIOS: audio input is %0.0fHz/%ld channels\n", mf.u.raw_audio.frame_rate, mf.u.raw_audio.channel_count);
							if (mf.u.raw_audio.frame_rate==44100) {
								
							} else error=-1;
						} else error=-2;
					} else error=-3;
				} else error=-4;
				
				if (error<0) {
				
					switch(error) {
						case -1: {
							st << "\n\n" << H->L("TFRAMERATEERROR");
							break;
						}
						case -2: {
							st << "\n\n" << H->L("TFORMATREADERROR");
							break;
						}
						case -3: {
							st << "\n\n" << H->L("TTRACKREADERROR");
							break;
						}
						case -4: {
							st << "\n\n" << H->L("TTRACKNUMBERERROR");
							break;
						}
						default: break;
					}
					eb=new ErrorBox(E_RED_COLOR, H->L("TERROR"), st.String(), H->L("TABORT"), H->L("TCONTINUE"));
					PlayErrorSound();
					if (eb->Go()==0) {
						delete mfile;
						entry.Unset();
						kill_thread(cdrecord_Thread);
						free(args);
						HideStatus2();
						errtype=H->L("TERROR");
						errstring=H->L("TINTERRUPTEDBYUSER");
						return B_ERROR;
					}
				}
			}
			delete mfile;
			entry.Unset();
		}
		// ok. all files are ok..



		buffer_Thread=spawn_thread(AudioBufferProducer,"Buffer Thread", BUFFER_PRIORITY,(void *)(&ap));
		rename_thread(buffer_Thread,BUFFERTHREADNAME);

		arguments=0;
		while (find_thread(CDRECORDTHREADNAME)!=cdrecord_Thread) {
			arguments++;
			snooze(1000);
			if (arguments>10000) break;
		}
		resume_thread(buffer_Thread);	
		resume_thread(cdrecord_Thread);	

		ReadOutputOfCdrecord(cdrecord_out, cdrecord_Thread, window1->view1->listview1->iCountItems());

		RemoveInfFiles();
		snooze(100000);
		if (find_thread(BUFFERTHREADNAME)==buffer_Thread) kill_thread(buffer_Thread);
//		if (find_thread(TORAWTHREADNAME)==toraw_Thread) kill_thread(toraw_Thread);
//		close(toraw_in);close(toraw_out);close(toraw_err);
		close(cdrecord_in);close(cdrecord_out);close(cdrecord_err);

		if (window5->interrupted) {
			errtype=H->L("TERROR");
			errstring=H->L("TINTERRUPTEDBYUSER");
			free(args);
			HideStatus2();
			return B_ERROR;
		}

		if (deviceCV->IsWriterEjected())
			eject_media(deviceCV->GetWriterDevicePath());
		free(args);
		HideStatus2();
		return B_OK;
}


status_t Application1::BlankRW(char *mode)
{
		int arguments=0;
		thread_id Thread;
		const char **args;
		BString str="";
		BString device="dev=";
		BString endstr="";
		int in, out, err;
		char info[40];
		// wait till a disc is in the drive and drive is ready
	
		window5->Lock();
		if (heliosCV->IsSimulation())
			window5->SetTitle(H->L("TSTATUSWINDOWTITLESIMULATION")); else
			window5->SetTitle(H->L("TSTATUSWINDOWTITLE"));
		window5->Unlock();

		ShowStatus2(H->L("TWAITINGFORDISC"),0);
		if (WaitForDisc(deviceCV->GetWriterDevicePath())==B_TIMEOUT) {
			errtype=H->L("TTIMEOUTTYPE");
			errstring=H->L("TTIMEOUTERROR");
			HideStatus2();
			return B_ERROR;
		}
		
		device << deviceCV->GetWriterDeviceNumber();
		args = (const char **)malloc(sizeof(char *) * (22));
		args[arguments++] = strdup(CM->GetString(PATH_TO_CDRECORD)->String());
		args[arguments++] = strdup(device.String());
		if (heliosCV->IsSimulation()) args[arguments++] = strdup("-dummy");
		if (heliosCV->IsForcedErasing()) args[arguments++] = strdup("-force");
		args[arguments++] = strdup(mode);
		args[arguments++] = strdup("-v");
		endstr << "-speed=" << heliosCV->GetWriterSpeed();
		args[arguments++] = strdup(endstr.String());
		args[arguments]=NULL;
		
		printArguments("cdrecord non-streaming args",args);


		Thread=pipe_command(arguments, args, in, out, err);
		set_thread_priority(Thread,CDRECORD_PRIORITY);
		if (Thread<0) {
			errtype=H->L("TERROR");
			errstring="cdrecord\n\n";
			errstring << H->L("TCOULDNOTLOAD");
			free(args);
			HideStatus2();
			return B_ERROR;
		}
		SetStatus2(H->L("TWAITING"),Thread);
		rename_thread(Thread,CDRECORDTHREADNAME);
		resume_thread(Thread);
		
		size_t bytesread=0;
		char buffer[10000];
		int speed=0;
		BString s;
		
		while (Thread==find_thread(CDRECORDTHREADNAME)) {
			bytesread=read(out,(void *)buffer, 10000);
			buffer[bytesread]=0;
			
			
			cdrecord_output->AddText(buffer);
			
			
			if (bytesread>0) {

				// get speed
				if (speed==0)
				if (strstr(buffer,"at speed")) {
					s=strstr(buffer,"at speed");
					s.Remove(0,8);
					s.Remove(s.FindFirst("in"),s.Length()-s.FindFirst("in"));
					speed=atoi(s.String());
					sprintf(info,"%dx (%0.1f MB/s)",speed,176400.0*(float)speed/(1024.0*1024.0));
					SetStatus2Info(info);
				}
				

				// Fixating...
				if (strstr(buffer,"Blanking time:")) {
					SetStatus2(H->L("TFINISHEDERASING"),-1);
					SetPercentage(100.0);
				} else
				// start blanking...
				if (strstr(buffer,"Blanking")) {
					SetStatus2(H->L("TBLANKINGCDRW"),-1);
					SetPercentage(33.33);
				}


			}
			snooze(250000);
		}
		close(in);close(out);close(err);
		window5->Lock();
		window5->SetTitle(H->L("TSTATUSWINDOWTITLE"));
		window5->Unlock();
		if (window5->interrupted) {
			errtype=H->L("TERROR");
			errstring=H->L("TINTERRUPTEDBYUSER");
			free(args);
			HideStatus2();
			return B_ERROR;
		}
		
		if (deviceCV->IsWriterEjected())
			eject_media(deviceCV->GetWriterDevicePath());

		HideStatus2();
		free(args);
		return B_OK;
}


// verschiebt einen Audio-eintrag um eine position nach oben
void Application1::MoveUp()
{
/*
has_project_changed=true;
int32 to, from;
//printf("%d\n",window1->view1->listview1->CountItems());
window1->Lock();
		for (int32 i=1; i<window1->view1->listview1->iCountItems();i++)
 			if (window1->view1->listview1->IsSelected(i)) {
 				to=i;
 				from=i;
  				to--;
				while ((((AudioRow *)(window1->view1->listview1->RowAt(to)))->IsSplitted()) ||
 					(((AudioRow *)(window1->view1->listview1->RowAt(to)))->GetSplitIndex()==0)) {
   					to--;
				};
			 	// move item when it is NOT a part of a splitted track
				if (!((((AudioRow *)(window1->view1->listview1->RowAt(from)))->IsSplitted()) ||
 					(((AudioRow *)(window1->view1->listview1->RowAt(from)))->GetSplitIndex()==0)))
					if (to>=0) window1->view1->listview1->MoveRow(from,to);
			}
window1->view1->listview1->Invalidate();
window1->Unlock();
*/
}

// verschiebt einen Audio-eintrag um eine position nach unten
void Application1::MoveDown()
{
/*
has_project_changed=true;
int32 max=window1->view1->listview1->CountItems();
int32 to;
window1->Lock();
		  for (int32 i=window1->view1->listview1->CountItems()-2;i>=0;i--)
 			if (window1->view1->listview1->IsItemSelected(i)) {
 				to=i;
  				to++;
				while ((((AudioItem *)(window1->view1->listview1->ItemAt(to)))->IsSplitted()) ||
 					(((AudioItem *)(window1->view1->listview1->ItemAt(to)))->IsParentSplitterTrack())) {
  				to++;
				};
			 	// move item when it is NOT a part of a splitted track
				if (!((((AudioItem *)(window1->view1->listview1->ItemAt(i)))->IsSplitted()) ||
 					(((AudioItem *)(window1->view1->listview1->ItemAt(i)))->IsParentSplitterTrack())))
					if (to<max) window1->view1->listview1->MoveItem(i,to);
			} 
window1->view1->listview1->Invalidate();
window1->Unlock();
*/
}





status_t Application1::ReadImage(BString *filename, bool alloweject) {

		int in, out, err;
		thread_id Thread;
		const char **args;
		BString str="f=";
		BString device="dev=";
		BString endstr="";
		int arguments=0;
		BEntry entry;
		char *info;
	
		// wait till a disc is in the drive and drive is ready
	
		ShowStatus2(H->L("TWAITINGFORDISC"),0);
		if (WaitForDisc(deviceCV->GetReaderDevicePath())==B_TIMEOUT) {
			errtype=H->L("TTIMEOUTTYPE");
			errstring=H->L("TTIMEOUTERROR");
			HideStatus2();
			return B_ERROR;
		}
		
		device << deviceCV->GetReaderDeviceNumber();
		args = (const char **)malloc(sizeof(char *) * (22));
		args[arguments++] = strdup(CM->GetString(PATH_TO_READCD)->String());
		args[arguments++] = strdup(device.String());
		str << filename->String();
		args[arguments++] = strdup(str.String());
//		args[arguments++] = strdup("-v");
		args[arguments]=NULL;

		printArguments("readcd args",args);

		// delete image file if it exists
		entry.SetTo(filename->String(), true);
		if (entry.Exists()) entry.Remove();
		entry.Unset();

		Thread=pipe_command(arguments, args, in, out, err);
		set_thread_priority(Thread,16);
		if (Thread<0) {
			errtype=H->L("TERROR");
			errstring="readcd\n\n";
			errstring << H->L("TCOULDNOTLOAD");
			free(args);
			HideStatus2();
			return B_ERROR;
		}
		SetStatus2(H->L("TREADINGCD"),Thread);
		rename_thread(Thread,READCDTHREADNAME);
		resume_thread(Thread);
		
		size_t bytesread=0;
		char buffer[10000];
		float length=-1;
		float progressed, olddone=0;
		BString s;
		BStopWatch *watch=new BStopWatch("MBps",true);

		SetStatus2Info("--- MB/s");
		
		do {
			bytesread=read(err,(void *)buffer, 10000);
			buffer[bytesread]=0;
			if (bytesread>0) {
//				printf(">>>%s\n",buffer);
				if (length<0)
				if (strstr(buffer,"end:")) {
					s=strstr(buffer,"end:");
					s.RemoveFirst("end:");
					//s.Remove(s.FindFirst("Blocks"),s.Length());
					length=atof(s.String());
				} // strstr			

				if (strstr(buffer,"addr:")) {
					s=strstr(buffer,"addr:");
					s.Remove(0,5);
					s.Remove(s.FindFirst("cnt:"),s.Length()-s.FindFirst("cnt:"));
					progressed=atof(s.String());
					if (length>0) {
						if (progressed>olddone+3072) {
							sprintf(info,"%0.1f MB/s",2048.0*(progressed-olddone)/(watch->ElapsedTime()*1.048576));
							olddone=progressed;
							SetStatus2Info(info);
							watch->Reset();
						}
						SetPercentage(100*progressed/length);
					}
				} // strstr			
			}
			snooze(100000);
		} while (Thread==find_thread(READCDTHREADNAME)) ;
		close(in);close(out);close(err);
		delete watch;
		if (window5->interrupted) {
			errtype=H->L("TERROR");
			errstring = H->L("TINTERRUPTEDBYUSER");
			free(args);
			HideStatus2();
			return B_ERROR;
		}
		if (alloweject)
			if (deviceCV->IsReaderEjected())
				eject_media(deviceCV->GetReaderDevicePath());
			
		free(args);
		HideStatus2();
		//delete filename;
		return B_OK;
}


status_t Application1::MakeImageOnly(const char *filename) {
	status_t	error;
	BList		*list;
	bool		bfs=false;
	
	switch(window1->view1->CDType()) {
		case DATACD_INDEX: {
			list=window1->view1->datapanel->GetFilesystem();
			if (list->CountItems()==1)
				if (((BString *)list->ItemAt(0))->Compare("BFS")==0)
					bfs=true;
			
			for (int i=list->CountItems()-1; i>=0; i--)
				delete ((BString *)list->RemoveItem(i));
			delete list;
			break;
		}

		case BOOTABLECD_INDEX: {
			list=window1->view1->bootablepanel->GetFilesystem();
			if (list->CountItems()==1)
				if (((BString *)list->ItemAt(0))->Compare("BFS")==0)
					bfs=true;
			
			for (int i=list->CountItems()-1; i>=0; i--)
				delete ((BString *)list->RemoveItem(i));
			delete list;
			break;
		}
		case DVDVIDEO_INDEX: {
			list=window1->view1->dvdvideopanel->GetFilesystem();
			if (list->CountItems()==1)
				if (((BString *)list->ItemAt(0))->Compare("BFS")==0)
					bfs=true;
			
			for (int i=list->CountItems()-1; i>=0; i--)
				delete ((BString *)list->RemoveItem(i));
			delete list;
			break;
		}
		default:break;
	}
	
	if (bfs)
		error=MakeBFSImage(strdup(filename));
	else
		error=MakeMKISOImage(strdup(filename));
		 
	return error;
}

void Application1::eject_media
	(
	const char * dev
	)
{
	int 		fd;
	status_t	status;

	fd = open(dev, 0);
	// printf("ejecting %s\n", dev);
	status = ioctl(fd, B_EJECT_DEVICE, 0, 0);
	close(fd);
}


void Application1::load_media
	(
	const char * dev
	)
{
	int 		fd;
	status_t	status;

	fd = open(dev, 0);
	// printf("ejecting %s\n", dev);
	status = ioctl(fd, B_LOAD_MEDIA, 0, 0);
	close(fd);
}


///////////////////////////////////////////////////////////////////////////////
///////////////////////////////////////////////////////////////////////////////
///////////////////////////////////////////////////////////////////////////////
status_t Application1::CopyAudioCD() {
//	BString		oldbase=base_folder;
	
//	base_folder=pathCV->GetTemporaryPath();
//	base_folder=&AppendTrailingSlash(base_folder);
	if (ReadAudioCD()>0) {
		eject_media(deviceCV->GetReaderDevicePath());
		
		if (window1->view1->CDType()!=AUDIOCD_INDEX)		
			if (window1->Lock()) {
				window1->view1->SetCDType(AUDIOCD_INDEX);
				window1->Unlock();
				ShowFolder(true);
			}
		
		return BurnAudio();
	} else {
		errtype=H->L("TERROR");
		errstring=H->L("TCOPYERROR");
		return -1;
	}
//	base_folder=oldbase;
}
///////////////////////////////////////////////////////////////////////////////
///////////////////////////////////////////////////////////////////////////////
///////////////////////////////////////////////////////////////////////////////


status_t Application1::CopyDataCD() {

	BString filename=pathCV->GetTemporaryPath();
	int	timeout=applicationCV->GetTimeoutLength()*10;
	char info[40];

			window5->Lock();
			if (heliosCV->IsSimulation())
				window5->SetTitle(H->L("TSTATUSWINDOWTITLESIMULATION"));
			else
				window5->SetTitle(H->L("TSTATUSWINDOWTITLE"));
			window5->Unlock();
	AppendTrailingSlash(&filename);
	//	if (filename[filename.Length()-1]!='/') filename.Append("/");
	filename.Append(pathCV->GetDefaultImageName());
	if (!FileCopy(deviceCV->GetReaderDevicePath(),filename.String(), GetDeviceSize(deviceCV->GetReaderDevicePath()))) {
		HideStatus2();
		errtype=H->L("TERROR");
		errstring=H->L("TCOPYERROR");
		return B_ERROR;
	}
	
//	if (!ReadImage(filename, true)) return;
	// wait till door has been opened
	SetStatus2( H->L("TWAITING"),0);
	if ((deviceCV->GetWriterNumber()==deviceCV->GetReaderNumber()) && (applicationCV->IsTimeout()))
	while (DeviceStatus(deviceCV->GetReaderDevicePath())!=E_DOOR_OPEN) {
		snooze(100000);
		timeout--;
		if (timeout==0) {
			HideStatus2();
			errtype=H->L("TTIMEOUT");
			errstring=H->L("TTIMEOUTERROR");
			return B_ERROR;
		}
		if (window5->interrupted) {
			HideStatus2();
			errtype=H->L("TERROR");
			errstring=H->L("TINTERRUPTEDBYUSER");
			return B_ERROR;
		}
		sprintf(info,"%d\"",timeout/10);
		SetStatus2Info(info);
		SetPercentage((float)timeout*100/((float)applicationCV->GetTimeoutLength()*10));
		};

	
	return BurnImage(filename);
}

status_t Application1::CopyOnTheFlyAudio() {
		int arguments=0;
		thread_id cdrecord_Thread, cdda2wav_Thread;
		const char **args;
		BString str="";
		BString device="dev=";
		BString filesystem="";
		int cdrecord_in, cdrecord_out, cdrecord_err, cdda2wav_in, cdda2wav_out, cdda2wav_err;

		// wait till a disc is in the drive and drive is ready
		if (WaitForDisc(deviceCV->GetWriterDevicePath())==B_TIMEOUT) {
			errtype=H->L("TTIMEOUTTYPE");
			errstring=H->L("TTIMEOUTERROR");
			HideStatus2();
			return B_ERROR;
		}
		if (WaitForDisc(deviceCV->GetReaderDevicePath())==B_TIMEOUT) {
			errtype=H->L("TTIMEOUTTYPE");
			errstring=H->L("TTIMEOUTERROR");
			HideStatus2();
			return B_ERROR;
		}
		
		window5->Lock();
		if (heliosCV->IsSimulation())
			window5->SetTitle(H->L("TSTATUSWINDOWTITLESIMULATION"));
		else
			window5->SetTitle(H->L("TSTATUSWINDOWTITLE"));
		window5->Unlock();

		//GetImageSize();
		device << deviceCV->GetReaderDeviceNumber();
		args = (const char **)malloc(sizeof(char *) * (36));

		// Make CDRECORD options
		device="dev=";
		device << deviceCV->GetWriterDeviceNumber();
		arguments=0;
		args[arguments++] = strdup(CM->GetString(PATH_TO_CDRECORD)->String());
		args[arguments++] = strdup(device.String());
		if (heliosCV->IsBurnproof()) args[arguments++] = strdup("driveropts=burnproof");
		args[arguments++] = strdup("-v");
		if (heliosCV->IsOverburning()) args[arguments++] = strdup("-overburn");
		// set fifo-buffer size
		str.SetTo("fs=");
		str << heliosCV->GetFIFOSize() << "M";
		args[arguments++] = strdup(str.String());
		// wait for input stream
		args[arguments++] = strdup("-waiti");
		args[arguments++] = strdup("-speed=");
		str.SetTo("");
		str << heliosCV->GetWriterSpeed();
		args[arguments++] = strdup(str.String());
		if (!heliosCV->IsFixating()) args[arguments++] = strdup("-nofix");
		//if (window1->view1->IsMultisession()) args[arguments++] = strdup("-multi");
		if (heliosCV->IsDAO()) args[arguments++] = strdup("-dao");
		if (heliosCV->IsSimulation()) args[arguments++] = strdup("-dummy");
		str.SetTo("");
		size_t	volumesize=GetDeviceSize(deviceCV->GetReaderDevicePath());
		str << "tsize=" << volumesize;
		args[arguments++] = strdup(str.String());
		printf("HELIOS: IMAGESIZE IS %s\n",str.String());
		args[arguments++] = strdup("-data");
		args[arguments++] = strdup(deviceCV->GetReaderDevicePath());
		args[arguments]=NULL;

		printArguments("cdrecord streaming args",args);

		// launch cdrecord thread now
		cdrecord_Thread=pipe_command(arguments, args, cdrecord_in, cdrecord_out, cdrecord_err);
		set_thread_priority(cdrecord_Thread,CDRECORD_PRIORITY);
		if (cdrecord_Thread<0) {
			errtype=H->L("TERROR");
			errstring="cdrecord\n\n";
			errstring << H->L("TCOULDNOTLOAD");
			free(args);
			HideStatus2();
			return B_ERROR;
		}
		SetStatus2( H->L("TWAITING"),cdrecord_Thread);
		rename_thread(cdrecord_Thread,CDRECORDTHREADNAME);







		// Make CDRECORD options
		device="";
		device << deviceCV->GetReaderDeviceNumber();
		arguments=0;
		args[arguments++] = strdup(CM->GetString(PATH_TO_CDDA2WAV)->String());
		args[arguments++] = strdup("-D");
		args[arguments++] = strdup(device.String());

		args[arguments++] = strdup("-t");
		args[arguments++] = strdup("1+");

		args[arguments++] = strdup("-O");
		args[arguments++] = strdup("cdr");

		args[arguments]=NULL;

		printArguments("cdda2wav streaming args",args);

		// launch cdda2wav thread now
		cdda2wav_Thread=pipe_command(arguments, args, cdda2wav_in, cdda2wav_out, cdda2wav_err);
		set_thread_priority(cdda2wav_Thread,CDRECORD_PRIORITY);
		if (cdda2wav_Thread<0) {
			errtype=H->L("TERROR");
			errstring="cdda2wav\n\n";
			errstring << H->L("TCOULDNOTLOAD");
			free(args);
			HideStatus2();
			return B_ERROR;
		}
		rename_thread(cdda2wav_Thread,CDDA2WAVTHREADNAME);
		resume_thread(cdrecord_Thread);	

		resume_thread(cdda2wav_Thread);	






		ReadOutputOfCdrecord(cdrecord_out, cdrecord_Thread);
		close(cdrecord_in);close(cdrecord_out);close(cdrecord_err);

		if (window5->interrupted) {
			errtype=H->L("TERROR");
			errstring=H->L("TINTERRUPTEDBYUSER");
			free(args);
			HideStatus2();
			return B_ERROR;
		}
			
		if (deviceCV->IsWriterEjected())
			eject_media(deviceCV->GetWriterDevicePath());
		if (deviceCV->IsReaderEjected())
			eject_media(deviceCV->GetReaderDevicePath());
		free(args);
		HideStatus2();
		return B_OK;
}

status_t Application1::CopyOnTheFly()
{
		int arguments=0;
		thread_id cdrecord_Thread;
		const char **args;
		BString str="";
		BString device="dev=";
		BString filesystem="";
		char info[40];
		int cdrecord_in, cdrecord_out, cdrecord_err;

		// wait till a disc is in the drive and drive is ready
		if (WaitForDisc(deviceCV->GetWriterDevicePath())==B_TIMEOUT) {
			errtype=H->L("TTIMEOUTTYPE");
			errstring=H->L("TTIMEOUTERROR");
			HideStatus2();
			return B_ERROR;
		}
		if (WaitForDisc(deviceCV->GetReaderDevicePath())==B_TIMEOUT) {
			errtype=H->L("TTIMEOUTTYPE");
			errstring=H->L("TTIMEOUTERROR");
			HideStatus2();
			return B_ERROR;
		}
		
		window5->Lock();
		if (heliosCV->IsSimulation())
			window5->SetTitle(H->L("TSTATUSWINDOWTITLESIMULATION"));
		else
			window5->SetTitle(H->L("TSTATUSWINDOWTITLE"));
		window5->Unlock();

		//GetImageSize();
		device << deviceCV->GetReaderDeviceNumber();
		args = (const char **)malloc(sizeof(char *) * (36));

		// Make CDRECORD options
		device="dev=";
		device << deviceCV->GetWriterDeviceNumber();
		arguments=0;
		args[arguments++] = strdup(CM->GetString(PATH_TO_CDRECORD)->String());
		args[arguments++] = strdup(device.String());
		if (heliosCV->IsBurnproof()) args[arguments++] = strdup("driveropts=burnproof");
		args[arguments++] = strdup("-v");
		if (heliosCV->IsOverburning()) args[arguments++] = strdup("-overburn");
		// set fifo-buffer size
		str.SetTo("fs=");
		str << heliosCV->GetFIFOSize() << "M";
		args[arguments++] = strdup(str.String());
		// wait for input stream
		args[arguments++] = strdup("-waiti");
		args[arguments++] = strdup("-speed=");
		str.SetTo("");
		str << heliosCV->GetWriterSpeed();
		args[arguments++] = strdup(str.String());
		if (!heliosCV->IsFixating()) args[arguments++] = strdup("-nofix");
		//if (window1->view1->IsMultisession()) args[arguments++] = strdup("-multi");
		if (heliosCV->IsDAO()) args[arguments++] = strdup("-dao");
		if (heliosCV->IsSimulation()) args[arguments++] = strdup("-dummy");
		str.SetTo("");
		size_t	volumesize=GetDeviceSize(deviceCV->GetReaderDevicePath());
		str << "tsize=" << volumesize;
		args[arguments++] = strdup(str.String());
		printf("HELIOS: IMAGESIZE IS %s\n",str.String());
		args[arguments++] = strdup("-data");
		args[arguments++] = strdup(deviceCV->GetReaderDevicePath());
		args[arguments]=NULL;

		printArguments("cdrecord streaming args",args);

		// launch cdrecord thread now
		cdrecord_Thread=pipe_command(arguments, args, cdrecord_in, cdrecord_out, cdrecord_err);
		set_thread_priority(cdrecord_Thread,CDRECORD_PRIORITY);
		if (cdrecord_Thread<0) {
			errtype=H->L("TERROR");
			errstring="cdrecord\n\n";
			errstring << H->L("TCOULDNOTLOAD");
			free(args);
			HideStatus2();
			return B_ERROR;
		}
		SetStatus2(H->L("TWAITING"),cdrecord_Thread);
		rename_thread(cdrecord_Thread,CDRECORDTHREADNAME);

		resume_thread(cdrecord_Thread);	





		size_t bytesread=0;
		char buffer[10000];
		int speed=0;
		int track=1;
		float length=0;	// in MB
		float op=0,progressed=0;
		char info2[40];
		BString s;
		
		do {
			bytesread=read(cdrecord_out,(void *)buffer, 10000);
			//printf("HELIOSBUFFER: >%s<\n",buffer);

			if (bytesread>0) {
	
				buffer[bytesread]=0;
				cdrecord_output->AddText(buffer);
				// get speed
				if (speed==0)
				if (strstr(buffer,"at speed")) {
					s=strstr(buffer,"at speed");
					s.Remove(0,8);
					s.Remove(s.FindFirst("in"),s.Length()-s.FindFirst("in"));
					speed=atoi(s.String());
					sprintf(info,"%dx (%0.1f MB/s)",speed,153600.0*(float)speed/(1024.0*1024.0));
					SetStatus2Info(info);
				}
				
				// Performing OPC...
				if (strstr(buffer,"Performing OPC...")) {
					SetStatus2(H->L("TPERFORMINGOPC"),-1);
				}

				// finished fixating...
				if (strstr(buffer,"Fixating time:")) {
					SetStatus2(H->L("TFINISHEDWRITING"),-1);
				}

				// start burning...
				if (strstr(buffer,"Starting new track")) {
					SetStatus2(H->L("TBURNINGDATACD"),-1);
				}

				// Fixating...
				if (strstr(buffer,"Fixating...")) {
					SetStatus2(H->L("TFIXATING"),-1);
				}

				// get track number
				if (strstr(buffer,"Track")) {
					s=strstr(buffer,"Track");
					s.RemoveFirst("Track");
					s.Remove(s.FindFirst(":"),s.Length()-s.FindFirst(":"));
					track=atoi(s.String());
				}		

				// get track length
				if (length==0)
				if (strstr(buffer,"of")) {
					s=strstr(buffer,"of");
					s.RemoveFirst("of");
					s.Remove(s.FindFirst("MB"),s.Length()-s.FindFirst("MB"));
					length=atof(s.String());
				}		
	
				// get write position
				if (strstr(buffer,"Track")) {
					s=strstr(buffer,"Track");
					s.Remove(0,9);
					s.Remove(s.FindFirst("of"),s.Length()-s.FindFirst("of"));
					progressed=atof(s.String());
					if (op<progressed) {
						op=progressed;
						if (length>0) {
							sprintf(info,"%0.1f%%",100.0*(float)progressed/(float)length);
							sprintf(info2,"%s %d",H->L("TTRACK"),track);
							SetStatus2Info(info, info2);
							SetPercentage(100.0*(float)progressed/(float)length);
						}
					}
				}		
			}
			snooze(250000);
		}while (cdrecord_Thread==find_thread(CDRECORDTHREADNAME));
		close(cdrecord_in);close(cdrecord_out);close(cdrecord_err);
		if (window5->interrupted) {
			errtype=H->L("TERROR");
			errstring=H->L("TINTERRUPTEDBYUSER");
			free(args);
			HideStatus2();
			return B_ERROR;
		}
			
		if (deviceCV->IsWriterEjected())
			eject_media(deviceCV->GetWriterDevicePath());
		if (deviceCV->IsReaderEjected())
			eject_media(deviceCV->GetReaderDevicePath());
		free(args);
		HideStatus2();
		return B_OK;
}


int Application1::MountBFSImage(const char *filename, const char *mountpoint) {
	int	timeout=applicationCV->GetTimeoutLength()/5;
	char info[40];
	int retvalue;
		
	SetStatus2(H->L("TMOUNTING"),0);
	while ((retvalue=mount("bfs",mountpoint,filename,2,NULL,0))<0) {
		snooze(5000000);
		timeout--;
		if ((timeout==0) || (window5->interrupted)){
			return -1;
		}
		sprintf(info,"%s",H->L("TWAITING"));
		SetStatus2Info(info);
		SetPercentage((float)timeout*100/((float)applicationCV->GetTimeoutLength()/5));
	};
	return retvalue;
}

int Application1::UnmountBFSImage(const char *mountpoint) {
		int	timeout=applicationCV->GetTimeoutLength();
		char info[40];
		int retvalue;
		BString s;

	SetStatus2(H->L("TUNMOUNTING"),0);
	while ((retvalue=unmount(mountpoint))<0) {
		snooze(1000000);
		timeout--;
		if ((timeout==0) || (window5->interrupted)){
			return -1;
		}
		sprintf(info,"%d\"",timeout);
		SetStatus2Info(info);
		SetPercentage((float)timeout*100/((float)applicationCV->GetTimeoutLength()));
	};
	s.SetTo("/boot/beos/bin/rmdir ").Append(mountpoint);
	system(s.String());
	return retvalue;
}



// Returns size of free space of a volume by passing the path of the device only.
size_t FreeVolumeSpace(const char *path) {
	
	fs_info	info;
	dev_t	device;
	
	
	device=dev_for_path(path);
	if (device>=0) {
		if (fs_stat_dev(device, &info)==0)
			return info.free_blocks*info.block_size;
	}
	return 0;	
}




status_t Application1::MakeBFSImage(const char *filename) {

		BFile file;
		BEntry entry;
		const char **args;
		status_t ret;
		int arguments=0;
		thread_id Thread;
		BString str="";
		size_t		FATsize,size, sizetowrite;
		int buffersize=1048576;
		status_t returnvalue=B_OK;
		extern char **environ;
		int	timeout=applicationCV->GetTimeoutLength();
		char info[40];
		char *buffer(NULL);



				
		
		imgsize=0;
		files=0;
		directories=0;
		
		// delete image file if it exists
		entry.SetTo(filename, true);
		if (entry.InitCheck()==B_OK) 
			entry.Remove();
		entry.Unset();
		
		// get size of directories and files
		GetSize(BDirectory(base_folder.String()));
		/*
		// displays info about files and folders
		str="";
		str<<imgsize<<" bytes\n" <<files<<" files\n"<<directories<< " folders";
		window1->Alert((char *)str.String());
		*/
		
		args = (const char **)malloc(sizeof(char *) * (22));
		str=pathCV->GetDefaultImageName();
		str=*RemoveTrailingSlash(str);
		CreateFolder(pathCV->GetDefaultMountPoint());
		printf("HELIOS: creating mount point\n");
		
		imgsize+=files*4096+directories*133200;	// die informationen für fensterpositionen und
								// positionen der icons in den fenstern
								// werden auch abgespeichert...
		FATsize=(5144*1024)+64*imgsize/(1024*1024)+384;
		imgsize+=FATsize;
		file.SetTo(filename,B_READ_WRITE | B_CREATE_FILE | B_ERASE_FILE);
		ret=file.SetSize(imgsize);
		// shit happens...
		if (ret==B_DEVICE_FULL)
			{
			errtype=H->L("TERROR");
			errstring=H->L("TNOSPACE");
			free(args);
			file.Unset();
			return B_ERROR;
			}
		size=0;
		printf("SIZE: imagesize is %lld bytes\n",imgsize);
		sizetowrite=buffersize;

		size_t olddone=0;
		
		ShowStatus2(H->L("TCREATINGIMAGEFILE"),0);
		SetStatus2Info("--- MB/s");
		BStopWatch *watch=new BStopWatch("MBps",true);

		buffer=(char *)malloc(buffersize);
		for (int i=0; i<buffersize; i++) buffer[i]=0x0;		

		while (sizetowrite>0) {
			if (window5->interrupted){
				delete watch;
				free(args);
				free(buffer);
				errtype=H->L("TERROR");
				errstring=H->L("TINTERRUPTEDBYUSER");
				file.Unset();
				HideStatus2();
				return B_ERROR;
			}
			if ((imgsize-size)>buffersize) sizetowrite=buffersize; else sizetowrite=imgsize-size;
		 	size+=file.Write((const void *)buffer,sizetowrite);


						if (size>olddone+5*buffersize) {
							sprintf(info,"%0.1f MB/s",((float)size-(float)olddone)/(watch->ElapsedTime()*0.953674316));
							olddone=size;
							SetStatus2Info(info);
							watch->Reset();
						}
		 	//asprintf(&info,"&0.1f%% (xx.x MB/s)",(float)size*100.0/((float)imgsize));
			SetPercentage((float)size*100/((float)imgsize));
		};
		file.Unset();
		delete watch;
		free(buffer);
		// create a Be File System
		// initialize the image file as a BFS image
		arguments=0;
		args[arguments++] = strdup(CM->GetString(PATH_TO_MKBFS)->String());
		args[arguments++] = strdup("2048");
		args[arguments++] = strdup(filename);
		args[arguments++] = strdup(window1->view1->GetVolumeName());
		args[arguments]=NULL;

		Thread=load_image(arguments, args, (const char **)environ);
		set_thread_priority(Thread,20);
		if (Thread<0) {
			errtype=H->L("TERROR");
			errstring="mkbfs\n\n";
			errstring << H->L("TCOULDNOTLOAD");
			free(args);
			HideStatus2();
			return B_ERROR;
		}
		SetStatus2(H->L("TCREATINGIMAGEFILE"),Thread);
		printf("HELIOS: creating Be File System\n");
		wait_for_thread(Thread,&returnvalue);
		if (window5->interrupted) {
			free(args);
			errtype=H->L("TERROR");
			errstring=H->L("TINTERRUPTEDBYUSER");
			HideStatus2();
			return B_ERROR;
		}
		sync();
		// mount the image file!
		printf("HELIOS: mounting %s\n", filename);
		if (MountBFSImage(strdup(filename), strdup(pathCV->GetDefaultMountPoint()))<0) {
			UnmountBFSImage(strdup(pathCV->GetDefaultMountPoint()));
			errtype=H->L("TERROR");
			errstring=filename;
			errstring<<"\n\n"<<H->L("TMOUNTERROR");
			HideStatus2();
			free(args);
			return B_ERROR;
		}

//		printf("HELIOS: Checking for home directory\n");
//		timeout=DEFAULTTIMEOUT*2;
//		SetStatus2(TWAITING,0);
//		str.SetTo(IMG_MOUNTPOINT).Append("/home");
//		entry.SetTo(str.String());
//		while (!entry.Exists()) {
//			snooze(500000);
//			timeout--;
//			if ((timeout==0) || (window5->interrupted)){
//				free(args);
//				entry.Unset();
//				UnmountBFSImage(IMG_MOUNTPOINT);
//				HideStatus2();
//				return false;
//			}
//			asprintf(&info,"%d\"",timeout*100/(DEFAULTTIMEOUT*2));
//			SetStatus2Info(info);
//			SetPercentage((float)timeout*100/((float)DEFAULTTIMEOUT*2));
//		};
//		entry.Unset();
		sync();
		snooze(500000);
		// sometimes, freshly created image files aren't empty...
		// so, delete all files, please.
		printf("HELIOS: erasing files in mounted image file\n");
		DeleteAllIn(BDirectory(pathCV->GetDefaultMountPoint()));
		sync();
		snooze(500000);
//		str.SetTo("/boot/beos/bin/rm -r -v ").Append(IMG_MOUNTPOINT).Append("/*");
//		system(str.String());



//		printf("HELIOS: Checking for empty space in image file.\n");
//		timeout=DEFAULTTIMEOUT*2;
//		SetStatus2(TWAITING,0);
//		while ((size=FreeVolumeSpace(IMG_MOUNTPOINT))<imgsize-FATsize) {
//			snooze(500000);
//			timeout--;
//			if ((timeout==0) || (window5->interrupted)){
//				free(args);
//				UnmountBFSImage(IMG_MOUNTPOINT);
//				HideStatus2();
//				return false;
//			}
//			asprintf(&info,"%0.1f%%",100*size/(imgsize-FATsize));
//			SetStatus2Info(TWAITING);
//			SetPercentage((float)timeout*100/((float)DEFAULTTIMEOUT*2));
//		};



		printf("HELIOS: copying files to image file.\n");
		// copy files into mounted image file
		arguments=0;
		args[arguments++] = strdup("/boot/beos/bin/cp");
		args[arguments++] = strdup("--recursive");
		str.SetTo(base_folder.String()).Append(".");
		args[arguments++] = strdup(str.String());
		str.SetTo(pathCV->GetDefaultMountPoint()).Append("/");
		args[arguments++] = strdup(str.String());
		args[arguments]=NULL;
		

		Thread=load_image(arguments, args, (const char **)environ);
		set_thread_priority(Thread,12);
		if (Thread<0) {
			UnmountBFSImage(pathCV->GetDefaultMountPoint());
			errtype=H->L("TERROR");
			errstring="cdrecord\n\n";
			errstring << H->L("TCOULDNOTLOAD");
			free(args);
			HideStatus2();
			return B_ERROR;
		}
		SetStatus2(H->L("TCOPYINGFILES"),Thread);

		rename_thread(Thread,CPTHREADNAME);
		resume_thread(Thread);
		timeout=applicationCV->GetTimeoutLength()*100;
		
		// waiting for copy-thread
		while (find_thread(CPTHREADNAME)!=Thread) {
			snooze(100000);
			timeout--;
			if (timeout==0) {
				free(args);
				UnmountBFSImage(pathCV->GetDefaultMountPoint());
				HideStatus2();
				errtype=H->L("TTIMEOUT");
				errstring=H->L("TTIMEOUTERROR");
				return B_ERROR;
			}
			if (window5->interrupted) {
				free(args);
				UnmountBFSImage(pathCV->GetDefaultMountPoint());
				HideStatus2();
				errtype=H->L("TERROR");
				errstring=H->L("TINTERRUPTEDBYUSER");
				return B_ERROR;
			}
			SetPercentage((float)timeout*100/((float)applicationCV->GetTimeoutLength()*100));
		};

		// COPYING FILES
		while (find_thread(CPTHREADNAME)==Thread) {
			sleep(1);
			if (window5->interrupted) {
				free(args);
				UnmountBFSImage(pathCV->GetDefaultMountPoint());
				HideStatus2();
				errtype=H->L("TERROR");
				errstring=H->L("TINTERRUPTEDBYUSER");
				return B_ERROR;
			}
		};



		// unmount image file
		if (UnmountBFSImage(pathCV->GetDefaultMountPoint())<0) {
			errtype=H->L("TERROR");
			errstring=pathCV->GetDefaultImageName();
			errstring<<"\n\n"<<H->L("TUNMOUNTERROR");
			HideStatus2();
			free(args);
			return B_ERROR;
		}
			
		HideStatus2();
		free(args);
		return B_OK;
}


// UNUSED!!!!!!!!!!!!!!!!!!!!!!!!!!!!!
/*
int Application1::MakeMKISOFSCommandline(const char **args, const char *filename, bool printsize) {
		int arguments=0;

		args[arguments++] = strdup(CM->GetString(PATH_TO_MKISOFS)->String());
		if (printsize) {
			args[arguments++] = strdup("-print-size");
			args[arguments++] = strdup("-gui");
		}
		if (strlen(imageCV->GetApplication())!=0) {
			args[arguments++] = strdup("-A");
			args[arguments++] = strdup(imageCV->GetApplication());
		}

		if (strlen(imageCV->GetPreparer())!=0) {
			args[arguments++] = strdup("-p");
			args[arguments++] = strdup(imageCV->GetPreparer());
		}
		
		if (strlen(imageCV->GetPublisher())!=0) {
			args[arguments++] = strdup("-P");
			args[arguments++] = strdup(imageCV->GetPublisher());
		}
		args[arguments++] = strdup("-follow-links");
		args[arguments++] = strdup("-R");
		
		switch(window1->view1->CDType()) {
			case DATACD_INDEX: {
				if (strlen(window1->view1->datapanel->GetFileSystem())>0)
					args[arguments++] = strdup(window1->view1->datapanel->GetFileSystem());
				if (strcmp(window1->view1->datapanel->GetFileSystem(),"-J")==0)
					args[arguments++] = strdup("-l");
				break;
			}
			case BOOTABLECD_INDEX: {
				if (strlen(window1->view1->bootablepanel->GetFileSystem())>0)
					args[arguments++] = strdup(window1->view1->bootablepanel->GetFileSystem());
				if (strcmp(window1->view1->bootablepanel->GetFileSystem(),"-J")==0)
					args[arguments++] = strdup("-l");
				break;
			}
			case DVDVIDEO_INDEX: {
				if (strlen(window1->view1->dvdvideopanel->GetFileSystem())>0)
					args[arguments++] = strdup(window1->view1->dvdvideopanel->GetFileSystem());
				if (strcmp(window1->view1->dvdvideopanel->GetFileSystem(),"-J")==0)
					args[arguments++] = strdup("-l");
				break;
			}
			default: {
				break;
			}
		}
		

		// bootable CD
		if (window1->view1->CDType()==BOOTABLECD_INDEX) {
			BString	bootimage=window1->view1->GetImageNameBootCD();
			bootimage.RemoveFirst("/");
			// add where to find the boot image file
			args[arguments++] = strdup("-b");
			args[arguments++] = strdup(bootimage.String());
			// and where the boot catalog can be found
			
			// if there is a dot in the image file name, cut off the
			// string after the dot and add "catalog"
			// the dot has to be after the last slash
			if (bootimage.FindLast(".")>bootimage.FindLast("/")) {
				bootimage.Remove(bootimage.FindLast("."), bootimage.Length()-bootimage.FindLast("."));
			}
				bootimage.Append(".cat");
			
			args[arguments++] = strdup("-c");
			args[arguments++] = strdup(bootimage.String());
		}
	

		args[arguments++] = strdup("-V");
		args[arguments++] = strdup(window1->view1->GetVolumeName());
		
		// output to file else to stdout...
		if ((strlen(filename)!=0) && (!printsize)) {
			args[arguments++] = strdup("-o");
			args[arguments++] = strdup(filename);
		}	// else output to stdout
		
args[arguments++] = strdup(base_folder.String());
		args[arguments]=NULL;
		
		// send arguments to stdout
		if (!printsize)
			printArguments("MKISOFS Arguments",args);
		
		return arguments;
}
*/


status_t Application1::MakeMKISOImage(const char *filename) {

		const char **args;
		int arguments=0;
		thread_id Thread;
		BString lastarg, str="";
		BString filesystem="";
		int in, out, err;
				
		// allocate some memory for the command line array
		args = (const char **)malloc(sizeof(char *) * (50));
		// fill array with arguments
		if (MKISOFS_CommandLine(args, &arguments, false, filename)!=B_OK) {
			free(args);
			HideStatus2();
			return B_ERROR;
		}

		window5->Lock();
		window5->interrupted=false;
		window5->Unlock();
//		// multisession disc selected???
//		if (window1->view1->IsMultisession()) {
//			// ask whether this session should be the first, last or another
//			// session of the disc
//			ErrorBox *alert=new ErrorBox(E_ORANGE_COLOR, "", H->L("MULTISESSION_ALERT"),H->L("MULTISESSION_ALERT_B0"),H->L("MULTISESSION_ALERT_B1"),H->L("MULTISESSION_ALERT_B2"));
//			int selected=alert->Go();
//		
//			if (selected!=2)
//				if (GetMultisessionSectors(start_sec, end_sec)!=B_OK) {
//					errtype=H->L("TERROR");
//					errstring=H->L("MULTISESSION_FAILS_ALERT");
//					free(args);
//					HideStatus2();
//					return B_ERROR;
//				}
//		
//			if (selected==0) {		// it is another session!
//				if (window1->view1->CDType()!=CDEXTRA_INDEX) {
//					args[arguments++] = strdup("-M");
//					args[arguments++] = strdup(deviceCV->GetWriterDeviceNumber());
//				}
//				args[arguments++] = strdup("-C");
//				filesystem << start_sec << "," << end_sec;
//				args[arguments++] = strdup(filesystem.String());
//			}
//
//			if (selected==1) {		// it is the last session!
//				if (window1->view1->CDType()!=CDEXTRA_INDEX) {
//					args[arguments++] = strdup("-M");
//					args[arguments++] = strdup(deviceCV->GetWriterDeviceNumber());
//				}
//				args[arguments++] = strdup("-C");
//				filesystem << start_sec << "," << end_sec;
//				args[arguments++] = strdup(filesystem.String());
//				// leave the "-multi"-option for cdrecord!
//			}
//		
//		}
//		args[arguments++]=strdup(base_folder.String());
//		args[arguments]=NULL;
		// and _please_ launch a new thread!
		Thread=pipe_command(arguments, args, in, out, err);
		set_thread_priority(Thread,16);
		if (Thread<0) {
			errtype=H->L("TERROR");
			errstring="mkisofs\n\n";
			errstring << H->L("TCOULDNOTLOAD");
			free(args);
			HideStatus2();
			return B_ERROR;
		}
		ShowStatus2(H->L("TCREATINGIMAGEFILE"),Thread);
		rename_thread(Thread,MKISOFSTHREADNAME);
		resume_thread(Thread);
		
		size_t bytesread=0;
		char buffer[10000];
		char prev;
		char info[40];
		int32 length;
		float percentage;
		
		while (Thread==find_thread(MKISOFSTHREADNAME)) {
			bytesread=read(err,(void *)buffer, 10000);
			buffer[bytesread]=0;
			if (bytesread>0) {
				mkisofs_output->AddText(buffer);
				if (strstr(buffer," done, estimate")) {
					buffer[(length=(bytesread-strlen(strstr(buffer," done, estimate"))-1))]=0;
					if (buffer[length-1]==' ') {
						prev=buffer[length-2];	
						buffer[length-2]='0';
						buffer[length-1]=prev;
					}
					percentage=atof(buffer);
					sprintf(info,"%0.1f%%",percentage);
					SetStatus2Info(info);
					SetPercentage(percentage);
				} // strstr			
			}
			snooze(100000);
		}
		close(in);close(out);close(err);
		free(args);
		HideStatus2();

		if (window5->interrupted) {
			errtype=H->L("TERROR");
			errstring=H->L("TINTERRUPTEDBYUSER");
			return B_ERROR;
		}
			
		return B_OK;
}


void Application1::GetSize(BDirectory dir) {

	int32 c=dir.CountEntries();
	BEntry entry;
//	static off_t size=0;
	
	if (c>0)
	for (int32 i=0; i<c; i++) {
		dir.GetNextEntry(&entry, true);
	//	printf("SIZE: %lld\n",imgsize);
		if (entry.IsDirectory())
			{
			directories++;
	//		printf("DIRECTORY\n");
			GetSize(BDirectory(&entry));
			//entry->GetParent(&dir);
			}
			else
			{
			files++;
	//		printf("FILE\n");
			BFile file;
			char buf[B_ATTR_NAME_LENGTH];
			off_t s=0;
			attr_info info;
			
			file.SetTo(&entry, B_READ_ONLY);
			while (file.GetNextAttrName(buf)==B_OK)
				{
				file.GetAttrInfo((const char *)buf,&info);
				imgsize+=info.size;
				}
			file.GetSize(&s);
			file.Unset();
			if (s % 2048!=0) imgsize+=(int(s / 2048)+1)*2048; else
			imgsize+=s;
			}	
			entry.Unset();		
	}
	
}


void
Application1::SaveSettings() {
	BMessage	*msg=new BMessage();

	configW->Lock();
	window1->Lock();

	// HeliosConfigView
	heliosCV->GetSettings(msg);
	
	CM->SetMessage("Preferences", msg);
	
	// DeviceConfigView
	CM->SetBool(EJECT_WRITER, deviceCV->IsWriterEjected());
	CM->SetBool(EJECT_READER, deviceCV->IsReaderEjected());
	CM->SetBool(TWODRIVECOPY, deviceCV->IsTwoDriveCopy());
	CM->SetInt8(WRITERDEVICE, deviceCV->GetWriterNumber());
	CM->SetInt8(READERDEVICE, deviceCV->GetReaderNumber());
	CM->SetInt8(SCANNINGMETHOD, deviceCV->GetScanMethod());
	CM->SetBool(SHOWALLDEVICES, deviceCV->GetShowsAllDevices());
	
	// PathConfigView
	CM->SetString(TEMPORARY_PATH, pathCV->GetTemporaryPath());
	CM->SetString(PROJECT_PATH, pathCV->GetProjectPath());
	CM->SetString(DEFIMAGENAME, pathCV->GetDefaultImageName());
	CM->SetString(DEFMOUNTPOINT, pathCV->GetDefaultMountPoint());
	
	// ImageConfigView
	CM->SetString(I_PUBLISHER, imageCV->GetPublisher());
	CM->SetString(I_PREPARER, imageCV->GetPreparer());
	CM->SetString(I_APPLICATION, imageCV->GetApplication());
	
	// FilesystemConfigView
	CM->SetBool(F_ISO9660_CB, fspanelCV->GetISO9660());
	CM->SetBool(F_JOLIET_CB, fspanelCV->GetJoliet());
	CM->SetBool(F_RR_CB, fspanelCV->GetRockRidge());
	CM->SetBool(F_HFS_CB, fspanelCV->GetHFS());
	CM->SetBool(F_UDF_CB, fspanelCV->GetUDF());
	
	CM->SetBool(F_83FN_RB, fspanelCV->GetISO96608Dot3());
	CM->SetBool(F_31FN_RB, fspanelCV->GetISO966031());
	CM->SetBool(F_37FN_RB, fspanelCV->GetISO966037());
	CM->SetBool(F_JOLDEF_RB, fspanelCV->GetJolietDefault());
	CM->SetBool(F_JOLUNI_RB, fspanelCV->GetJolietUnicode());
	CM->SetBool(F_RRDEF_RB, fspanelCV->GetRockRidgeDefault());
	CM->SetBool(F_RRRAT_RB, fspanelCV->GetRockRidgeRationalized());
	CM->SetBool(F_UDFDEF_RB, fspanelCV->GetUDFDefault());
	CM->SetBool(F_UDFDVD_RB, fspanelCV->GetUDFDVDVideo());
	CM->SetBool(F_MAPPING_CB, fspanelCV->GetMapping());
	CM->SetString(F_MAPPING_TC, fspanelCV->GetMappingFile());
		
	// StandardConfigView
	CM->SetInt32(S_FILESYSTEM, standardCV->GetFilesystem());
	CM->SetString(S_VOLUMENAME, standardCV->GetVolumeName());
	CM->SetInt8(S_CDTYPE, standardCV->GetCDType());
	CM->SetString(S_PUBLISHER, standardCV->GetPublisher());
	CM->SetString(S_PREPARER, standardCV->GetPreparer());
	CM->SetString(S_APPLICATION, standardCV->GetApplication());
	//CM->SetString(S_TEMPPATH, standardCV->GetTemporaryPath());
	//CM->SetString(S_PROJECTPATH, standardCV->GetProjectPath());
	
	// ApplicationConfigView
	CM->SetBool(APP_TIMEOUT, applicationCV->IsTimeout());
	CM->SetInt16(APP_TIMEOUTLENGTH, applicationCV->GetTimeoutLength());
	CM->SetBool(APP_AUTOUPDATE, applicationCV->IsAutoUpdating());
	CM->SetInt16(APP_UPDATEFREQUENCY, applicationCV->GetUpdateFrequency());
	
	// SoundConfigView
	CM->SetBool(BEEPWHENWRITTEN, soundCV->HasFinishedWritingSound());
	CM->SetBool(BEEPWHENIMAGE, soundCV->HasFinishedImageSound());
	CM->SetBool(BEEPWHENERROR, soundCV->HasErrorSound());
	
	// SaveConfigView
	CM->SetBool(STREAMING_MODE, saveCV->IsStreamed());
	CM->SetBool(SAVEWHENQUIT, saveCV->IsSavedWhenQuit());
	CM->SetInt8(EXITMODE, (int8)saveCV->GetExitMode());
	CM->SetBool(AUTOUPDATE, saveCV->AutoUpdate());
	CM->SetInt32(LANGUAGE_ID, saveCV->GetLanguage());
	CM->SetBool(BUBBLEHELP, saveCV->BalloonHelp());
	
	// DATA-CD
	CM->SetInt32(DATACD_FILESYSTEM, window1->view1->GetFilesystem());
	CM->SetString(DATACD_VOLUMENAME, window1->view1->GetVolumeName());
	CM->SetBool(DATACD_MULTISESSION, window1->view1->IsMultisession());
	// AUDIO-CD
	CM->SetBool(AUDIOCD_WRITECDTEXT, window1->view1->WriteCDText());
	CM->SetBool(AUDIOCD_SCMS, window1->view1->ProtectAudioCD());
	CM->SetString(AUDIOCD_ALBUMNAME, window1->view1->GetCDAlbum());
	// BOOTABLE-CD
	CM->SetString(BOOTCD_VOLUMENAME, window1->view1->GetVolumeNameBootCD());
	CM->SetString(BOOTCD_FLOPPYIMAGE, window1->view1->GetImageNameBootCD());
	CM->SetInt32(BOOTCD_FILESYSTEM, window1->view1->GetFilesystemBootCD());
	// CD-EXTRA
	CM->SetString(CDEXTRA_DATAIMAGE, window1->view1->GetImageNameCDExtra());
	CM->SetString(CDEXTRA_ALBUMNAME, window1->view1->GetCDAlbumCDExtra());
	CM->SetBool(CDEXTRA_WRITECDTEXT, window1->view1->WriteCDTextCDExtra());
	CM->SetInt32(CDEXTRA_FILESYSTEM, window1->view1->GetFilesystemCDExtra());
	CM->SetString(CDEXTRA_VOLUMENAME, window1->view1->GetVolumeNameCDExtra());
	// DVDVIDEO
	CM->SetInt32(DVDVIDEO_FILESYSTEM, window1->view1->GetFilesystemDVDVideo());
	CM->SetString(DVDVIDEO_VOLUMENAME, window1->view1->GetVolumeNameDVDVideo());
	
	// WINDOW POSITIONS AND SIZES
	CM->SetRect(MAINWINDOW_RECT, window1->Frame());
	CM->SetRect(WINDOW2_RECT, window2->Frame());
	CM->SetRect(PREFERENCESWINDOW_RECT, configW->Frame());
	CM->SetRect(CDRECORDLOGWINDOW_RECT, cdrecord_output->Frame());
	CM->SetRect(MKISOFSLOGWINDOW_RECT, mkisofs_output->Frame());
	
	// COLORS
	CM->SetColor(BURNING_STATUS_BAR_COLOR, colorCV->GetColor(0));
		if (window5->Lock()) {
			window5->SetBarColor(colorCV->GetColor(0));
			window5->Unlock();
		}
	CM->SetColor(CDSIZE_STATUS_BAR_COLOR, colorCV->GetColor(1));
		if (window1->Lock()) {
			window1->view1->box2->SetBarColor(colorCV->GetColor(1));
			window1->Unlock();
		}
	CM->SetColor(LOGWIN_BG_COLOR, colorCV->GetColor(2));
	CM->SetColor(LOGWIN_FG_COLOR, colorCV->GetColor(3));
		if (cdrecord_output->Lock()) {
			cdrecord_output->SetColors(colorCV->GetColor(3), colorCV->GetColor(2));
			cdrecord_output->Unlock();
		}
		if (mkisofs_output->Lock()) {
			mkisofs_output->SetColors(colorCV->GetColor(3), colorCV->GetColor(2));
			mkisofs_output->Unlock();
		}
	CM->SetColor(LISTSELECTION_COLOR, colorCV->GetColor(4));
	if (window1->Lock()) {
		window1->view1->listview1->SetSelectionColor(colorCV->GetColor(4));
		window1->Unlock();
	}
	CM->SetColor(LISTBACKGROUND_COLOR, colorCV->GetColor(5));
	if (window1->Lock()) {
		window1->view1->listview1->SetBackgroundColor(colorCV->GetColor(5));
		window1->Unlock();
	}


	window1->view1->listview1->SaveAllStates();
	
	// ProtocolWindow Settings
	if (cdrecord_output->Lock()) {
		CM->SetBool(CDRECORD_LOGWIN_VISIBLE, !cdrecord_output->IsHidden());
		CM->SetBool(CDRECORD_LOG, cdrecord_output->DoesLogToFile());
		CM->SetString(CDRECORD_LOGFILENAME, cdrecord_output->GetLogFileName());
		cdrecord_output->Unlock();
	}
	
	if (mkisofs_output->Lock()) {
		CM->SetBool(MKISOFS_LOGWIN_VISIBLE, !mkisofs_output->IsHidden());
		CM->SetBool(MKISOFS_LOG, mkisofs_output->DoesLogToFile());
		CM->SetString(MKISOFS_LOGFILENAME, mkisofs_output->GetLogFileName());
		mkisofs_output->Unlock();
	}
	
	
	CM->WriteConfiguration();
	configW->Unlock();
	window1->Unlock();
	
	delete msg;
	return;

}



void
Application1::ReadSettings() {
	BMessage *msg;
	
	msg=CM->GetMessage("Preferences");

	// Set default values...

	// Paths of command line tools
	if (!CM->HasData(PATH_TO_MKISOFS))		CM->SetString(PATH_TO_MKISOFS, "/boot/home/config/bin/mkisofs");
	if (!CM->HasData(PATH_TO_CDRECORD))	CM->SetString(PATH_TO_CDRECORD, "/boot/home/config/bin/cdrecord");
	if (!CM->HasData(PATH_TO_MKBFS))		CM->SetString(PATH_TO_MKBFS, "/boot/beos/bin/mkbfs");
	if (!CM->HasData(PATH_TO_READCD))		CM->SetString(PATH_TO_READCD, "/boot/home/config/bin/readcd");
	if (!CM->HasData(PATH_TO_CDDA2WAV))	CM->SetString(PATH_TO_CDDA2WAV, "/boot/home/config/bin/cdda2wav");
	if (!CM->HasData(TEMP_PATH))			CM->SetString(TEMP_PATH, "/boot/Helios/");

	// HeliosConfigView
	/*
	if (!CM->HasData(BURNPROOF))		CM->SetBool(BURNPROOF, false);
	if (!CM->HasData(SIMULATION))		CM->SetBool(SIMULATION, false);
	if (!CM->HasData(FIXATE_DISC))		CM->SetBool(FIXATE_DISC, true);
	if (!CM->HasData(USE_DAO))		CM->SetBool(USE_DAO, true);
	if (!CM->HasData(OVERBURNING))		CM->SetBool(OVERBURNING, false);
	if (!CM->HasData(FORCEDERASING))	CM->SetBool(FORCEDERASING, false);
	if (!CM->HasData(WRITINGSPEED))	CM->SetInt32(WRITINGSPEED, 10);
	if (!CM->HasData(FIFOBUFFERSIZE))	CM->SetInt8(FIFOBUFFERSIZE, 4);
*/
	// DeviceConfigView
	if (!CM->HasData(EJECT_WRITER))	CM->SetBool(EJECT_WRITER, true);
	if (!CM->HasData(EJECT_READER))	CM->SetBool(EJECT_READER, true);
	if (!CM->HasData(WRITERDEVICE))	CM->SetInt8(WRITERDEVICE, 0);
	if (!CM->HasData(READERDEVICE))	CM->SetInt8(READERDEVICE, 0);
	if (!CM->HasData(TWODRIVECOPY))	CM->SetBool(TWODRIVECOPY, false);
	if (!CM->HasData(SCANNINGMETHOD))	CM->SetInt8(SCANNINGMETHOD, DCV_DEFAULT_METHOD);
	if (!CM->HasData(SHOWALLDEVICES))	CM->SetBool(SHOWALLDEVICES, false);
	
	// PathConfigView
	if (!CM->HasData(TEMPORARY_PATH))	CM->SetString(TEMPORARY_PATH, "/boot/home/");
	if (!CM->HasData(PROJECT_PATH))	CM->SetString(PROJECT_PATH, "/boot/Helios/");
	if (!CM->HasData(DEFIMAGENAME))	CM->SetString(DEFIMAGENAME, "Helios.imagefile");
	if (!CM->HasData(DEFMOUNTPOINT))	CM->SetString(DEFMOUNTPOINT, "/HeliosMountPoint");

	// ImageConfigView
	if (!CM->HasData(I_PUBLISHER))		CM->SetString(I_PUBLISHER, "BeOS");
	if (!CM->HasData(I_PREPARER))		CM->SetString(I_PREPARER, "BeOS");
	if (!CM->HasData(I_APPLICATION))	CM->SetString(I_APPLICATION, "Helios");

	// FilesystemConfigView
	if (!CM->HasData(F_ISO9660_CB))	CM->SetBool(F_ISO9660_CB, true);
	if (!CM->HasData(F_JOLIET_CB))		CM->SetBool(F_JOLIET_CB, true);
	if (!CM->HasData(F_RR_CB))		CM->SetBool(F_RR_CB, true);
	if (!CM->HasData(F_HFS_CB))		CM->SetBool(F_HFS_CB, false);
	if (!CM->HasData(F_UDF_CB))		CM->SetBool(F_UDF_CB, false);
	if (!CM->HasData(F_83FN_RB))		CM->SetBool(F_83FN_RB, false);
	if (!CM->HasData(F_31FN_RB))		CM->SetBool(F_31FN_RB, true);
	if (!CM->HasData(F_37FN_RB))		CM->SetBool(F_37FN_RB, false);
	if (!CM->HasData(F_JOLDEF_RB))		CM->SetBool(F_JOLDEF_RB, true);
	if (!CM->HasData(F_JOLUNI_RB))		CM->SetBool(F_JOLUNI_RB, false);
	if (!CM->HasData(F_RRDEF_RB))		CM->SetBool(F_RRDEF_RB, true);
	if (!CM->HasData(F_RRRAT_RB))		CM->SetBool(F_RRRAT_RB, false);
	if (!CM->HasData(F_UDFDEF_RB))		CM->SetBool(F_UDFDEF_RB, true);
	if (!CM->HasData(F_UDFDVD_RB))		CM->SetBool(F_UDFDVD_RB, false);
	if (!CM->HasData(F_MAPPING_CB))	CM->SetBool(F_MAPPING_CB, true);
	if (!CM->HasData(F_MAPPING_TC))	CM->SetString(F_MAPPING_TC, "");

	// StandardConfigView
	if (!CM->HasData(S_FILESYSTEM))		CM->SetInt32(S_FILESYSTEM, 0);
	if (!CM->HasData(S_VOLUMENAME))	CM->SetString(S_VOLUMENAME, H->L("L:untitled"));
	if (!CM->HasData(S_CDTYPE))		CM->SetInt8(S_CDTYPE, 0);
	if (!CM->HasData(S_PUBLISHER))		CM->SetString(S_PUBLISHER, "BeOS");
	if (!CM->HasData(S_PREPARER))		CM->SetString(S_PREPARER, "BeOS");
	if (!CM->HasData(S_APPLICATION))	CM->SetString(S_APPLICATION, "Helios");
	//if (!CM->HasData(S_TEMPPATH))		CM->SetString(S_TEMPPATH, "/boot/home/");
	//if (!CM->HasData(S_PROJECTPATH))	CM->SetString(S_PROJECTPATH, "/boot/Helios/");
	
	// ApplicationConfigView
	if (!CM->HasData(APP_AUTOUPDATE))		CM->SetBool(APP_AUTOUPDATE, false);
	if (!CM->HasData(APP_UPDATEFREQUENCY))	CM->SetBool(APP_UPDATEFREQUENCY, 60);
	if (!CM->HasData(APP_TIMEOUTLENGTH))	CM->SetBool(APP_TIMEOUTLENGTH, 90);
	if (!CM->HasData(APP_TIMEOUT))			CM->SetBool(APP_TIMEOUT, true);
	
	// SoundConfigView
	if (!CM->HasData(BEEPWHENWRITTEN))		CM->SetBool(BEEPWHENWRITTEN, false);
	if (!CM->HasData(BEEPWHENIMAGE))		CM->SetBool(BEEPWHENIMAGE, false);
	if (!CM->HasData(BEEPWHENERROR))		CM->SetBool(BEEPWHENERROR, false);
	
	// SaveConfigView
	if (!CM->HasData(STREAMING_MODE))	CM->SetBool(STREAMING_MODE, true);
	if (!CM->HasData(SAVEWHENQUIT))	CM->SetBool(SAVEWHENQUIT, true);
	if (!CM->HasData(EXITMODE))		CM->SetInt8(EXITMODE, EM_ASK_USER);
	if (!CM->HasData(INFOBAR_ENABLED))	CM->SetBool(INFOBAR_ENABLED, false);
	if (!CM->HasData(AUTOUPDATE))		CM->SetBool(AUTOUPDATE, false);
	if (!CM->HasData(LANGUAGE_ID))		CM->SetInt32(LANGUAGE_ID, 0);
	if (!CM->HasData(BUBBLEHELP))		CM->SetBool(BUBBLEHELP, true);

	// ColorConfigView
	if (!CM->HasData(BURNING_STATUS_BAR_COLOR))	CM->SetColor(BURNING_STATUS_BAR_COLOR, (rgb_color){60,225,60,255});
	if (!CM->HasData(CDSIZE_STATUS_BAR_COLOR))	CM->SetColor(CDSIZE_STATUS_BAR_COLOR, (rgb_color){50, 150, 255,255});
	if (!CM->HasData(LOGWIN_BG_COLOR))			CM->SetColor(LOGWIN_BG_COLOR, (rgb_color){255,255,255,255});
	if (!CM->HasData(LOGWIN_FG_COLOR))			CM->SetColor(LOGWIN_FG_COLOR, (rgb_color){0,0,0,255});
	if (!CM->HasData(LISTSELECTION_COLOR))		CM->SetColor(LISTSELECTION_COLOR, (rgb_color){90,90,90,255});
	if (!CM->HasData(LISTBACKGROUND_COLOR))		CM->SetColor(LISTBACKGROUND_COLOR, (rgb_color){255,255,255,255});


	// CD-MODE-SPECIFIC CONFIGURATION
	// DATA-CD
	if (!CM->HasData(DATACD_FILESYSTEM))	CM->SetInt32(DATACD_FILESYSTEM, CM->GetInt32(S_FILESYSTEM));
	if (!CM->HasData(DATACD_VOLUMENAME))	CM->SetString(DATACD_VOLUMENAME, CM->GetString(S_VOLUMENAME)->String());
	if (!CM->HasData(DATACD_MULTISESSION))	CM->SetBool(DATACD_MULTISESSION, false);
	// AUDIO-CD
	if (!CM->HasData(AUDIOCD_WRITECDTEXT))	CM->SetBool(AUDIOCD_WRITECDTEXT, false);
	if (!CM->HasData(AUDIOCD_SCMS))		CM->SetBool(AUDIOCD_SCMS, false);
	if (!CM->HasData(AUDIOCD_ALBUMNAME))	CM->SetString(AUDIOCD_ALBUMNAME, "");
	// BOOTABLE-CD
	if (!CM->HasData(BOOTCD_FILESYSTEM))	CM->SetInt32(BOOTCD_FILESYSTEM, CM->GetInt32(S_FILESYSTEM));
	if (!CM->HasData(BOOTCD_VOLUMENAME))	CM->SetString(BOOTCD_VOLUMENAME, CM->GetString(S_VOLUMENAME)->String());
	if (!CM->HasData(BOOTCD_FLOPPYIMAGE))	CM->SetString(BOOTCD_FLOPPYIMAGE, "");
	// CD EXTRA
	if (!CM->HasData(CDEXTRA_WRITECDTEXT))	CM->SetBool(CDEXTRA_WRITECDTEXT, false);
	if (!CM->HasData(CDEXTRA_ALBUMNAME))	CM->SetString(CDEXTRA_ALBUMNAME, "");
	if (!CM->HasData(CDEXTRA_DATAIMAGE))	CM->SetString(CDEXTRA_DATAIMAGE, "");
	if (!CM->HasData(CDEXTRA_FILESYSTEM))	CM->SetInt32(CDEXTRA_FILESYSTEM, CM->GetInt32(S_FILESYSTEM));
	if (!CM->HasData(CDEXTRA_VOLUMENAME))	CM->SetString(CDEXTRA_VOLUMENAME, CM->GetString(S_VOLUMENAME)->String());
	// DATA-DVD
	if (!CM->HasData(DVDVIDEO_FILESYSTEM))	CM->SetInt32(DVDVIDEO_FILESYSTEM, CM->GetInt32(S_FILESYSTEM));
	if (!CM->HasData(DVDVIDEO_VOLUMENAME))	CM->SetString(DVDVIDEO_VOLUMENAME, CM->GetString(S_VOLUMENAME)->String());


	if (!CM->HasData(PATHTOTHEIMAGE))		CM->SetString(PATHTOTHEIMAGE, "/boot/home/image.img");
	
	// ProtocolWindow Settings
	if (!CM->HasData(CDRECORD_LOGWIN_VISIBLE))	CM->SetBool(CDRECORD_LOGWIN_VISIBLE, false);
	if (!CM->HasData(MKISOFS_LOGWIN_VISIBLE))		CM->SetBool(MKISOFS_LOGWIN_VISIBLE, false);
	if (!CM->HasData(CDRECORD_LOG))			CM->SetBool(CDRECORD_LOG, false);
	if (!CM->HasData(MKISOFS_LOG))				CM->SetBool(MKISOFS_LOG, false);
	if (!CM->HasData(CDRECORD_LOGFILENAME))		CM->SetString(CDRECORD_LOGFILENAME, "");
	if (!CM->HasData(MKISOFS_LOGFILENAME))		CM->SetString(MKISOFS_LOGFILENAME, "");

	// CLV-column widths
	if (!CM->HasData(FILENAMECOLUMN_WIDTH))		CM->SetInt16(FILENAMECOLUMN_WIDTH, 200);
	if (!CM->HasData(SIZECOLUMN_WIDTH))			CM->SetInt16(SIZECOLUMN_WIDTH, 57);
	if (!CM->HasData(TRACKNUMBERCOLUMN_WIDTH))	CM->SetInt16(TRACKNUMBERCOLUMN_WIDTH, 47);
	if (!CM->HasData(TRACKNAMECOLUMN_WIDTH))	CM->SetInt16(TRACKNAMECOLUMN_WIDTH, 200);
	if (!CM->HasData(LENGTHCOLUMN_WIDTH))		CM->SetInt16(LENGTHCOLUMN_WIDTH, 80);
	if (!CM->HasData(BYTESCOLUMN_WIDTH))		CM->SetInt16(BYTESCOLUMN_WIDTH, 50);
	if (!CM->HasData(INDEXCOLUMN_WIDTH))		CM->SetInt16(INDEXCOLUMN_WIDTH, 150);
	if (!CM->HasData(CDTEXTCOLUMN_WIDTH))		CM->SetInt16(CDTEXTCOLUMN_WIDTH, 150);


	CM->WriteConfiguration();
	CM->ReadConfiguration();
	
	// apply values to GUI elements...
	/*heliosCV->SetBurnproof(CM->GetBool(BURNPROOF));
	heliosCV->SetSimulation(CM->GetBool(SIMULATION));
	heliosCV->SetFixating(CM->GetBool(FIXATE_DISC));
	heliosCV->SetDAO(CM->GetBool(USE_DAO));
	heliosCV->SetOverburning(CM->GetBool(OVERBURNING));
	heliosCV->SetWriterSpeed(CM->GetInt32(WRITINGSPEED));
	heliosCV->SetFIFOSize(CM->GetInt8(FIFOBUFFERSIZE));
	heliosCV->SetForcedErasing(CM->GetBool(FORCEDERASING));*/
	heliosCV->SetSettings(msg);

	deviceCV->SetWriterEjected(CM->GetBool(EJECT_WRITER));
	deviceCV->SetReaderEjected(CM->GetBool(EJECT_READER));
	deviceCV->SetWriterNumber(CM->GetInt8(WRITERDEVICE));
	deviceCV->SetTwoDriveCopy(CM->GetBool(TWODRIVECOPY));
	deviceCV->SetReaderNumber(CM->GetInt8(READERDEVICE));
	deviceCV->SetScanMethod(CM->GetInt8(SCANNINGMETHOD));
	deviceCV->SetShowAllDevices(CM->GetBool(SHOWALLDEVICES));
	
	pathCV->SetTemporaryPath(CM->GetString(TEMPORARY_PATH)->String());
	pathCV->SetProjectPath(CM->GetString(PROJECT_PATH)->String());
	pathCV->SetDefaultImageName(CM->GetString(DEFIMAGENAME)->String());
	pathCV->SetDefaultMountPoint(CM->GetString(DEFMOUNTPOINT)->String());
	
	imageCV->SetPublisher(CM->GetString(I_PUBLISHER)->String());
	imageCV->SetPreparer(CM->GetString(I_PREPARER)->String());
	imageCV->SetApplication(CM->GetString(I_APPLICATION)->String());

	fspanelCV->SetISO9660(CM->GetBool(F_ISO9660_CB));
	fspanelCV->SetJoliet(CM->GetBool(F_JOLIET_CB));
	fspanelCV->SetRockRidge(CM->GetBool(F_RR_CB));
	fspanelCV->SetUDF(CM->GetBool(F_UDF_CB));
	fspanelCV->SetHFS(CM->GetBool(F_HFS_CB));
	fspanelCV->SetISO96608Dot3(CM->GetBool(F_83FN_RB));
	fspanelCV->SetISO966031(CM->GetBool(F_31FN_RB));
	fspanelCV->SetISO966037(CM->GetBool(F_37FN_RB));
	fspanelCV->SetJolietDefault(CM->GetBool(F_JOLDEF_RB));
	fspanelCV->SetJolietUnicode(CM->GetBool(F_JOLUNI_RB));
	fspanelCV->SetRockRidgeDefault(CM->GetBool(F_RRDEF_RB));
	fspanelCV->SetRockRidgeRationalized(CM->GetBool(F_RRRAT_RB));
	fspanelCV->SetUDFDefault(CM->GetBool(F_UDFDEF_RB));
	fspanelCV->SetUDFDVDVideo(CM->GetBool(F_UDFDVD_RB));
	fspanelCV->SetMapping(CM->GetBool(F_MAPPING_CB));
	fspanelCV->SetMappingFile(CM->GetString(F_MAPPING_TC)->String());

	standardCV->SetFilesystem(CM->GetInt32(S_FILESYSTEM));	
	standardCV->SetVolumeName(CM->GetString(S_VOLUMENAME)->String());
	standardCV->SetCDType(CM->GetInt8(S_CDTYPE));
	standardCV->SetPublisher(CM->GetString(S_PUBLISHER)->String());
	standardCV->SetPreparer(CM->GetString(S_PREPARER)->String());
	standardCV->SetApplication(CM->GetString(S_APPLICATION)->String());
	//standardCV->SetTemporaryPath(CM->GetString(S_TEMPPATH)->String());
	//standardCV->SetProjectPath(CM->GetString(S_PROJECTPATH)->String());
	
	applicationCV->SetAutoUpdating(CM->GetBool(APP_AUTOUPDATE));
	applicationCV->SetTimeout(CM->GetBool(APP_TIMEOUT));
	applicationCV->SetUpdateFrequency(CM->GetInt16(APP_UPDATEFREQUENCY));
	applicationCV->SetTimeoutLength(CM->GetInt16(APP_TIMEOUTLENGTH));

	soundCV->SetFinishedWritingSound(CM->GetBool(BEEPWHENWRITTEN));
	soundCV->SetFinishedImageSound(CM->GetBool(BEEPWHENIMAGE));
	soundCV->SetErrorSound(CM->GetBool(BEEPWHENERROR));

	saveCV->SetStreamed(CM->GetBool(STREAMING_MODE));
	saveCV->SetSaveWhenQuit(CM->GetBool(SAVEWHENQUIT));
	saveCV->SetExitMode((exit_modes)CM->GetInt8(EXITMODE));
	saveCV->SetLanguage(CM->GetInt32(LANGUAGE_ID));
	saveCV->SetAutoUpdate(CM->GetBool(AUTOUPDATE));
	saveCV->SetBalloonHelp(CM->GetBool(BUBBLEHELP));

	
	if (configW->Lock()) {
		colorCV->SetColor(0, CM->GetColor(BURNING_STATUS_BAR_COLOR));
			if (window5->Lock()) {
				window5->SetBarColor(colorCV->GetColor(0));
				window5->Unlock();
			}
		colorCV->SetColor(1, CM->GetColor(CDSIZE_STATUS_BAR_COLOR));
			if (window1->Lock()) {
				window1->view1->box2->SetBarColor(colorCV->GetColor(1));
				window1->Unlock();
			}
		colorCV->SetColor(2, CM->GetColor(LOGWIN_BG_COLOR));
		colorCV->SetColor(3, CM->GetColor(LOGWIN_FG_COLOR));
			if (cdrecord_output->Lock()) {
				cdrecord_output->SetColors(colorCV->GetColor(3), colorCV->GetColor(2));
				cdrecord_output->Unlock();
			}
			if (mkisofs_output->Lock()) {
				mkisofs_output->SetColors(colorCV->GetColor(3), colorCV->GetColor(2));
				mkisofs_output->Unlock();
			}
		colorCV->SetColor(4, CM->GetColor(LISTSELECTION_COLOR));
			if (window1->Lock()) {
				window1->view1->listview1->SetSelectionColor(colorCV->GetColor(4));
				window1->Unlock();
			}
		colorCV->SetColor(5, CM->GetColor(LISTBACKGROUND_COLOR));
			if (window1->Lock()) {
				window1->view1->listview1->SetBackgroundColor(colorCV->GetColor(5));
				window1->Unlock();
			}
		configW->Unlock();
	}
	
	// CD-MODE-SPECIFIC...
	// DATA-CD
	window1->view1->SetVolumeName(CM->GetString(DATACD_VOLUMENAME)->String());
	window1->view1->SetMultisession(CM->GetBool(DATACD_MULTISESSION));
	window1->view1->SetFilesystem(CM->GetInt32(DATACD_FILESYSTEM));
	// AUDIO-CD
	window1->view1->SetCDText(CM->GetBool(AUDIOCD_WRITECDTEXT));
	window1->view1->SetProtection(CM->GetBool(AUDIOCD_SCMS));
	window1->view1->SetCDAlbum(CM->GetString(AUDIOCD_ALBUMNAME)->String());
	// BOOTABLE-CD
	window1->view1->SetVolumeNameBootCD(CM->GetString(BOOTCD_VOLUMENAME)->String());
	window1->view1->SetImageNameBootCD(CM->GetString(BOOTCD_FLOPPYIMAGE)->String());
	window1->view1->SetFilesystemBootCD(CM->GetInt32(BOOTCD_FILESYSTEM));
	// CD EXTRA
	window1->view1->SetImageNameCDExtra(CM->GetString(CDEXTRA_DATAIMAGE)->String());
	window1->view1->SetCDTextCDExtra(CM->GetBool(CDEXTRA_WRITECDTEXT));
	window1->view1->SetCDAlbumCDExtra(CM->GetString(CDEXTRA_ALBUMNAME)->String());
	window1->view1->SetVolumeNameCDExtra(CM->GetString(CDEXTRA_VOLUMENAME)->String());
	window1->view1->SetFilesystemCDExtra(CM->GetInt32(CDEXTRA_FILESYSTEM));
	// DATA-DVD
	window1->view1->SetVolumeNameDVDVideo(CM->GetString(DVDVIDEO_VOLUMENAME)->String());
	window1->view1->SetFilesystemDVDVideo(CM->GetInt32(DVDVIDEO_FILESYSTEM));
	

	if (CM->HasData(MAINWINDOW_RECT)) {
		window1->ResizeTo(CM->GetRect(MAINWINDOW_RECT)->Width(), CM->GetRect(MAINWINDOW_RECT)->Height());
		window1->MoveTo(CM->GetRect(MAINWINDOW_RECT)->left, CM->GetRect(MAINWINDOW_RECT)->top);
	}
	if (CM->HasData(WINDOW2_RECT)) {
		window2->ResizeTo(CM->GetRect(WINDOW2_RECT)->Width(), CM->GetRect(WINDOW2_RECT)->Height());
		window2->MoveTo(CM->GetRect(WINDOW2_RECT)->left, CM->GetRect(WINDOW2_RECT)->top);
	}
	if (CM->HasData(PREFERENCESWINDOW_RECT)) {
		configW->ResizeTo(CM->GetRect(PREFERENCESWINDOW_RECT)->Width(), CM->GetRect(PREFERENCESWINDOW_RECT)->Height());
		configW->MoveTo(CM->GetRect(PREFERENCESWINDOW_RECT)->left, CM->GetRect(PREFERENCESWINDOW_RECT)->top);
	}
	if (CM->HasData(CDRECORDLOGWINDOW_RECT)) {
		cdrecord_output->ResizeTo(CM->GetRect(CDRECORDLOGWINDOW_RECT)->Width(), CM->GetRect(CDRECORDLOGWINDOW_RECT)->Height());
		cdrecord_output->MoveTo(CM->GetRect(CDRECORDLOGWINDOW_RECT)->left, CM->GetRect(CDRECORDLOGWINDOW_RECT)->top);
	}
	if (CM->HasData(MKISOFSLOGWINDOW_RECT)) {
		mkisofs_output->ResizeTo(CM->GetRect(MKISOFSLOGWINDOW_RECT)->Width(), CM->GetRect(MKISOFSLOGWINDOW_RECT)->Height());
		mkisofs_output->MoveTo(CM->GetRect(MKISOFSLOGWINDOW_RECT)->left, CM->GetRect(MKISOFSLOGWINDOW_RECT)->top);
	}

	// ProtocolWindow Settings
	if (CM->GetBool(CDRECORD_LOGWIN_VISIBLE))	cdrecord_output->Show();
	if (CM->GetBool(MKISOFS_LOGWIN_VISIBLE))	mkisofs_output->Show();
	
	if (cdrecord_output->Lock()) {
		cdrecord_output->SetLogToFile(CM->GetBool(CDRECORD_LOG));
		cdrecord_output->SetLogFileName(CM->GetString(CDRECORD_LOGFILENAME)->String());
		cdrecord_output->Unlock();
	}
	if (mkisofs_output->Lock()) {
		mkisofs_output->SetLogToFile(CM->GetBool(MKISOFS_LOG));
		mkisofs_output->SetLogFileName(CM->GetString(MKISOFS_LOGFILENAME)->String());
		mkisofs_output->Unlock();
	}

	return;
}


void
Application1::Calculate_time() {
	off_t size2=window1->view1->box2->GetCDSize();
	if (heliosCV->IsFixating()) size2+=13824000;
	uint64 r=(2352*(uint64)size2/(2048*176400*(uint64)heliosCV->GetWriterSpeed()));
	BString str="";
	r+=10; // 10 Sekunden Wartezeit von cdrecord
	r+=10; // und 10 sekunden wartezeit für spinup/spindown
	r+=10; // und dann nochmal 10 sekunden, für alle fälle...
	uint64 u=r-60*(uint32)(r/60);
	char *s="";
	
	if (u<10) s="0"; else s="";
	str << (off_t)(r/60) << "m " << s << (off_t)u << "s";
	window1->Lock();
	window1->view1->box2->stringview6->SetText(str.String());
	window1->Unlock();
}


void Application1::GetFolder(BDirectory dir) {

	int32 c=dir.CountEntries();
	BEntry entry;
	
	if (c>0)
	for (int32 i=0; i<c; i++) {
		dir.GetNextEntry(&entry, true);
		if (entry.IsDirectory())
			{
			directories++;
			GetFolder(BDirectory(&entry));
			}
			else
			files++;
	}
}


void Application1::UpdateTrackNumbers() {

//	RemoveFileTrackNumbers();
	window1->Lock();
	if ((window1->view1->CDType()==AUDIOCD_INDEX) || (window1->view1->CDType()==CDEXTRA_INDEX)) {
		for (int i=0; i<window1->view1->listview1->iCountItems();i++) {
			((AudioRow *)(window1->view1->listview1->RowAt(i)))->SetTrackNumber(i+1);
//			window1->view1->listview1->InvalidateItem(i);
		}
		if (window1->view1->listview1->iCountItems()>0)
			((AudioRow *)(window1->view1->listview1->RowAt(0)))->SetPregap(150);
	}
	window1->Unlock();
//	AddFileTrackNumbers();
}


int Application1::DeviceStatus(const char *path) {

	if ( path == NULL || strlen(path) == 0 )
		return E_ERROR;
		
	// okay, path point to a device
	int 		fd;
	status_t	status;
	status_t	media_status;

	fd = open(path, 0);
	if (fd < 0 )
		return E_ERROR;
		
	status = ioctl(fd, B_GET_MEDIA_STATUS, &media_status, sizeof(media_status));
	close(fd);
	
	if ( status != B_OK )
		return E_ERROR;
	
	switch(media_status)
		{
		case B_NO_ERROR:
			return E_NO_ERROR;
		case B_DEV_NO_MEDIA:
			return E_NO_DISC;
		case B_DEV_NOT_READY:
			return E_NOT_READY;
		case B_DEV_MEDIA_CHANGED:
			return E_MEDIA_CHANGED;
		case B_DEV_DOOR_OPEN:
			return E_DOOR_OPEN;
		default:
			return E_ERROR;
		};
}






void Application1::DeleteAllIn(BDirectory dir) {

	int32 c=dir.CountEntries();
	BEntry entry;
	
	if (c>0)
	for (int32 i=0; i<c; i++) {
		dir.GetNextEntry(&entry, false);
		if (entry.IsDirectory()) {
			DeleteAllIn(BDirectory(&entry));
			entry.Remove();
		}
		else
			entry.Remove();
	}
}


//void Application1::ShowOpenPanel() {
//	
//	BEntry		*entry=new BEntry(pathCV->GetProjectPath(), true);
//	entry_ref	ref;
//	
//	entry->GetRef(&ref);
//	//if (filepanel!=NULL) delete filepanel;
//	filepanel=new BFilePanel(B_OPEN_PANEL, &be_app_messenger, new entry_ref(ref),
//				B_FILE_NODE, false, new BMessage(OPEN_FPMSG));
//	filepanel->SetButtonLabel(B_DEFAULT_BUTTON, H->L("C_OPEN"));
//	filepanel->SetButtonLabel(B_CANCEL_BUTTON, H->L("TCANCEL"));
//	filepanel->Show();
//
//	entry->Unset();
//	delete entry;
//}


//void Application1::ShowFolderPanel() {
//	
//	BEntry		*entry=new BEntry(pathCV->GetProjectPath(), true);
//	entry_ref	ref;
//	
//	entry->GetRef(&ref);
//	//if (filepanel!=NULL) delete filepanel;
//	filepanel=new BFilePanel(B_OPEN_PANEL, &be_app_messenger, new entry_ref(ref),
//				B_DIRECTORY_NODE, false, new BMessage(FOLDER_FPMSG));
//	filepanel->SetButtonLabel(B_DEFAULT_BUTTON, H->L("C_SELECT"));
//	filepanel->SetButtonLabel(B_CANCEL_BUTTON, H->L("TCANCEL"));
//	
//	// display some helpful text in the background view of the file panel,
//	// for example for what reason this panel popped up.
//	filepanel->Window()->Lock();
//	BView *background = filepanel->Window()->ChildAt(0);
//	BView *files = background->FindView("PoseView");
//	BView *cancel_button = background->FindView("cancel button");
//	//background->LockLooper();
//	BStringView	*filepanelSV=new BStringView(BRect(files->Frame().left-2, cancel_button->Frame().top, cancel_button->Frame().left-8, cancel_button->Frame().bottom-1), "infoSV",  H->L("TFOLDERPANELHELP"), B_FOLLOW_LEFT | B_FOLLOW_BOTTOM);
//	background->AddChild(filepanelSV);
//	//background->UnlockLooper();
//	filepanel->Window()->Unlock();
//	filepanel->Show();
//
//	entry->Unset();
//	delete entry;
//}


void Application1::AddToRecent(const char *filename) {
	int	i;
	// check if there is already a project with the same name
	for (i=0; i<CM->CountStrings(RECENTPROJECTS_LIST); i++)
		if (CM->GetStringI(RECENTPROJECTS_LIST, i)->Compare(filename)==0) {
			// ok, it's already there...
			return;
		}
	//TODO: should be moved to the top of the list...

	// shift projects in array
	if (CM->CountStrings(RECENTPROJECTS_LIST)>=9)
		for (i=0; i<CM->CountStrings(RECENTPROJECTS_LIST); i++) {
			CM->SetStringI(RECENTPROJECTS_LIST, CM->GetStringI(RECENTPROJECTS_LIST, i+1), i);
		}
	CM->SetStringI(RECENTPROJECTS_LIST,new BString(filename), CM->CountStrings(RECENTPROJECTS_LIST));
}


void Application1::ShowRecentProjects() {

	BEntry	entry;
	char	*filename;
	
	filename=(char *)malloc(B_FILE_NAME_LENGTH);
	window1->Lock();
	while (window1->view1->openrecentSM->CountItems()>0) delete window1->view1->openrecentSM->RemoveItem(0L);
	for (int i=0; i<CM->CountStrings(RECENTPROJECTS_LIST); i++) {
		if (CM->GetStringI(RECENTPROJECTS_LIST, i)->Length()>0) {
			// add if project exists, only
			if (entry.SetTo(CM->GetStringI(RECENTPROJECTS_LIST, i)->String(), true)==B_OK) {
					entry.GetName(filename);
					BMessage *msg=new BMessage(OPEN_FPMSG);		

					entry_ref	*ref=new entry_ref();
					entry.GetRef(ref);
					
					msg->AddRef("refs", ref);
					BMenuItem	*mi=new BMenuItem(filename, msg);
					mi->SetTarget(be_app_messenger);
					if (!entry.Exists()) mi->SetEnabled(false);
					window1->view1->openrecentSM->AddItem(mi, 0);
			}
		}
	}
	window1->Unlock();
	free(filename);
}


const char *Application1::SuggestProjectName() {
	switch(window1->view1->CDType()) {
		case DATACD_INDEX: {
			return window1->view1->GetVolumeName();
		}
		
		case AUDIOCD_INDEX: {
			return window1->view1->GetCDAlbum();
		}
		
		case BOOTABLECD_INDEX: {
			return window1->view1->GetVolumeNameBootCD();
		}
		
		case CDEXTRA_INDEX: {
			return window1->view1->GetVolumeNameCDExtra();
		}
		
		case DVDVIDEO_INDEX: {
			return window1->view1->GetVolumeNameDVDVideo();
		}
		
		default: {
			return "";
		}
	}
	return "";
}


//void Application1::ShowSavePanel() {
//	
//	BEntry		*entry=new BEntry(pathCV->GetProjectPath(), true);
//	entry_ref	ref;
//	
//	entry->GetRef(&ref);
//	//if (filepanel!=NULL) delete filepanel;
//	filepanel=new BFilePanel(B_SAVE_PANEL, &be_app_messenger, new entry_ref(ref),
//				0, false, new BMessage(SAVE_FPMSG));
//	filepanel->SetButtonLabel(B_DEFAULT_BUTTON, H->L("C_SAVE"));
//	filepanel->SetButtonLabel(B_CANCEL_BUTTON, H->L("TCANCEL"));
//	filepanel->SetSaveText(SuggestProjectName());
//	filepanel->Show();
//	
//	entry->Unset();
//	delete entry;
//}

void Application1::LoadProject(BEntry *entry) {
	if (!entry->Exists()) return;
	

	BFile 	*file=new BFile(entry, B_READ_ONLY);

	// check for correct file type...
	BNodeInfo *info=new BNodeInfo(file);
	char	type[B_FILE_NAME_LENGTH];
	info->GetType(type);
	delete info;
	
		if (strcmp(type, HELIOSPROJECTMIMETYPE)!=0) {
			ErrorBox	*eb=new ErrorBox(E_ORANGE_COLOR, H->L("TERROR"), H->L("Error:No Project File"), H->L("TOK"));
			eb->SetShortcut(0, B_ENTER);
			eb->Go();
			file->Unset();
			delete file;
			return;
		}
	


	BPath	path;
	int16	cdtype;
	BString	str;
	char	projectname[B_FILE_NAME_LENGTH];
	
	window1->Lock();
	entry->GetPath(&path);

	ConfigManager	*projM=new ConfigManager(path.Path());
	// determine which sort of project we will load
	if (projM->HasData("cdtype")) cdtype=projM->GetInt16("cdtype");
	
	// set new window title (containing the project's name)
	entry->GetName(projectname);
	BString *s=new BString(H->L("TWINDOWTITLE"));
	s->Append(" | ");
	s->Append(projectname);
	window1->SetTitle(s->String());
	delete s;

	// load general data
	if (projM->HasData("current_folder"))
		current_folder=projM->GetString("current_folder")->String();
	if (projM->HasData("base_folder"))
		base_folder=projM->GetString("base_folder")->String();
	if (projM->HasData("publisher"))
		imageCV->SetPublisher(projM->GetString("publisher")->String());
	if (projM->HasData("preparer"))
		imageCV->SetPreparer(projM->GetString("preparer")->String());
	if (projM->HasData("application"))
		imageCV->SetApplication(projM->GetString("application")->String());

	
	
	window1->view1->SetCDType(cdtype);
	Update_DATA_AUDIO();
	// load cd-mode-specific data
	switch(cdtype) {
		case DATACD_INDEX: {
			if (projM->HasData("filesystem"))
				window1->view1->SetFilesystem(projM->GetInt16("filesystem"));
			if (projM->HasData("volumename"))
				window1->view1->SetVolumeName(projM->GetString("volumename")->String());
			if (projM->HasData("multisession"))
				window1->view1->SetMultisession(projM->GetBool("multisession"));
			break;
		}
		case AUDIOCD_INDEX: {
			if (projM->HasData("writecdtext"))
				window1->view1->SetCDText(projM->GetBool("writecdtext"));
			if (projM->HasData("copyprotection"))
				window1->view1->SetProtection(projM->GetBool("copyprotection"));
			if (projM->HasData("cdtextalbum"))
				window1->view1->SetCDAlbum(projM->GetString("cdtextalbum")->String());
				
			for (int32 t=0; t<projM->CountStrings("filename"); t++) {
				AudioRow *item=new AudioRow(t+1, projM->GetStringI("filename", t)->String(), *(projM->GetInt64("pause", t)), (off_t)(*(projM->GetInt64("size", t))), (off_t)(*(projM->GetInt64("length", t))));
				if (projM->GetInt64("startF", t)) {
					item->SetStartFrame(*(projM->GetInt64("startF", t)));
				} else {
					item->SetStartFrame((int64)0);
				}
				if (projM->GetInt64("endF", t)) {
					item->SetEndFrame(*(projM->GetInt64("endF", t)));
				} else {
					item->SetEndFrame(75*60*60*4);
				}

				BList	*list=new BList();
				float	*f;
				char 	dataname[50];
				sprintf(dataname, "indices track %ld", t+1);
				for(int32 i=0; (f=projM->GetFloat(dataname, i))!=NULL; i++) {
					list->AddItem((void *)f);
				}
				item->SetIndexList(list);
				delete list;
				item->SetCDTitle(projM->GetStringI("cdtext title", t)->String());
				str=item->GetFilename();
				str.Prepend(base_folder);
				if (FileExists(str.String()))
					window1->view1->listview1->AddRow(item);
				else
					delete item;
			}
			break;
		}
		case BOOTABLECD_INDEX: {
			if (projM->HasData("filesystem"))
				window1->view1->SetFilesystemBootCD(projM->GetInt16("filesystem"));
			if (projM->HasData("volumename"))
				window1->view1->SetVolumeNameBootCD(projM->GetString("volumename")->String());
			if (projM->HasData("imagename"))
				window1->view1->SetImageNameBootCD(projM->GetString("imagename")->String());
			break;
		}
		case CDEXTRA_INDEX: {
			if (projM->HasData("imagename"))
				window1->view1->SetImageNameCDExtra(projM->GetString("imagename")->String());
			if (projM->HasData("writecdtext"))
				window1->view1->SetCDTextCDExtra(projM->GetBool("writecdtext"));
			if (projM->HasData("cdtextalbum"))
				window1->view1->SetCDAlbumCDExtra(projM->GetString("cdtextalbum")->String());
	
			for (int32 t=0; t<projM->CountStrings("filename"); t++) {
				AudioRow *item=new AudioRow(t+1, projM->GetStringI("filename", t)->String(), *(projM->GetInt64("pause", t)), (off_t)(*(projM->GetInt64("size", t))), (off_t)(*(projM->GetInt64("length", t))));
				if (projM->GetInt64("startF", t)) {
					item->SetStartFrame(*(projM->GetInt64("startF", t)));
				} else {
					item->SetStartFrame((int64)0);
				}
				if (projM->GetInt64("endF", t)) {
					item->SetEndFrame(*(projM->GetInt64("endF", t)));
				} else {
					item->SetEndFrame(75*60*60*4);
				}

				BList	*list=new BList();
				float	*f;
				char 	dataname[50];
				sprintf(dataname, "indices track %ld", t+1);
				for(int32 i=0; (f=projM->GetFloat(dataname, i))!=NULL; i++) {
					list->AddItem((void *)f);
				}
				item->SetIndexList(list);
				delete list;
				item->SetCDTitle(projM->GetStringI("cdtext title", t)->String());
				str=item->GetFilename();
				str.Prepend(base_folder);
				if (FileExists(str.String()))
					window1->view1->listview1->AddRow(item);
				else
					delete item;
			}
			break;
		}

#ifdef INCLUDE_DVD

		case DVDVIDEO_INDEX: {
			if (projM->HasData("filesystem"))
				window1->view1->SetFilesystemDVDVideo(projM->GetInt16("filesystem"));
			if (projM->HasData("volumename"))
				window1->view1->SetVolumeNameDVDVideo(projM->GetString("volumename")->String());
			break;
		}

#endif

		default: {
			break;
		}
	}
	
	UpdateTrackNumbers();
	window1->Unlock();

	CM->SetString(PATH_TO_LAST_PROJECT, path.Path());
	AddToRecent(path.Path());
	ShowRecentProjects();
	currentproject=path.Path();
	project_has_changed=false;
	
	printf("currentproject=%s\n", currentproject.String());
	
	if (saveCV->AutoUpdate())
		window1->view1->box2->Refresh();
	
	file->Unset();
	delete projM;
	delete file;
}


void Application1::SaveProject(BEntry *entry, bool export_project) {
	BFile 	*file=new BFile(entry, B_READ_WRITE | B_ERASE_FILE | B_CREATE_FILE);
	BPath path;
	char	projectname[B_FILE_NAME_LENGTH];
	BString	subdir=current_folder;
	
	subdir.RemoveFirst(base_folder);
	
	window1->Lock();
	entry->GetName(projectname);

	// set MIME type
	BNode *node=new BNode(entry);
	BNodeInfo	*nodeinfo=new BNodeInfo(node);
	
	nodeinfo->SetType(HELIOSPROJECTMIMETYPE);

	node->Unset();
	delete nodeinfo;
	delete node;
	
	
	// rename the base folder for the newly saved project
	// and set the base_folder-variable to the new path
	BString bf=base_folder;
	bf.RemoveLast("/");
	BEntry *basef=new BEntry(bf.String(), false);
	BString temp=projectname;
	printf("projectname=%s\n", projectname);
	temp.Prepend("Folder_of_");
	if (!export_project) {
		basef->Rename(temp.String(), true);
		basef->GetPath(&path);
		base_folder=path.Path();
	} else {
		temp.Prepend("/");
		temp.Prepend(pathCV->GetProjectPath());
		//temp=*AppendTrailingSlash(&temp);
		printf("export-folder: [%s]\n", temp.String());
		create_directory(temp.String(), 0777);
		currentproject=pathCV->GetProjectPath();
		AppendTrailingSlash(&currentproject);
		currentproject.Append(projectname);
		base_folder=temp;
	}
	//base_folder=temp;
	//if (base_folder[base_folder.Length()-1]!='/') base_folder.Append("/");
	AppendTrailingSlash(&base_folder);
	basef->Unset();
	delete basef;
	current_folder.SetTo(base_folder.String());
	if (!export_project)
		current_folder.Append(subdir);
	//printf("current_folder=\"%s\"\nbase_folder=\"%s\"\n", current_folder.String(), base_folder.String());
	entry->GetPath(&path);
	CM->SetString(PATH_TO_LAST_PROJECT, path.Path());
	//sprintf(settings._lastprojectfile,"%s",path.Path());
	if (!export_project) {
		AddToRecent(path.Path());
		ShowRecentProjects();
		currentproject=path.Path();
	}
	file->Unset();
	delete file;

	entry->GetName(projectname);
	BString *s=new BString(H->L("TWINDOWTITLE"));
	s->Append(" | ");
	s->Append(projectname);
	window1->SetTitle(s->String());
	delete s;
	window1->Unlock();
	project_has_changed=false;


	ConfigManager	*projM=new ConfigManager(path.Path());
	
	projM->SetInt16("cdtype", window1->view1->CDType());
	projM->SetString("current_folder", current_folder.String());
	projM->SetString("base_folder", base_folder.String());
	projM->SetString("publisher", imageCV->GetPublisher());
	projM->SetString("preparer", imageCV->GetPreparer());
	projM->SetString("application", imageCV->GetApplication());


	// save cd-mode-specific data
	switch(window1->view1->CDType()) {
		case DATACD_INDEX: {
			projM->SetInt16("filesystem", (int16)window1->view1->GetFilesystem());
			projM->SetString("volumename", window1->view1->GetVolumeName());
			projM->SetBool("multisession", window1->view1->IsMultisession());
			break;
		}
		case AUDIOCD_INDEX: {
			projM->SetBool("writecdtext", window1->view1->WriteCDText());
			projM->SetBool("copyprotection", window1->view1->ProtectAudioCD());
			projM->SetString("cdtextalbum", window1->view1->GetCDAlbum());
				
			for (int32 i=0; i<window1->view1->listview1->iCountItems(); i++) {
				AudioRow	*ar=(AudioRow *)window1->view1->listview1->RowAt(i);
				
				projM->SetStringI("filename", ar->GetFilename(), i);
				projM->SetStringI("cdtext title", ar->GetCDTitle(), i);
				projM->SetInt64("pause", (int64)ar->GetPregap(), i);
				projM->SetInt64("size", (int64)ar->GetBytes(), i);
				projM->SetInt64("length", (int64)ar->GetLength(), i);
				
				projM->SetInt64("startF", ar->GetStartFrame(), i);
				projM->SetInt64("endF", ar->GetEndFrame(), i);
				
				BList	*list=ar->GetIndexList();
				char 	dataname[50];
				sprintf(dataname, "indices track %ld", i+1);
				
				for (int32 j=0; j<list->CountItems(); j++) {
					projM->SetFloat(dataname, (*((float *)(list->ItemAt(j)))), j);
				}
			}
			break;
		}
		case BOOTABLECD_INDEX: {
			projM->SetInt16("filesystem", (int16)window1->view1->GetFilesystemBootCD());
			projM->SetString("volumename", window1->view1->GetVolumeNameBootCD());
			projM->SetString("imagename", window1->view1->GetImageNameBootCD());
			break;
		}
		case CDEXTRA_INDEX: {
			projM->SetString("imagename", window1->view1->GetImageNameCDExtra());
			projM->SetBool("writecdtext", window1->view1->WriteCDTextCDExtra());
			projM->SetString("cdtextalbum", window1->view1->GetCDAlbumCDExtra());
	
			for (int32 i=0; i<window1->view1->listview1->iCountItems(); i++) {
				AudioRow	*ar=(AudioRow *)window1->view1->listview1->RowAt(i);
				
				projM->SetStringI("filename", ar->GetFilename(), i);
				projM->SetStringI("cdtext title", ar->GetCDTitle(), i);
				projM->SetInt64("pause", (int64)ar->GetPregap(), i);
				projM->SetInt64("size", (int64)ar->GetBytes(), i);
				projM->SetInt64("length", (int64)ar->GetLength(), i);
				
				projM->SetInt64("startF", ar->GetStartFrame(), i);
				projM->SetInt64("endF", ar->GetEndFrame(), i);

				BList	*list=ar->GetIndexList();
				char 	dataname[50];
				sprintf(dataname, "indices track %ld", i+1);
				
				for (int32 j=0; j<list->CountItems(); j++) {
					projM->SetFloat(dataname, (*((float *)(list->ItemAt(j)))), j);
				}
			}
			break;
		}

#ifdef INCLUDE_DVD

		case DVDVIDEO_INDEX: {
			projM->SetInt16("filesystem", (int16)window1->view1->GetFilesystemDVDVideo());
			projM->SetString("volumename", window1->view1->GetVolumeNameDVDVideo());
			break;
		}

#endif

		default: {
			break;
		}
	}

	projM->WriteConfiguration();
	delete projM;
	
	//base_folder=oldbase_folder;
	//current_folder=oldcurrent_folder;
	// saved at quit?
	// so go on quitting Helios...
	release_sem(quitandsave_sem);
}




void Application1::NewProject() {
		
		imageCV->SetPublisher(standardCV->GetPublisher());
		imageCV->SetPreparer(standardCV->GetPreparer());
		imageCV->SetApplication(standardCV->GetApplication());
		
		//settingswindow->heliospathTC->SetText(base_folder.String());
		window1->Lock();
		window1->view1->SetVolumeName(standardCV->GetVolumeName());
		window1->view1->SetVolumeNameBootCD(standardCV->GetVolumeName());
		window1->view1->SetCDType(standardCV->GetCDType());
		window1->view1->SetFilesystemBootCD(standardCV->GetFilesystem());
		window1->view1->SetFilesystem(standardCV->GetFilesystem());

		BString	*p=new BString(pathCV->GetProjectPath());
		
		AppendTrailingSlash(p);

		base_folder=p->String();
		base_folder.Append(STANDARDPROJECTPATH);
		current_folder.SetTo(base_folder.String());

		//printf("base_folder=\"%s\"\n", base_folder.String());
		BDirectory HeliosDir;
		BString bf=base_folder;
		
		//bf=*RemoveTrailingSlash(bf);
		if (HeliosDir.SetTo(bf.String())!=B_OK)
			//HeliosDir.CreateDirectory(bf.String(),&HeliosDir);
			create_directory(bf.String(), 0777);
		DeleteAllIn(BDirectory(bf.String()));
		HeliosDir.Unset();


//		sprintf(currentproj->_currentpath,"%s",base_folder.String());
//		sprintf(currentproj->_basepath,"%s",base_folder.String());
//		currentproj->_filesystem=standardCV->GetFilesystem();
//		currentproj->_cdtype=standardCV->GetCDType();
//		sprintf(currentproj->_publisher,"%s",standardCV->GetPublisher());
//		sprintf(currentproj->_preparer,"%s",standardCV->GetPreparer());
//		sprintf(currentproj->_application,"%s",standardCV->GetApplication());
//		sprintf(currentproj->_volumename,"%s",standardCV->GetVolumeName());



		project_has_changed=false;
		window1->SetTitle(H->L("TWINDOWTITLE"));
		Update_DATA_AUDIO();
		window1->Unlock();
}



void Application1::SetAsStandardProject() {
/*	standardCV->
	sprintf(settings._s_publisher,"%s",imageCV->GetPublisher());
	sprintf(settings._s_preparer,"%s",imageCV->GetPreparer());
	sprintf(settings._s_application,"%s",imageCV->GetApplication());
	sprintf(settings._s_volumename,"%s",window1->view1->GetVolumeName());
	settings._s_cdtype=window1->view1->CDType();
	settings._s_filesystem=window1->view1->GetFilesystem();
	sprintf(settings._s_currentpath,"%s%s",base_folder.String(), STANDARDPROJECTPATH);
	sprintf(settings._s_basepath,"%s%s",base_folder.String(), STANDARDPROJECTPATH);*/
}




status_t Application1::GetMultisessionSectors(int32 &start, int32 &end) {
		int arguments=0;
		thread_id Thread;
		const char **args;
		BString str="";
		BString device="dev=";
		BString endstr="";
		int in, out, err;
	
		//if (WaitForDisc(deviceCV->GetWriterDevicePath())==B_TIMEOUT) return B_TIMEOUT;
		
		device << deviceCV->GetWriterDeviceNumber();
		args = (const char **)malloc(sizeof(char *) * (22));
		args[arguments++] = strdup(CM->GetString(PATH_TO_CDRECORD)->String());
		args[arguments++] = strdup(device.String());
		args[arguments++] = strdup("-msinfo");
		args[arguments]=NULL;
		
		printArguments("cdrecord multisessioninfo", args);

		Thread=pipe_command(arguments, args, in, out, err);
		set_thread_priority(Thread,CDRECORD_PRIORITY);
		if (Thread<0) {
			errtype=H->L("TERROR");
			errstring="cdrecord\n\n";
			errstring << H->L("TCOULDNOTLOAD");
			free(args);
			HideStatus2();
			return B_ERROR;
		}
		rename_thread(Thread,CDRECORDTHREADNAME);
		resume_thread(Thread);
		
		size_t bytesread=1;
		char buffer[10000];
		BString s, startsec, endsec;
		
		end=-1;
		start=0;
		while ((bytesread>0) || (Thread==find_thread(CDRECORDTHREADNAME))) {
//		while (Thread==find_thread(CDRECORDTHREADNAME)) {
			bytesread=read(out,(void *)buffer, 10000);
			buffer[bytesread]=0;
			if (bytesread>0) {

				// get start and end sector from stdout of cdrecord
				if (strstr(buffer,",")) {
					s=buffer;
					startsec=s.Remove(s.FindFirst(","), s.Length()-s.FindFirst(","));
					start=atol(startsec.String());
					s=buffer;
					endsec=s.Remove(0,s.FindFirst(",")+1);
					end=atol(endsec.String());
					kill_thread(Thread);
				} 

			}
			snooze(10000);
		}
		close(in);close(out);close(err);

		free(args);
		if (end>=start)
		return B_OK; else
		return B_ERROR;
}

/*
void Application1::ShowExportPanel(BMessage *msg) {
	
	BEntry		*entry=new BEntry(pathCV->GetProjectPath(), true);
	entry_ref	ref;
	
	entry->GetRef(&ref);
	if (filepanel!=NULL) delete filepanel;
	filepanel=new BFilePanel(B_SAVE_PANEL, &be_app_messenger, new entry_ref(ref),
				0, false, msg);
	filepanel->SetButtonLabel(B_DEFAULT_BUTTON, H->L("C_EXPORT"));
	filepanel->SetButtonLabel(B_CANCEL_BUTTON, H->L("TCANCEL"));
	filepanel->Show();
	
	entry->Unset();
	delete entry;
}
*/


void Application1::ExportAsM3U(BEntry *entry) {

	BFile 	*file=new BFile(entry, B_READ_WRITE | B_ERASE_FILE | B_CREATE_FILE);
	M3UList *list=new M3UList();
	BString path="";
	BEntry	*musicfile=new BEntry();
	BPath	pathtomusicfile;
	AudioRow	*ai;
	BString		filenumber;
	BString		title;
	BString		time;
	int32		count=0;
	
	list->AddItem("[playlist]");
	window1->Lock();
	if (window1->view1->listview1->iCountItems()>0) {
		for (int32 index=0; index<window1->view1->listview1->iCountItems(); index++) {
			ai=(AudioRow *)window1->view1->listview1->RowAt(index);
			
			if ((!ai->IsSplitted()) || (ai->GetSplitIndex()==0)) {
				// create file number entry
				// like this: "File3"
				count++;
				filenumber="File";
				filenumber << count << "=";
				path.SetTo(ai->GetFilename()).Prepend(base_folder);
				musicfile->SetTo(path.String(),true);
				musicfile->GetPath(&pathtomusicfile);
				// generate Title entry
				title="Title=";
				title.Append(ai->GetFilename());
				title.RemoveLast(".mp3");
				title.RemoveLast(".wav");
				title.RemoveLast(".aiff");
				title.RemoveLast(".aif");
				title.RemoveLast(".WAV");
				title.RemoveLast(".MP3");
				title.RemoveLast(".AIFF");
				title.RemoveLast(".AIF");
				title.RemoveLast(".ogg");
				title.RemoveLast(".OGG");
				// generate Time-line if track is not splitted
				if (!(ai->GetSplitIndex()==0)) {
					time="Time=";
					time << (int)ai->GetLength();
					list->AddItem(new BString(time));
				}
				
				list->AddItem(new BString(title));
				filenumber.Append(pathtomusicfile.Path());
				list->AddItem(new BString(filenumber));
			}
		}
		title.SetTo("NumberOfEntries=");
		title << count;
		list->AddItem(new BString(title));
		list->SaveToFile(entry);
	}
	BNode		*node=new BNode(entry);
	BNodeInfo	*nodeinfo=new BNodeInfo(node);
	nodeinfo->SetType("text/x-playlist");
	delete nodeinfo;
	delete node;
	file->Unset();
	delete file;
	delete musicfile;
	delete list;
	window1->Unlock();
}


// APML is the APlayer Playlist file format (APlayer Module List)
void Application1::ExportAsAPML(BEntry *entry) {

	BFile 	*file=new BFile(entry, B_READ_WRITE | B_ERASE_FILE | B_CREATE_FILE);
	M3UList *list=new M3UList();
	BString path="";
	BEntry	*musicfile=new BEntry();
	BPath	pathtomusicfile;
	AudioRow	*ai;
	BString		filenumber;
	BString		title;
	BString		time;
	BString		lastpath=":";
	BDirectory	dir;
	
	list->AddItem("@*ApML*@");
	window1->Lock();
	if (window1->view1->listview1->iCountItems()>0) {
		for (int32 index=0; index<window1->view1->listview1->iCountItems(); index++) {
			ai=(AudioRow *)window1->view1->listview1->RowAt(index);
			
			// path
			path.SetTo(ai->GetFilename()).Prepend(base_folder);
			musicfile->SetTo(path.String(),true);
			musicfile->GetParent(&dir);
			dir.GetEntry(musicfile);
			musicfile->GetPath(&pathtomusicfile);
			if (lastpath.Compare(pathtomusicfile.Path())!=0) {
				list->AddItem("");
				list->AddItem("@*Path*@");
				lastpath=pathtomusicfile.Path();
				if (lastpath[lastpath.Length()-1]=='/') lastpath.RemoveLast("/");
				list->AddItem(new BString(lastpath));
				list->AddItem("");
				list->AddItem("@*Names*@");
			}
			if ((!ai->IsSplitted()) || (ai->GetSplitIndex()==0))
				list->AddItem(new BString(ai->GetFilename()));
		}
		list->SaveToFile(entry);
	}
	BNode		*node=new BNode(entry);
	BNodeInfo	*nodeinfo=new BNodeInfo(node);
	nodeinfo->SetType("text/x-aplayer-playlist");
	delete nodeinfo;
	delete node;
	file->Unset();
	delete file;
	delete musicfile;
	delete list;
	window1->Unlock();
}



void Application1::GetEntryNamesIn(BDirectory dir, M3UList *list) {

	int32 c=dir.CountEntries();
	BEntry entry;
	BPath	path;
	
	if (c>0)
	for (int32 i=0; i<c; i++) {
		dir.GetNextEntry(&entry, true);
		if (entry.IsDirectory())
			{
			GetEntryNamesIn(BDirectory(&entry), list);
			} else {
				entry.GetPath(&path);
				list->AddItem(new BString(path.Path()));
			}
	}
	path.Unset();
	entry.Unset();
	
}



void Application1::ExportAsFileList(BEntry *entry) {

	BFile 	*file=new BFile(entry, B_READ_WRITE | B_ERASE_FILE | B_CREATE_FILE);
	M3UList *list=new M3UList();
	BEntry	*musicfile=new BEntry();
	BString		lastpath=base_folder;
	BDirectory	dir;
	
	if (lastpath[lastpath.Length()-1]=='/') lastpath.RemoveLast("/");
	dir.SetTo(lastpath.String());
	GetEntryNamesIn(dir, list);

	list->SaveToFile(entry);

	BNode		*node=new BNode(entry);
	BNodeInfo	*nodeinfo=new BNodeInfo(node);
	nodeinfo->SetType("text/plain");
	file->Unset();
	delete nodeinfo;
	delete node;
	delete file;
	delete musicfile;
	delete list;
}




bool Application1::CheckCDSize() {

	int64 	maxsize=80;
	int	blocksize;
	int64	maxsize_bytes;
	
	if (heliosCV->IsOverburning())
		maxsize=90;
	switch(window1->view1->CDType()) {
		case DATACD_INDEX: {
			blocksize=2048;
			break;
		}

		case AUDIOCD_INDEX: {
			blocksize=2352;
			break;
		}

		case BOOTABLECD_INDEX: {
			blocksize=2048;
			break;
		}

		case CDEXTRA_INDEX: {
			blocksize=2352;
			break;
		}

		// no checking for DVDs
		case DVDVIDEO_INDEX: {
			return true;
		}

		default: {
			blocksize=2048;
			break;
		}
	}
		
	maxsize_bytes=(int64)((maxsize*60*176400*(int64)blocksize)/2352);
	
	return window1->view1->box2->GetCDSize()<=maxsize_bytes;
}


void Application1::ExportAsInf(const char *filename, int32 index) {

window1->Lock();
	InfoFile		*inf=new InfoFile(filename);
	AudioRow	*ai=(AudioRow *)window1->view1->listview1->RowAt(index);
	char		albumtitle[160];
	

	
	switch(window1->view1->CDType()) {
		case AUDIOCD_INDEX: {
			sprintf(albumtitle, "%s",window1->view1->GetCDAlbum());
			break;
		}
		case CDEXTRA_INDEX: {
			sprintf(albumtitle, "%s",window1->view1->GetCDAlbumCDExtra());
			break;
		}
		default: {
			break;
		}
	}
	
	inf->SetPerformer("");
	inf->SetAlbumtitle(albumtitle);
	inf->SetTracktitle(ai->GetCDTitle());
	inf->Write();
	delete inf;

	/*M3UList *list=new M3UList();
	AudioRow	*ai;
	BString		title;
	
	BFile 	*file=new BFile(entry, B_READ_WRITE | B_ERASE_FILE | B_CREATE_FILE);
	file->Unset();
	delete file;
	list->AddItem("# Audio info file. Created by Helios CDRECORDer.");
	window1->Lock();
			ai=(AudioRow *)window1->view1->listview1->RowAt(index);
				// add performer (not supported by now)
				title="Performer=\t''";
				list->AddItem(strdup(title.String()));
				// add album title
				title="Albumtitle=\t'";
				title.Append(strdup(ai->GetCDTitle()));
				title.Append("'");
				list->AddItem(strdup(title.String()));
				// add album title
				title="Tracktitle=\t'";
				title.Append(strdup(ai->GetCDTitle()));
				title.Append("'");
				list->AddItem(strdup(title.String()));
				// add track number
				title="Tracknumber=\t";
				title << index+1;
				//list->AddItem(strdup(title.String()));
			
				list->SaveToFile(entry);

	delete list;*/
	window1->Unlock();
}

// creates a filename with serial numbers which are for temporarily use only
const char *Application1::GenerateInfFilename(int32 index) {

	BString	basename;
	BString	pathtoinf;
	BString	final="";
	
	pathtoinf.SetTo(base_folder.String());

	basename.SetTo(((AudioRow *)(window1->view1->listview1->RowAt(index)))->GetFilename());
	basename.RemoveLast(".mp3");
	basename.RemoveLast(".wav");
	basename.RemoveLast(".MP3");
	basename.RemoveLast(".WAV");
	basename.RemoveLast(".AIFF");
	basename.RemoveLast(".AIF");
	basename.RemoveLast(".aiff");
	basename.RemoveLast(".aif");
	basename.RemoveLast(".ogg");
	basename.RemoveLast(".OGG");
	basename.RemoveLast(".mpeg");
	basename.RemoveLast(".MPEG");
	basename.RemoveLast(".mpeg3");
	basename.RemoveLast(".MPEG3");
	basename.RemoveLast(".mpeg1");
	basename.RemoveLast(".MPEG1");
	//if (pathtoinf[pathtoinf.Length()-1]!='/') pathtoinf.Append("/");
	AppendTrailingSlash(&pathtoinf);
	basename << ".inf";
		
	final.SetTo(pathtoinf.String());
	final.Append(basename.String());
	printf("HELIOS: >%s<\n",final.String());
	return strdup(final.String());
}

// writes info files for all tracks kept by the current project
void Application1::SaveCDTEXTAlbum() {
	
	window1->Lock();
	for (int32 index=0; index<window1->view1->listview1->iCountItems();index++) {
		// call export routine
		ExportAsInf(GenerateInfFilename(index), index);
	}	
	window1->Unlock();	
}


// remove the temporary files (info files for audio-cd burning)
// located in the temp-path of helios
void Application1::RemoveInfFiles() {
	
	BEntry	*entry=new BEntry();
	
	window1->Lock();
	for (int32 index=0; index<window1->view1->listview1->iCountItems();index++) {
		entry->SetTo(GenerateInfFilename(index),true);
		if (entry->Exists())
			entry->Remove();
		entry->Unset();
	}	
	delete entry;
	window1->Unlock();	
}

BString SubString(BString *string, int32 first, int32 last) {
	BString temp;
	
	temp.SetTo(string->String());
	temp.Remove(last,temp.Length()-last);
	temp.Remove(0, first);
	return temp;
}

void Application1::PopulateFoldersMenu() {
	BString current=current_folder;
	current.RemoveLast("/");
	BitmapMenuItem	*mi;
	BMessage	*msg;
	BEntry		entry;
	entry_ref	ref;
	
	window1->Lock();
	window1->view1->listview1->statusView->RemoveMenuItems();
	if (current.Length()>base_folder.Length()) 
		current.Remove(current.FindLast("/"), current.Length()-current.FindLast("/"));

	while (current.Length()>base_folder.Length()) {
		msg=new BMessage(FOLDERMENU_MSG);
		msg->AddString("folder", current);
		if (entry.SetTo(current.String(), true)==B_OK) {
			entry.GetRef(&ref);
			entry.Unset();
		}
		mi=new BitmapMenuItem(SubString(&current, current.FindLast("/")+1, current.Length()).String(), GetIcon(&ref), msg);
		window1->view1->listview1->statusView->AddMenuItem(mi);
		current.Remove(current.FindLast("/"), current.Length()-current.FindLast("/"));
	}
	current=current_folder;
	if (current.Length()>base_folder.Length()) {
		BString t="";
		if (window1->view1->CDType()==DATACD_INDEX) t=window1->view1->GetVolumeName();
		if (window1->view1->CDType()==BOOTABLECD_INDEX) t=window1->view1->GetVolumeNameBootCD();
		msg=new BMessage(FOLDERMENU_MSG);
		msg->AddString("folder", base_folder);
		entry.SetTo(base_folder.String(), true);
		entry.GetRef(&ref);
		entry.Unset();
		mi=new BitmapMenuItem(t.String(), GetIcon(&ref),  msg);
		window1->view1->listview1->statusView->AddMenuItem(mi);
	}
	window1->Unlock();
}

// "path" is a devfs-path
size_t Application1::GetDeviceSize(const char *path) {
	int 	fd=open(path, O_RDONLY);
	size_t	data;
	
	ioctl(fd, B_GET_DEVICE_SIZE, &data);
	close(fd);
	return data;
}

/*

If you like to append a filesystem to a audio CD, first extract the 
CD by calling:

	cdda2wav -B -v255

This first audio session may be written in TAO mode with the command

	cdrecord -multi -audio file1 ....

or in DAO mode with the command

	cdrecord -dao -multi -useinfo -audio file1 ....

To add the second session that contains the data track, you need
to create an ISO-9660 file system that starts not at sector 0.

With the current mkisofs (1.12), you must use the following method:


-	First call cdrecord -msinfo for your prepared multi-session audio CD.

	you will get something like 0,12345

-	Now call mkisofs:

	mkisofs -o fs.raw -C 0,12345 root_directory_for_new_cd

	replace 0,12345 with your actual cdrecord -msinfo output.

the image in fs.raw may now be written with cdrecord as second session.

See my README.multi for more info on how to create multi session CD's


The procedure again in short form:

	cdda2wav -v255 -B

	cdrecord -multi -audio audio_tracks ....
or
	cdrecord -dao -useinfo -multi -audio audio_tracks ....

	cdrecord -msinfo ....
	(output is e.g. 0,12345)

	mkisofs -R -o cd_plus.raw -C 0,12345 root_dir_of_fs

	cdrecord -data cd_plus.raw
or
	cdrecord -multi cd_plus.raw


Note: If you want to create an HFS hybrid as the data track, then you must
use the '-part' option to mkisofs. Otherwise, the data track will be mounted
as an ISO9660/Joliet CD when used on a Mac.
*/


status_t Application1::BurnCDExtra() {
	
	if (BurnAudio(true)==B_OK) {
		//if (GetMultisessionSectors(start, end)==B_OK) {
			load_media(deviceCV->GetWriterDevicePath());

			BString	bf=base_folder;
			BString tp;
		
			base_folder=window1->view1->GetImageNameCDExtra();
			AppendTrailingSlash(&base_folder);

			if ((!saveCV->IsStreamed())
					//||
				/* bfs? *//* (window1->view1->GetFilesystem()==2)*/)
				MakeImage_NOSTREAMING();
			else
				if (saveCV->IsStreamed())
					MakeImage();
					
			base_folder=bf;
		//}
	}
	return B_OK;
}

void Application1::DeleteAndCount(BDirectory dir) {

	int32 c=dir.CountEntries();
	BEntry entry;
	
	if (c>0)
	for (int32 i=0; i<c; i++) {
		dir.GetNextEntry(&entry, false);
		if (entry.IsDirectory()) {
			DeleteAndCount(BDirectory(&entry));
			entry.Remove();
		}
		else {
			entry.Remove();
		}

		
		if (filecounter % 20==0) {
			if (window1->Lock()) {
				char	text[100];
				
				sprintf(text, "%s (%ld)", H->L("L:Removing files..."), filecounter);
				window1->view1->statusBAR->SetText(text);
				window1->Unlock();
			}
		}
		filecounter++;
	/*	if (statusSW->Lock()) {
			statusSW->SetFileCounter(statusSW->GetFiles()+1);
			statusSW->Unlock();
		}*/
	}
}


void Application1::PlayFinishedBurningSound() {

	if (soundCV->HasFinishedWritingSound()) {
		if (system_beep(WRITTEN_BEEP_EVENT)!=B_OK) {
			if (add_system_beep_event(WRITTEN_BEEP_EVENT)==B_OK) {
				system_beep(WRITTEN_BEEP_EVENT);
			} else {
				ErrorBox *eb=new ErrorBox(E_RED_COLOR, H->L("TERROR"), H->L("TCOULDNOTADDSOUNDEVENT"), H->L("TOK"));
				eb->Go();
			}
		}
	}
}


void Application1::PlayFinishedImageSound() {

	if (soundCV->HasFinishedImageSound()) {
		if (system_beep(IMAGE_BEEP_EVENT)!=B_OK) {
			if (add_system_beep_event(IMAGE_BEEP_EVENT)==B_OK) {
				system_beep(IMAGE_BEEP_EVENT);
			} else {
				ErrorBox *eb=new ErrorBox(E_RED_COLOR, H->L("TERROR"), H->L("TCOULDNOTADDSOUNDEVENT"), H->L("TOK"));
				eb->Go();
			}
		}
	}
}



void Application1::PlayErrorSound() {

	if (soundCV->HasErrorSound()) {
		if (system_beep(ERROR_BEEP_EVENT)!=B_OK) {
			if (add_system_beep_event(ERROR_BEEP_EVENT)==B_OK) {
				system_beep(ERROR_BEEP_EVENT);
			} else {
				ErrorBox *eb=new ErrorBox(E_RED_COLOR, H->L("TERROR"), H->L("TCOULDNOTADDSOUNDEVENT"), H->L("TOK"));
				eb->Go();
			}
		}
	}
}


// optimized for 1440k-floppy disks
status_t	Application1::ReadFloppyImage(const char *path) {
	BFile	*src, *dst;
	ssize_t	numread;
	char	*buffer=new char[36*512];
	char	info[20];
	int	buffersize=18*512;
	const int sectorsize=512;
	float	progressed=0, sum=0;
	bool	retval=true;
	
	src=new BFile("/dev/disk/floppy/raw", B_READ_ONLY);
	dst=new BFile(path, B_READ_WRITE | B_CREATE_FILE);
	
	BStopWatch *watch=new BStopWatch("kBps",true);
	ShowStatus2(H->L("Status:Reading Image File"), -1);
	SetStatus2Info("--- kB/s");

	if ((src->InitCheck()==B_OK) && (dst->InitCheck()==B_OK)) {
		dst->SetSize(0);
		
		numread=src->Read(buffer, buffersize);
		while ((dst->Write(buffer, numread)==numread) && (buffersize>=sectorsize)) {
			numread=src->Read(buffer, buffersize);
			if ((numread<=0) && (buffersize>sectorsize)) {
				buffersize/=2;
				printf("HELIOS: Reducing Buffer Size To %d Bytes.\n", buffersize);
			}
			if (buffersize<=sectorsize) buffersize=0;
			if (window5->interrupted) {
				retval=false;
				break;
			}

			progressed+=numread;
			if (progressed>18*512) {
					sprintf(info,"%0.1f kB/s",((float)progressed)/(((float)watch->ElapsedTime())*0.001024F));
					SetStatus2Info(info);
					watch->Reset();
					sum+=progressed;
					SetPercentage(100*sum/(1440*1024));
					progressed=0.0F;
			}

		}
		dst->Sync();
		src->Unset();
		dst->Unset();
	} else retval=false;
	HideStatus2();
	delete src;
	delete dst;
	delete [] buffer;
	delete watch;
	return B_OK;
}

















